@@ -28,8 +28,8 @@ Boston, MA 02111-1307, USA.  */
 
 cmd_bus c_bus;
 
-extern void dump_bus_list(void);
-extern void add_bus(char *node_name);
+//extern void dump_bus_list(void);
+//extern void add_bus(char *node_name);
 
 static cmd_options cmd_bus_options[] =
 {
@@ -37,10 +37,9 @@ static cmd_options cmd_bus_options[] =
 };
 
 
-cmd_bus::cmd_bus(void)
+cmd_bus::cmd_bus()
+  : command("bus",0)
 { 
-  name = "bus";
-
   brief_doc = string("Add or display node busses");
 
   long_doc = string ("bus [new_bus1 new_bus2 ...]\n"
@@ -60,14 +59,13 @@ cmd_bus::cmd_bus(void)
 
 void cmd_bus::list_busses(void)
 {
-
-  dump_bus_list();
+  //  dump_bus_list();
 }
 
 
 void cmd_bus::add_busses(list <string> * busses)
 {
-
+  /*
   if(busses) {
 
     list <string> :: iterator si;
@@ -81,4 +79,5 @@ void cmd_bus::add_busses(list <string> * busses)
     }
 
   }
+  */
 }
