@@ -19,12 +19,11 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include <string>
 
 #include "command.h"
-#include "parse.h"
 
 #include "cmd_quit.h"
 
@@ -32,19 +31,18 @@ cmd_quit quit;
 
 static struct cmd_options cmd_quit_options[] = 
 {
-  NULL,0,0
+  {0,0,0}
 };
 
 
-cmd_quit::cmd_quit(void) 
+cmd_quit::cmd_quit() 
+  : command("quit",0)
 { 
-  name = "quit";
 
   brief_doc = string("Quit gpsim");
 
   long_doc = string ("Quit gpsim\n");
 
-  token_value = QUIT;
   op = cmd_quit_options;
 }
 
