@@ -25,17 +25,18 @@ Boston, MA 02111-1307, USA.  */
 
 extern int gpsim_read (char *buf, unsigned max_size);
 extern void initialize_readline (void);
-extern void exit_gpsim(void);
-void SetLastFullCommand(const char *pCmd);
-const char * GetLastFullCommand();
-void EnableSTCEcho(bool bEnable);
+extern void exit_cli(void);
+extern void exit_gpsim(int);
+extern void SetLastFullCommand(const char *pCmd);
+extern const char * GetLastFullCommand();
+extern void EnableSTCEcho(bool bEnable);
 
 #ifndef HAVE_GUI
 extern char *gnu_readline (char *s, unsigned int force_readline);
 #endif
 
 class Processor;
-int gpsim_open(Processor *cpu, const char *file, 
-	       const char *pProcessorType, const char *pProcessorName);
+extern int gpsim_open(Processor *cpu, const char *file,
+                      const char *pProcessorType, const char *pProcessorName);
 
 #endif
