@@ -54,7 +54,7 @@ extern int gui_init (int argc, char **argv);
 extern void gui_main(void);
 extern void cli_main(void);
 // os_dependent.cc'
-extern void AddModulePathFromFilePath(string &sFolder);
+extern void AddModulePathFromFilePath(char *arg);
 
 void initialize_gpsim();
 
@@ -207,7 +207,6 @@ main (int argc, char *argv[])
 
   // Perform basic initialization before parsing invocation arguments
 
-  optCon = poptGetContext(0, argc, (const char **)argv, optionsTable, 0);
 
   welcome();
 
@@ -217,8 +216,14 @@ main (int argc, char *argv[])
   initialize_gpsim();
   initialize_commands();
 
+  optCon = poptGetContext(0, argc, (const char **)argv, optionsTable, 0);
   if(argc>=2) {
     while ((c = poptGetNextOpt(optCon)) >= 0  && !usage) {
+
+        const char * optArg = poptGetOptArg(optCon);
+#ifndef _WIN32
+        free((char *)optArg);
+#endif
       switch (c) {
 
       default:
@@ -230,6 +235,9 @@ main (int argc, char *argv[])
 
       case 'L':
         set_search_path (search_path);
+#ifndef _WIN32
+        free((char *)search_path);
+#endif
         break;
 
       case 'd':
@@ -250,6 +258,9 @@ main (int argc, char *argv[])
         snprintf(command_str, sizeof(command_str),
                  "symbol %s\n",defineSymbol);
         parse_string(command_str);
+#ifndef _WIN32
+        free((char *)defineSymbol);
+#endif
         defineSymbol = "";
         break;
 
@@ -263,6 +274,9 @@ main (int argc, char *argv[])
         else {
           usage = 1;
         }
+#ifndef _WIN32
+        free ((char *)sourceEnabled);
+#endif
         break;
 
       case 'E':
@@ -277,6 +291,9 @@ main (int argc, char *argv[])
         else {
           printf("%s is invalid exit condition for -e option.\n", sExitOn);
         }
+#ifndef _WIN32
+        free ((char *)sExitOn);
+#endif
         break;
       }
 
@@ -284,6 +301,7 @@ main (int argc, char *argv[])
         break;
 
     }
+    poptFreeContext(optCon);
   }
 
   if (usage) {
@@ -300,7 +318,6 @@ main (int argc, char *argv[])
   if(poptPeekArg(optCon))
     hex_name=strdup(poptPeekArg(optCon));
 
-  poptFreeContext(optCon);
 
   initialize_readline();
 
@@ -327,8 +344,7 @@ main (int argc, char *argv[])
 #endif
 
 
-  string sGpsimPath(argv[0]);
-  AddModulePathFromFilePath(sGpsimPath);
+  AddModulePathFromFilePath(argv[0]);
 
   initialization_is_complete();
 
@@ -357,7 +373,7 @@ main (int argc, char *argv[])
 
     if(*hex_name){
       snprintf(command_str, sizeof(command_str),
-        "load \%s \"%s\"\n",processor_name, hex_name);
+        "load %s \"%s\"\n",processor_name, hex_name);
       parse_string(command_str);
 
     }
@@ -412,7 +428,6 @@ main (int argc, char *argv[])
         exit_gpsim(1);
       }
     }
-
   exit_gpsim(0);
   return 0;
 }
