@@ -26,6 +26,7 @@ Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_GUI
 
+#include <cstdio>
 #include <unistd.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -101,6 +102,11 @@ public:
   enum window_types parent_window_type;
   gpointer     parent_window;
   gpointer     data;
+
+  virtual ~CrossReferenceToGUI()
+  {
+  }
+
   virtual void Update(int new_value);
   virtual void Remove(void);
 };
@@ -114,6 +120,11 @@ class EntryWidget
 {
 public:
   EntryWidget();
+
+  virtual ~EntryWidget()
+  {
+  }
+
   virtual void Update()=0;
   virtual void Create(bool isEditable=true);
   void SetEntryWidth(int string_width);
@@ -135,6 +146,11 @@ public:
   GtkWidget *label;
 
   LabeledEntry(void);
+
+  virtual ~LabeledEntry()
+  {
+  }
+
   void Create(GtkWidget *box,char *clabel, int string_width, bool isEditable);
   void NewLabel(char *clabel);
   virtual void Update(void);
@@ -150,6 +166,10 @@ public:
 
   RegisterLabeledEntry(GtkWidget *,Register *,bool);
 
+  virtual ~RegisterLabeledEntry()
+  {
+  }
+
   virtual void put_value(unsigned int);
   void AssignRegister(Register *new_reg);
   virtual void Update(void);
@@ -160,6 +180,11 @@ class CyclesLabeledEntry : public LabeledEntry {
 public:
 
   CyclesLabeledEntry();
+
+  virtual ~CyclesLabeledEntry()
+  {
+  }
+
   virtual void Update(void);
 };
 
@@ -172,14 +197,19 @@ typedef enum {
 } time_menu_id;
 
 typedef struct _time_menu_item {
-  char *name;
+  const char *name;
   time_menu_id id;
 } time_menu_item;
 
-class TimeLabeledEntry : public LabeledEntry 
+class TimeLabeledEntry : public LabeledEntry
 {
 public:
   TimeLabeledEntry();
+
+  virtual ~TimeLabeledEntry()
+  {
+  }
+
   virtual void Update(void);
   GtkWidget *build_menu();
 
@@ -194,12 +224,16 @@ public:
 };
 
 //------------------------------------------------------------------------
-// The KeyEvent class is used for key mapping (e.g. mapping the 's' 
+// The KeyEvent class is used for key mapping (e.g. mapping the 's'
 // key to step)
 
 class KeyEvent
 {
 public:
+  virtual ~KeyEvent()
+  {
+  }
+
   virtual void press(gpointer data = 0) = 0;
   virtual void release(gpointer data = 0) = 0;
 };
@@ -233,7 +267,7 @@ gint gui_object_configure_event(GtkWidget *widget, GdkEventConfigure *e, GUI_Obj
 
 void ProfileWindow_notify_start_callback(Profile_Window *pw);
 void ProfileWindow_notify_stop_callback(Profile_Window *pw);
-int gui_get_value(char *prompt);
+int gui_get_value(const char *prompt);
 
 
 //------------------------------------------------------------------------
