@@ -118,7 +118,7 @@ about_cb (gpointer             callback_data,
   show_message(  "The GNUPIC Simulator - " VERSION, "A simulator for Microchip PIC microcontrollers.\n"
                  "by T. Scott Dattalo - mailto:scott@dattalo.com\n"
                  "   Ralf Forsberg - mailto:rfg@home.se\n"
-                 "   Borut Ra" "\xc5\xbe" "em - mailto:borut.razem@siol.net\n\n"
+                 "   Borut Ra" "\xc5\xbe" "em - mailto:borut.razem@gmail.com\n\n"
                  "gpsim homepage: http://gpsim.sourceforge.net/gpsim.html\n"
                  "gpsimWin32: http://gpsim.sourceforge.net/gpsimWin32/gpsimWin32.html\n");
 
@@ -791,7 +791,7 @@ gpsimGuiPreferences::~gpsimGuiPreferences()
 
   gtk_widget_destroy (window);
 
-  //delete m_SourceBrowser;
+  delete m_SourceBrowser;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
