@@ -1798,7 +1798,7 @@ float calculate_stddev(GList *start, GList *stop, float average)
     float variance;
     int count=0;
     float sum=0;
-    struct cycle_histogram_counter *chc_start, *chc_stop;
+    struct cycle_histogram_counter *chc_start;
 
     if(start==stop)
         return 0.0;
@@ -1815,7 +1815,6 @@ float calculate_stddev(GList *start, GList *stop, float average)
         float diff, diff2;
 
         chc_start=(struct cycle_histogram_counter*)start->data;
-        chc_stop=(struct cycle_histogram_counter*)stop->data;
 
         diff=chc_start->histo_cycles-average;
 
@@ -2377,7 +2376,6 @@ void Profile_Window::Build(void)
   GtkWidget *main_vbox;
   GtkWidget *scrolled_window;
 
-  gint column_width,char_width;
   window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                      GTK_SIGNAL_FUNC(delete_event), this);
@@ -2397,7 +2395,13 @@ void Profile_Window::Build(void)
 
   // Instruction profile clist
   profile_clist=GTK_CLIST(gtk_clist_new_with_titles(PROFILE_COLUMNS, (gchar **)profile_titles));
-  //profile_clist = GTK_CLIST(profile_clist);
+
+
+//  The following line is silly but it stops GTK printing an assert
+//  in Fedora 19 with Adwaita theme
+  gtk_widget_set_style(GTK_WIDGET(profile_clist), 
+	gtk_widget_get_style(GTK_WIDGET(profile_clist)));
+
   gtk_clist_set_column_auto_resize(GTK_CLIST(profile_clist),0,TRUE);
   gtk_clist_set_column_auto_resize(GTK_CLIST(profile_clist),1,TRUE);
 //  gtk_clist_set_sort_column (pw->profile_clist,1);
@@ -2424,6 +2428,13 @@ void Profile_Window::Build(void)
 
   // Instruction range profile clist
   profile_range_clist=GTK_CLIST(gtk_clist_new_with_titles(PROFILE_RANGE_COLUMNS,(gchar **)profile_range_titles));
+
+
+//  The following line is silly but it stops GTK printing an assert
+//  in Fedora 19 with Adwaita theme
+  gtk_widget_set_style(GTK_WIDGET(profile_range_clist), 
+	gtk_widget_get_style(GTK_WIDGET(profile_range_clist)));
+
   gtk_clist_set_column_auto_resize(profile_range_clist,0,TRUE);
   gtk_clist_set_column_auto_resize(profile_range_clist,1,TRUE);
   gtk_clist_set_sort_column (profile_range_clist,2);
@@ -2461,6 +2472,12 @@ void Profile_Window::Build(void)
 
   // Register profile clist
   profile_register_clist=GTK_CLIST(gtk_clist_new_with_titles(PROFILE_REGISTER_COLUMNS, (gchar **)profile_register_titles));
+
+//  The following line is silly but it stops GTK printing an assert
+//  in Fedora 19 with Adwaita theme
+  gtk_widget_set_style(GTK_WIDGET(profile_register_clist), 
+	gtk_widget_get_style(GTK_WIDGET(profile_register_clist)));
+
   gtk_clist_set_column_auto_resize(profile_register_clist,0,TRUE);
   gtk_clist_set_column_auto_resize(profile_register_clist,1,TRUE);
   gtk_clist_set_column_auto_resize(profile_register_clist,2,TRUE);
@@ -2489,6 +2506,13 @@ void Profile_Window::Build(void)
 
   // Execution time statistics tab
   profile_exestats_clist=GTK_CLIST(gtk_clist_new_with_titles(PROFILE_EXESTATS_COLUMNS, (gchar **)profile_exestats_titles));
+
+//  The following line is silly but it stops GTK printing an assert
+//  in Fedora 19 with Adwaita theme
+  gtk_widget_set_style(GTK_WIDGET(profile_exestats_clist), 
+	gtk_widget_get_style(GTK_WIDGET(profile_exestats_clist)));
+
+
   gtk_clist_set_column_auto_resize(profile_exestats_clist,0,TRUE);
   gtk_clist_set_column_auto_resize(profile_exestats_clist,1,TRUE);
   gtk_clist_set_column_auto_resize(profile_exestats_clist,2,TRUE);
@@ -2525,12 +2549,6 @@ void Profile_Window::Build(void)
   gtk_window_set_wmclass(GTK_WINDOW(window),name(),"Gpsim");
 
   normal_style = gtk_style_new ();
-#if GTK_MAJOR_VERSION >= 2
-  char_width = gdk_string_width(gtk_style_get_font(normal_style) ,"9");
-#else
-  char_width = gdk_string_width (normal_style->font,"9");
-#endif
-  column_width = 3 * char_width + 6;
 
   gtk_signal_connect_after(GTK_OBJECT(window), "configure_event",
                            GTK_SIGNAL_FUNC(gui_object_configure_event),this);
