@@ -175,12 +175,18 @@ void GUIRegister::put_value(unsigned int new_value)
   Register *reg = get_register();
 
   if(reg)
+  {
     reg->put_value(new_value);
 
   // Shadow a copy of the register value so that we can tell if it has changed
   // when we go to perform an update in the future.
 
-  shadow = reg->getRV_notrace();
+    shadow = reg->getRV_notrace();
+  }
+  else
+  {
+	fprintf(stderr, "RRR %s %d reg is 0 new_value=%x\n", __FUNCTION__, __LINE__, new_value);
+  }
 }
 
 void GUIRegister::put_shadow(RegisterValue new_value)
@@ -205,7 +211,9 @@ RegisterValue GUIRegister::getRV()
   Register *reg = get_register();
 
   if(reg)
+  {
     return reg->getRV_notrace();
+  }
 
   return RegisterValue(0,0);
 }
@@ -323,7 +331,7 @@ bool GUIRegister::bIsSFR()
   return false;
 }
 
-GUIRegister:: GUIRegister()
+GUIRegister::GUIRegister()
 {
   rma = 0;
   xref = 0;
@@ -1055,7 +1063,6 @@ do_popup(GtkWidget *widget, GdkEventButton *event, Register_Window *rw)
 
   GtkWidget *popup;
   //    GdkModifierType mods;
-  GtkSheet *sheet;
 
   popup=rw->popup_menu;
 
@@ -1065,7 +1072,7 @@ do_popup(GtkWidget *widget, GdkEventButton *event, Register_Window *rw)
       return 0;
     }
 
-  sheet=GTK_SHEET(widget);
+  GTK_SHEET(widget);
 
   if( (event->type == GDK_BUTTON_PRESS) &&  (event->button == 3) )
     {
@@ -1544,6 +1551,11 @@ move_handler(GtkWidget *widget,
   if(!widget || !old_range || !new_range || !rw)
       return;
 
+  if (old_range->row0 < 0 || old_range->col0 < 0 ||
+	new_range->row0 < 0 || new_range->col0)
+      return;
+
+
   cti = new_range->coli - new_range->col0 + 1;
   ctj = new_range->rowi - new_range->row0 + 1;
 
@@ -1838,12 +1850,14 @@ gboolean Register_Window::UpdateRegisterCell(unsigned int reg_number)
 
   GUIRegister *guiReg = registers->Get(reg_number);
 
+
   if (!guiReg || !guiReg->rma)
     return 0;
 
   if(reg_number >= guiReg->rma->get_size())
     return 0;
 
+  
   range.row0=guiReg->row;
   range.rowi=guiReg->row;
   range.col0=guiReg->col;
@@ -1856,6 +1870,7 @@ gboolean Register_Window::UpdateRegisterCell(unsigned int reg_number)
 
   RegisterValue last_value=guiReg->get_shadow();
 
+
   if(bTrace)
     printf("UpdateRegisterCell() Entry: regID=%3d, Full=%s, hasChanged=%s\n",
       reg_number, guiReg->bUpdateFull ? "true " : "false",\
@@ -1879,7 +1894,7 @@ gboolean Register_Window::UpdateRegisterCell(unsigned int reg_number)
     // else the register is invalid and out of the register sheet
 
 
-    //if(new_value != last_value) {
+    //if(new_value != last_value) 
     if(guiReg->hasChanged(new_value)) {
 
       guiReg->put_shadow(new_value);
@@ -2052,6 +2067,7 @@ GUIRegisterList::GUIRegisterList(RegisterMemoryAccess *pRMA) {
 
   uRegisterSize = (pRMA->get_size() < MAX_REGISTERS) ? (pRMA->get_size()) : MAX_REGISTERS;
 
+
   for(uAddress=0; uAddress < uRegisterSize; uAddress++) {
     GUIRegister *pReg = new GUIRegister();
     pReg->rma = m_pRMA;
