@@ -412,7 +412,7 @@ class SourcePage
   GtkAdjustment *source_layout_adj;
   GtkWidget *source_layout;
   GtkWidget *source_text;
-  unsigned int pageindex_to_fileid;
+  int 	    pageindex_to_fileid;
   GtkWidget *source_pcwidget;
   GtkWidget *notebook;   // parent
   GtkWidget *notebook_child;
@@ -563,6 +563,16 @@ class SourceBrowserOpcode_Window : public SourceBrowser_Window
 
   // Font strings
   char normalfont_string[256];
+#if GTK_MAJOR_VERSION >= 2
+  PangoFontDescription *normalPFD;
+  PangoFontDescription *current_line_numberPFD;
+  PangoFontDescription *breakpoint_line_numberPFD;
+#else
+  GdkFont *normalfont;
+  GdkFont *current_line_numberfont;
+  GdkFont *breakpoint_line_numberfont;
+#endif
+
   char breakpointfont_string[256];
   char pcfont_string[256];
   GtkStyle *normal_style;
@@ -572,7 +582,7 @@ class SourceBrowserOpcode_Window : public SourceBrowser_Window
   GdkColor normal_pm_bg_color;
   GdkColor breakpoint_color;
 
-  gchar **column_titles; //
+  const gchar **column_titles; //
   int  columns;         //
 
   GtkWidget *notebook;
