@@ -204,10 +204,8 @@ void Watch_Window::WriteSymbolList()
   WatchEntry *entry;
   guint uSize = g_list_length(watches);
   char cwv[100];
-  char *vname;
   for (guint i = 0; i<uSize; i++) {
     snprintf(cwv, sizeof(cwv), "WV%d",i);
-    vname = 0;
     entry = (WatchEntry*) g_list_nth_data(watches, i);
     if (entry && entry->pRegister)
       config_set_string(name(), cwv, entry->pRegister->name().c_str());
@@ -573,7 +571,7 @@ void Watch_Window::UpdateWatch(WatchEntry *entry)
 
   if(entry->pRegister) {
     rvMaskedNewValue = entry->pRegister->getRV_notrace();
-    uBitmask = entry->pRegister->bit_mask; //getBitmask();
+    uBitmask = entry->pRegister->mValidBits; //getBitmask();
   }
   else {
     rvMaskedNewValue = entry->getRV();
@@ -595,8 +593,8 @@ void Watch_Window::UpdateWatch(WatchEntry *entry)
   gtk_clist_set_text(GTK_CLIST(watch_clist), row, HEXCOL, str);
 
   // ASCII representation
-  str[0] = (rvNewValue.data>'0' && rvNewValue.data<='z') ? rvNewValue.data : 0;
-  str[1] =0;
+  str[0] = ( rvNewValue.data>0x20 && rvNewValue.data<0x7F ) ? rvNewValue.data : 0;
+  str[1] = 0;
   gtk_clist_set_text(GTK_CLIST(watch_clist), row, ASCIICOL, str);
 
   // Bit representation
@@ -804,6 +802,10 @@ void Watch_Window::Build(void)
                            GTK_SIGNAL_FUNC(gui_object_configure_event),this);
 
   watch_clist = gtk_clist_new_with_titles(COLUMNS,(gchar **)watch_titles);
+
+//  The following line is silly but it stops GTK printing an assert
+//  in Fedora 19 with Adwaita theme
+  gtk_widget_set_style(watch_clist, gtk_widget_get_style(watch_clist));
   gtk_widget_show(watch_clist);
 
   for(i=0;i<LASTCOL;i++) {
