@@ -1,22 +1,22 @@
 /*
    Copyright (C) 2000 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim_modules library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 /*
   Logic.cc
@@ -39,7 +39,7 @@ In here you'll find some simple logic devices:
 #include <gtk/gtk.h>
 
 /* XPM */
-static gchar * and2_pixmap[] = {
+static const gchar * and2_pixmap[] = {
 "32 32 3 1",
 "       c black",
 ".      c None",
@@ -78,7 +78,7 @@ static gchar * and2_pixmap[] = {
 "                    ............"};
 
 /* XPM */
-static gchar * or2_pixmap[] = {
+static const gchar * or2_pixmap[] = {
 "32 32 3 1",
 "       c black",
 ".      c None",
@@ -116,7 +116,7 @@ static gchar * or2_pixmap[] = {
 "                        ........",
 "                    ............"};
 
-static gchar * xor2_pixmap[] = {
+static const gchar * xor2_pixmap[] = {
 "40 32 3 1",
 "       c None",
 ".      c black",
@@ -154,7 +154,7 @@ static gchar * xor2_pixmap[] = {
 " ....   ..XXXXXXXXXXXXXXXXXX....        ",
 "  ..    ........................        "};
 
-static gchar * not_pixmap[] = {
+static const gchar * not_pixmap[] = {
 "32 32 3 1",
 "       c black",
 ".      c None",
@@ -213,7 +213,7 @@ static gchar * not_pixmap[] = {
 void Logic_Input::setDrivenState( bool new_state)
 {
 
-  if(1)
+  if(verbose)
     cout << name()<< " setDrivenState= "
          << (new_state ? "high" : "low") << endl;
 
@@ -426,7 +426,7 @@ AND2Gate::AND2Gate(const char *name) : ANDGate(name, "And2Gate")
 {
 #ifdef HAVE_GUI
   if(get_interface().bUsingGUI())
-    set_widget(create_pixmap(and2_pixmap));
+    set_widget(create_pixmap((gchar **)and2_pixmap));
 #endif
 
 }
@@ -451,7 +451,7 @@ OR2Gate::OR2Gate(const char *name) : ORGate(name, "OR2Gate")
 #ifdef HAVE_GUI
 
   if(get_interface().bUsingGUI())
-    set_widget(create_pixmap(or2_pixmap));
+    set_widget(create_pixmap((gchar **)or2_pixmap));
 #endif
 }
 OR2Gate::~OR2Gate()
@@ -499,7 +499,7 @@ NOTGate::NOTGate(const char *name) : LogicGate(name, "NOTGate")
 #ifdef HAVE_GUI
 
   if(get_interface().bUsingGUI())
-    set_widget(create_pixmap(not_pixmap));
+    set_widget(create_pixmap((gchar **)not_pixmap));
 #endif
 }
 NOTGate::~NOTGate()
@@ -509,7 +509,8 @@ NOTGate::~NOTGate()
 
 void NOTGate::update_state()
 {
-  cout << name() << " update_state\n";
+  if (verbose)
+    cout << name() << " update_state\n";
   pOutputPin->putState((input_state & input_bit_mask) == 0);
 }
 
@@ -522,7 +523,7 @@ XOR2Gate::XOR2Gate(const char *name) : XORGate(name, "XOR2Gate")
 #ifdef HAVE_GUI
 
   if(get_interface().bUsingGUI())
-    set_widget(create_pixmap(xor2_pixmap));
+    set_widget(create_pixmap((gchar **)xor2_pixmap));
 #endif
 }
 XOR2Gate::~XOR2Gate()
