@@ -16,7 +16,7 @@ node_test p16f84 p18f452_ports p16f628 digital_stim analog_stim p12ce518 eeprom_
 interrupts_14bit macro_test logic_test resistor usart_test txisr_test 
 tmr0_16bit tmr1_16bit 
 switch_test p18f comparator a2d psp ttl ccp ccp_628 wavegen spi i2c port_stim 
-p12c509 wdt p12f675 p16f676 p16f690'
+p12c509 wdt p12f675 p16f676 p16f690 p16f684 p1xf18xx'
 
         echo ${DIRS}
         for i in ${DIRS} ; do
@@ -96,6 +96,8 @@ ${RT} usart_test sim_pir1v2
 
 ${RT} usart_test sim_eusart
 
+${RT} usart_test sim_eusart_2455
+
 ${RT} txisr_test sim
 
 ${RT} tmr0_16bit sim
@@ -140,6 +142,8 @@ ${RT} a2d sim_p10f222
 
 ${RT} psp sim_p18f452
 
+${RT} psp sim_p18f6520
+
 ${RT} psp sim_p16f871
 
 ${RT} ttl sim_ttl377
@@ -193,7 +197,28 @@ ${RT} wdt sim_nwdt_16c64
 
 ${RT} wdt sim_wdt_18f452
 ${RT} wdt sim_nwdt_18f452
+${RT} wdt sim_wdt_18f4620
+${RT} wdt sim_nwdt_18f4620
 
 ${RT} p16f676 sim_p16f676
 ${RT} p16f676 sim_reset
-${RT} p16f690 sim
+${RT} p16f690 sim_epwm 
+${RT} p16f690 sim_eusart 
+${RT} p16f690 sim_nwdt_16f631
+${RT} p16f690 sim_p16f690
+${RT} p16f690 sim_wdt_16f677
+${RT} p16f690 sim_wdt_16f685
+
+${RT}       p16f684 sim_a2d_684
+${RT}       p16f684 sim_compar_684
+${RT}       p16f684 sim_epwm
+${RT}       p16f684 sim_nwdt_16f684
+${RT}       p16f684 sim_reset
+${RT}       p16f684 sim_wdt_16f684
+
+${RT}       p1xf18xx sim_p12f1822
+${RT}       p1xf18xx sim_p16f1823
+${RT}       p1xf18xx sim_p16f1823_comp
+${RT}       p1xf18xx sim_p16f1823_i2c
+${RT}       p1xf18xx sim_p16f1823_i2c_v2
+${RT}       p1xf18xx sim_p16f1823_spi
