@@ -116,7 +116,7 @@ _12bit_processor::_12bit_processor(const char *_name, const char *desc)
   option_reg->set_write_trace(rv);
   option_reg->set_read_trace(rv);
 
-  stack = new Stack();
+  stack = new Stack(this);
 }
 
 _12bit_processor::~_12bit_processor()
@@ -135,7 +135,7 @@ void _12bit_processor::create_symbols()
   addSymbol(option_reg);
   // Create an alias for the option_reg
   option_reg->new_name("option");
-  addSymbol(W);
+  addSymbol(Wreg);
 }
 
 void _12bit_processor::reset(RESET_TYPE r)
