@@ -1,6 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
-   Copyright (C) 2006,2009,2010 Roy R Rankin
+   Copyright (C) 2006,2009,2010,2013 Roy R Rankin
 
 This file is part of the libgpsim library of gpsim
 
@@ -190,25 +190,22 @@ unsigned int CCPRH::get()
 class CCPSignalSource : public SignalControl
 {
 public:
-  CCPSignalSource(CCPCON *_ccp)
+  CCPSignalSource(CCPCON *_ccp, int _index)
     : m_ccp(_ccp),
-    state('?')
+    state('?'), index(_index)
   {
     assert(m_ccp);
   }
-  ~CCPSignalSource()
-  {
-  }
+  virtual ~CCPSignalSource() { }
 
   void setState(char m_state) { state = m_state; }
   virtual char getState() { return state; }
-  virtual void release() 
-  {
-    delete this;
-  }
+  virtual void release() { m_ccp->releasePins(index); }
+
 private:
   CCPCON *m_ccp;
   char state;
+  int index;
 };
 
 //--------------------------------------------------
@@ -218,22 +215,21 @@ private:
 class CCPSignalSink : public SignalSink
 {
 public:
-  CCPSignalSink(CCPCON *_ccp)
-    : m_ccp(_ccp)
+  CCPSignalSink(CCPCON *_ccp, int _index)
+    : m_ccp(_ccp), index(_index)
   {
     assert(_ccp);
   }
 
-  void release() 
-  {
-    delete this;
-  }
+  virtual ~CCPSignalSink(){}
+  virtual void release() { m_ccp->releaseSink(); }
   void setSinkState(char new3State)
   {
     m_ccp->new_edge( new3State=='1' || new3State=='W');
   }
 private:
   CCPCON *m_ccp;
+  int index;
 };
 
 class Tristate : public SignalControl
@@ -242,7 +238,7 @@ public:
   Tristate() { }
   ~Tristate() { }
   char getState() { return '1'; }	// set port to high impedance by setting it to input
-  void release() { delete this; }
+  virtual void release() { }
 };
 //--------------------------------------------------
 // CCPCON
@@ -252,7 +248,6 @@ CCPCON::CCPCON(Processor *pCpu, const char *pName, const char *pDesc)
     pstrcon(0),
     pwm1con(0),
     eccpas(0),
-    bit_mask(0x3f),
     m_sink(0),
     m_tristate(0),
     m_bInputEnabled(false),
@@ -261,13 +256,78 @@ CCPCON::CCPCON(Processor *pCpu, const char *pName, const char *pDesc)
     edges(0), bridge_shutdown(false),
     ccprl(0), pir(0), tmr2(0), adcon0(0)
 {
-	m_PinModule[0] = 0;
+    for(int i=0; i<4; i++)
+    {
+	m_PinModule[i] = 0;
+	m_source[i] = 0;
+	source_active[i] = false;
+    }
+    mValidBits = 0x3f;
 }
 
 CCPCON::~CCPCON() 
 { 
+
+
+    for(int i = 0; i<4; i++)
+    {
+	if (m_source[i])
+	{
+	    if ( source_active[i] && m_PinModule[i] )
+	    {
+		m_PinModule[i]->setSource(0);
+	    }
+	    delete m_source[i];
+	}
+    }
+
+    if (m_tristate) delete m_tristate;
+    if (m_PinModule[0] && m_sink) m_PinModule[0]->removeSink(m_sink);
+    if (m_sink) delete m_sink;
 }
 
+void CCPCON::setIOPin1(PinModule *p1)
+{
+  if (p1 && &(p1->getPin()))
+  {
+    if (m_PinModule[0])
+    {
+	if (m_PinModule[0] != p1)
+	{
+	    m_PinModule[0]->removeSink(m_sink);
+	    m_PinModule[0] = p1;
+	    p1->addSink(m_sink);
+	}
+    }
+    else
+    {
+        m_PinModule[0] = p1;
+        m_sink = new CCPSignalSink(this, 0);
+	m_tristate = new Tristate();
+        m_source[0] = new CCPSignalSource(this, 0);
+        p1->addSink(m_sink);
+    }
+  }
+}
+
+void CCPCON::setIOPin2(PinModule *p2)
+{
+  if (p2)
+  {
+    m_PinModule[1] = p2;
+    if (!m_source[1])
+        m_source[1] = new CCPSignalSource(this, 1);
+  }
+  else
+  {
+	if (m_source[1])
+	{
+		delete m_source[1];
+		m_source[1] = 0;
+	}
+	m_PinModule[1] = 0;
+  }
+}
 // EPWM has four outputs PWM 1
 void CCPCON::setIOpin(PinModule *p1, PinModule *p2, PinModule *p3, PinModule *p4)
 {
@@ -275,44 +335,39 @@ void CCPCON::setIOpin(PinModule *p1, PinModule *p2, PinModule *p3, PinModule *p4
   if (!&(p1->getPin()))
   {
 	Dprintf(("FIXME %s::setIOpin called where p1 has unassigned pin\n", name().c_str()));
+	return;
   }
 
-  if (p1 && &(p1->getPin()))
-  {
-    if (m_PinModule[0])
-    {
-	if (m_PinModule[0] != p1)
-	fprintf(stderr, "FIXME %s::setIOpin called for port %s then %s %p %p\n", 
-		name().c_str(), 
-		m_PinModule[0]->getPin().name().c_str(), 
-		p1->getPin().name().c_str() , m_PinModule[0], p1
-	);
-    }
-    else
-    {
-        m_PinModule[0] = p1;
-        m_sink = new CCPSignalSink(this);
-	m_tristate = new Tristate();
-        m_source[0] = new CCPSignalSource(this);
-        p1->addSink(m_sink);
-    }
-  }
-  m_PinModule[1] = m_PinModule[2] = m_PinModule[3] = 0;
-  m_source[1] = m_source[2] = m_source[3] = 0;
-  if (p2)
-  {
-    m_PinModule[1] = p2;
-    m_source[1] = new CCPSignalSource(this);
-  }
+  setIOPin1(p1);
+  setIOPin2(p2);
   if (p3)
   {
     m_PinModule[2] = p3;
-    m_source[2] = new CCPSignalSource(this);
+    if (!m_source[2])
+        m_source[2] = new CCPSignalSource(this, 2);
+  }
+  else
+  {
+	if (m_source[2])
+	{
+		delete m_source[2];
+		m_source[2] = 0;
+	}
+	m_PinModule[2] = 0;
   }
   if (p4)
   {
     m_PinModule[3] = p4;
-    m_source[3] = new CCPSignalSource(this);
+    if (!m_source[3])
+        m_source[3] = new CCPSignalSource(this, 3);
+  }
+  else
+  {
+	if (m_source[3])
+	{
+		delete m_source[3];
+		m_source[3] = 0;
+	}
   }
 
 }
@@ -453,7 +508,7 @@ void CCPCON::compare_match()
       if(adcon0)
 	adcon0->start_conversion();
 
-      Dprintf(("-- CCPCON triggering an A/D conversion\n"));
+      Dprintf(("-- CCPCON triggering an A/D conversion ccprl %p\n", ccprl));
       break;
 
     case PWM0:
@@ -483,6 +538,26 @@ void CCPCON::callback()
         delay_source1 = false;
     }
 }
+
+void CCPCON::releaseSink()
+{
+    delete m_sink;
+    m_sink = 0;
+}
+void CCPCON::releasePins(int i)
+{
+    if (m_PinModule[i])
+    {
+	if (m_source[i])
+	{
+	    delete m_source[i];
+	    m_source[i] = 0;
+	}
+
+	m_PinModule[i] = 0;
+    }
+}
+
 void CCPCON::pwm_match(int level)
 {
   unsigned int new_value = value.get();
@@ -521,10 +596,11 @@ void CCPCON::pwm_match(int level)
         m_cOutputState = level ? '1' : '0';
         m_source[0]->setState(level ? '1' : '0');
         m_PinModule[0]->setSource(m_source[0]);
+	source_active[0] = true;
     
     
         if(level && !ccprl->ccprh->pwm_value)  // if duty cycle == 0 output stays low
-              m_source[0]->setState('0');
+            m_source[0]->setState('0');
 
         m_PinModule[0]->updatePinModule();
     
@@ -599,24 +675,33 @@ void CCPCON::drive_bridge(int level, int new_value)
 		    if (pstrcon_value & (1<<i))
 		    {
 			m_PinModule[i]->setSource(m_source[i]);
+			source_active[i] = true;
 			// follow level except where duty cycle = 0
 			if (level && ccprl->ccprh->pwm_value)
           		    m_source[i]->setState(active_high[i]?'1':'0');
 			else
           		    m_source[i]->setState(active_high[i]?'0':'1');
+
     			m_PinModule[i]->updatePinModule();
 		    }
 		    else if (m_PinModule[i])
+		    {
 			m_PinModule[i]->setSource(0);
+			source_active[i] = false;
+		    }
 		}
 		break;
 	
 	    case 2:	// Half-Bridge
 		Dprintf(("half-bridge\n"));
 		m_PinModule[0]->setSource(m_source[0]);
+		source_active[0] = true;
 		m_PinModule[1]->setSource(m_source[1]);
+		source_active[1] = true;
 		m_PinModule[2]->setSource(0);
+		source_active[2] = false;
 		m_PinModule[3]->setSource(0);
+		source_active[3] = false;
 		delay_source0 = false;
 		delay_source1 = false;
 		// FIXME need to add deadband
@@ -666,6 +751,10 @@ void CCPCON::drive_bridge(int level, int new_value)
 		m_PinModule[1]->setSource(m_source[1]);
 		m_PinModule[2]->setSource(m_source[2]);
 		m_PinModule[3]->setSource(m_source[3]);
+		source_active[0] = true;
+		source_active[1] = true;
+		source_active[2] = true;
+		source_active[3] = true;
 		// P1D toggles
 		if (level && ccprl->ccprh->pwm_value)
           	    m_source[3]->setState(active_high[3]?'1':'0');
@@ -688,6 +777,10 @@ void CCPCON::drive_bridge(int level, int new_value)
 		m_PinModule[1]->setSource(m_source[1]);
 		m_PinModule[2]->setSource(m_source[2]);
 		m_PinModule[3]->setSource(m_source[3]);
+		source_active[0] = true;
+		source_active[1] = true;
+		source_active[2] = true;
+		source_active[3] = true;
 		// P1B toggles
 		if (level && ccprl->ccprh->pwm_value)
           	    m_source[1]->setState(active_high[1]?'1':'0');
@@ -763,7 +856,7 @@ void CCPCON::put(unsigned int new_value)
 {
 
   unsigned int old_value =  value.get();
-  new_value &= bit_mask;
+  new_value &= mValidBits;
   
   Dprintf(("%s::put() new_value=0x%x\n",name().c_str(), new_value));
   trace.raw(write_trace.get() | value.get());
@@ -867,11 +960,15 @@ void CCPCON::put(unsigned int new_value)
     if (oldbOutEn != m_bOutputEnabled && m_PinModule)
     {
 	if (m_bOutputEnabled)
+	{
             m_PinModule[0]->setSource(m_source[0]);
+	    source_active[0] = true;
+	}
 	else
 	{
             m_PinModule[0]->setSource(0);
 	    m_source[0]->setState('?');
+	    source_active[0] = false;
 	}
     }
 
@@ -955,6 +1052,10 @@ T1CON::T1CON(Processor *pCpu, const char *pName, const char *pDesc)
   new_name("T1CON");
 
 }
+T1CON::~T1CON()
+{
+    delete freq_attribute;
+}
 
 void T1CON::put(unsigned int new_value)
 {
@@ -991,7 +1092,262 @@ unsigned int T1CON::get_prescale()
 }
 
 
+//--------------------------------------------------
+//
+//--------------------------------------------------
 
+//
+//  Signal T1GCon on change of state of Gate pin
+//
+class T1GCon_GateSignalSink : public SignalSink
+{
+public:
+  T1GCon_GateSignalSink(T1GCON *_t1gcon)
+    : m_t1gcon(_t1gcon)
+  {
+    assert(_t1gcon);
+  }
+  virtual ~T1GCon_GateSignalSink()
+  {
+  }
+
+  virtual void release() { }
+  virtual void setSinkState(char new3State)
+  {
+    m_t1gcon->IO_gate( new3State=='1' || new3State=='W');
+  }
+private:
+  T1GCON *m_t1gcon;
+};
+
+T1GCON::T1GCON(Processor *pCpu, const char *pName, const char *pDesc, T1CON_G *_t1con_g)
+  : sfr_register(pCpu, pName, pDesc), sink(0), write_mask(0xfb), tmrl(0),
+    t1con_g(_t1con_g),
+    IO_gate_state(false), T0_gate_state(false), CM1_gate_state(false),
+    CM2_gate_state(false), last_t1g_in(false), gate_pin(0)
+{
+}
+
+T1GCON::~T1GCON()
+{
+    if(sink)
+    {
+	delete sink;
+    }
+}
+
+void T1GCON::put(unsigned int new_value)
+{
+  unsigned int old_value = value.get();
+  new_value = (new_value & write_mask) | (old_value & ~write_mask);
+  unsigned int diff = new_value ^ old_value;
+  bool t1ggo = new_value & T1GGO;
+
+  assert(pir_set);
+  assert(tmrl);
+
+  if (!diff) return;
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value);
+
+
+  if (diff & (T1GSS1 | T1GSS0 | T1GPOL))
+  {
+      switch(new_value & (T1GSS1 | T1GSS0))
+      {
+      case 0:
+	IO_gate(IO_gate_state);
+	break;
+
+      case 1:
+	T0_gate(T0_gate_state);
+	break;
+
+      case 2:
+	CM1_gate(CM1_gate_state);
+	break;
+
+      case 3:
+	CM2_gate(CM2_gate_state);
+	break;
+      }
+      // Dont't allow gate change to clear new T1GG0
+      if((diff & T1GGO) && t1ggo)
+	value.put(value.get() | T1GGO);
+  }
+  if (diff & T1GTM)
+  {
+	if ((value.get() & T1GTM) && t1con_g->get_tmr1on()) // T1GTM going high, set t1g_in to 0
+	{
+	   if(value.get() & T1GVAL)
+	   {
+		value.put(value.get() & ~(T1GVAL|T1GGO));
+		pir_set->set_tmr1gif();
+		tmrl->IO_gate(false); // Counting should be stopped
+	   }
+	}
+  }
+
+  tmrl->update();
+}
+
+void T1GCON::setGatepin(PinModule *pin)
+{
+    
+
+    if (pin != gate_pin)
+    {
+        if(sink) 
+	{
+	    gate_pin->removeSink(sink);
+        }
+  	else
+	    sink = new T1GCon_GateSignalSink(this);
+
+	gate_pin = pin;
+        Dprintf(("T1GCON::setGatepin %s\n", pin->getPin().name().c_str()));
+        pin->addSink(sink);
+    }
+}
+
+// The following 4 functions are called on a state change.
+// They pass the state to new_gate if that input is selected.
+void T1GCON::IO_gate(bool state)
+{
+    IO_gate_state = state;
+    if((value.get() & (T1GSS0|T1GSS1)) == 0)
+	new_gate(state);
+}
+void T1GCON::T0_gate(bool state)
+{
+    T0_gate_state = state;
+    if((value.get() & (T1GSS0|T1GSS1)) == 1)
+	new_gate(state);
+}
+void T1GCON::CM1_gate(bool state)
+{
+    CM1_gate_state = state;
+    if((value.get() & (T1GSS0|T1GSS1)) == 2)
+	new_gate(state);
+}
+void T1GCON::CM2_gate(bool state)
+{
+    CM2_gate_state = state;
+    if((value.get() & (T1GSS0|T1GSS1)) == 3)
+	new_gate(state);
+}
+void T1GCON::new_gate(bool state)
+{
+    // TMR1 counts when state low (unless t1gpol is set)
+    // t1g_in is inverted as per XOR in spec sheet flow chart
+    bool t1g_in = (!get_t1GPOL()) ^ state ; 
+    bool t1g_val = value.get() & T1GVAL;
+    unsigned int reg_value = value.get();
+
+    
+    if (t1g_in == last_t1g_in) // no state change, do nothing
+    {
+  //  	tmrl->IO_gate(t1g_val);
+	return;
+    }
+
+    last_t1g_in = t1g_in;
+
+    if ( reg_value & T1GTM) // Toggle mode
+    {
+	t1g_val = reg_value & T1GVAL;
+	if (t1g_in)	// rising edge
+	{
+	    t1g_val = ! t1g_val;		// t1gval changes state
+	}
+	t1g_in = !t1g_val;
+    }
+    else	// Gate directly in control
+    {
+	t1g_val = t1g_in;
+    }
+
+    if (reg_value & T1GSPM)	// Single pulse mode
+    {
+	if (!(reg_value & T1GGO))  // do nothing if T1GGO clear
+	    return;
+
+	if (!t1g_in)		// End of gate
+	{
+	    reg_value &= ~T1GGO;  //set done
+	}
+	t1g_val = t1g_in;
+    }
+
+    
+    if (t1g_val)
+    {
+	reg_value |= T1GVAL;
+    }
+    else
+    {
+	if (reg_value & T1GVAL)	// interrupt on T1GVAL negative edge 
+	{
+	    pir_set->set_tmr1gif();
+
+	}
+	reg_value &= ~T1GVAL;
+    }
+
+    value.put(reg_value);
+	    
+	    
+    // get_t1GINV is set to true so t1g_val active high works OK
+    tmrl->IO_gate(t1g_val);
+}
+
+//--------------------------------------------------
+// T1CON_G
+//--------------------------------------------------
+T1CON_G::T1CON_G(Processor *pCpu, const char *pName, const char *pDesc)
+  //: sfr_register(pCpu, pName, pDesc),
+  : T1CON(pCpu, pName, pDesc),
+    tmrl(0),
+    t1gcon(pCpu, "t1gcon", "TM1 Gate Control Register", this)
+{
+
+  pCpu->addSymbol(freq_attribute = new TMR1_Freq_Attribute(pCpu, 32768.));
+  new_name("T1CON");
+
+}
+
+T1CON_G::~T1CON_G()
+{
+    delete freq_attribute;
+}
+void T1CON_G::put(unsigned int new_value)
+{
+
+  trace.raw(write_trace.get() | value.get());
+  //trace.register_write(address,value.get());
+
+  unsigned int diff = value.get() ^ new_value;
+  value.put(new_value);
+  if (!tmrl)
+    return;
+  // First, check the tmr1 clock source bit to see if we are  changing from
+  // internal to external (or vice versa) clocks.
+  if( diff & (TMR1CS0 | TMR1CS1 | T1OSCEN))
+    tmrl->new_clock_source();
+
+  if( diff & TMR1ON)
+    tmrl->on_or_off(value.get() & TMR1ON);
+  else  if( diff & (T1CKPS0 | T1CKPS1 ))
+    tmrl->update();
+
+}
+
+// If Cap. sensing oscillator T1 clock source, pass to T1
+void T1CON_G::t1_cap_increment()
+{
+	if (get_tmr1cs() == 3)	// T1 input Cap. sensing oscillator
+	    tmrl->increment();
+}
 //--------------------------------------------------
 // member functions for the TMRH base class
 //--------------------------------------------------
@@ -1022,7 +1378,6 @@ void TMRH::put(unsigned int new_value)
 	- (gint64)((tmrl->value.get() + (value.get()<<8) 
 	* tmrl->prescale * tmrl->ext_scale) +0.5);
 
-
   if(tmrl->t1con->get_tmr1on())
     tmrl->update();
 
@@ -1068,10 +1423,7 @@ public:
     assert(_tmr1l);
   }
 
-  void release() 
-  {
-    delete this;
-  }
+  virtual void release() { delete this; }
   void setSinkState(char new3State)
   {
     m_tmr1l->IO_gate( new3State=='1' || new3State=='W');
@@ -1100,7 +1452,8 @@ public:
 //--------------------------------------------------
 TMRL::TMRL(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    m_cState('?'), m_GateState(false), m_bExtClkEnabled(false), 
+    m_cState('?'), m_GateState(false), m_compare_GateState(true),
+    m_io_GateState(true), m_bExtClkEnabled(false), 
     m_sleeping(false), m_t1gss(true), m_Interrupt(0)
 {
 
@@ -1126,15 +1479,19 @@ TMRL::~TMRL()
  * If we are similating an external RTC crystal for timer1,
  * compute scale factor between crsytal speed and processor
  * instruction cycle rate
+ *
+ * If tmr1cs = 1 Fosc is 4 x normal speed so reduce ticks by 1/4
  */ 
 void TMRL::set_ext_scale() 
 { 
     current_value();
-    if (t1con->get_t1oscen() && t1con->get_tmr1cs())
+    if (t1con->get_t1oscen()  && (t1con->get_tmr1cs() == 2)) // external clock
     {
     	ext_scale = get_cycles().instruction_cps()/
 	            t1con->freq_attribute->get_freq();
     }
+    else if (t1con->get_tmr1cs() == 1) // Fosc
+	ext_scale = 0.25;
     else
 	ext_scale = 1.;
 
@@ -1249,7 +1606,8 @@ void TMRL::compare_gate(bool state)
 void TMRL::IO_gate(bool state)
 {
   m_io_GateState = state;
-  if (m_t1gss && m_GateState != state)
+
+  if (m_t1gss && (m_GateState != state))
   {
     m_GateState = state;
     
@@ -1286,6 +1644,10 @@ void TMRL::increment()
   if (t1con->get_t1sync() == 0 && m_sleeping)
     return;
 
+  // prescaler works but rest of timer turned off
+  if (!t1con->get_tmr1on())
+	return;
+
     // If TMRH/TMRL have been manually changed, we'll want to 
     // get the up-to-date values;
 
@@ -1353,10 +1715,22 @@ void TMRL::update()
   // pin has been defined by a call to TMRL::setGatepin()
   //
   bool gate = t1con->get_t1GINV() ? m_GateState : !m_GateState;
+  Dprintf(("TMRL::update gate %d GateState %d inv %d get_tmr1on %x tmr1GE %x tmr1cs %x t1oscen %x\n", gate, m_GateState, t1con->get_t1GINV(), t1con->get_tmr1on(), t1con->get_tmr1GE(), t1con->get_tmr1cs(), t1con->get_t1oscen()));
   if(t1con->get_tmr1on() && (t1con->get_tmr1GE() ? gate : true)) 
   {
-    if(t1con->get_tmr1cs() && t1con->get_t1oscen()) 
+    switch(t1con->get_tmr1cs())
     {
+    case 0:	// internal clock Fosc/4
+	if(verbose & 0x4)
+	    cout << "Tmr1 Internal clock\n";
+	break;
+
+    case 1:	// internal clock Fosc
+	break;
+
+    case 2:	// External clock
+	if (t1con->get_t1oscen())	// External clock enabled
+	{
 	/*
 	 external timer1 clock runs off a crystal which is typically
 	 32768 Hz and is independant on the instruction clock, but
@@ -1364,24 +1738,31 @@ void TMRL::update()
 	 of these two clocks so the breakpoint can be adjusted to be
 	 triggered at the correct time. 
 	*/
-      if(verbose & 0x4)
-	cout << "Tmr1 External clock\n";
-      
-    }
-    else if(t1con->get_tmr1cs() && !  t1con->get_t1oscen()) 
-    {
-      prescale = 1 << t1con->get_prescale();
-      prescale_counter = prescale;
-      set_ext_scale();
-      return;
-    }
-    else
-    {
-      if(verbose & 0x4)
-	cout << "Tmr1 Internal clock\n";
-    }
+	      if(verbose & 0x4)
+		cout << "Tmr1 External clock\n";
+	}
+	else			// External stimuli(pin)
+	{
+	      prescale = 1 << t1con->get_prescale();
+	      prescale_counter = prescale;
+	      set_ext_scale();
+	      return;
+	}
+	break;
 
-    set_ext_scale();
+     case 3:			// Cap. sensing oscillator
+	prescale = 1 << t1con->get_prescale();
+	prescale_counter = prescale;
+	set_ext_scale();
+	return;
+	break;
+
+     default:
+	cout << "TMR1SC reserved value " << t1con->get_tmr1cs() << endl;
+	break;
+     }
+
+     set_ext_scale();
 
 
       // Note, unlike TMR0, anytime something is written to TMRL, the 
@@ -1437,7 +1818,6 @@ void TMRL::update()
         get_cycles().clear_break(this);
 	future_cycle = 0;
       } 
-      //cout << "TMR1: not running\n";
     }
 }
 
@@ -1531,37 +1911,66 @@ unsigned int TMRL::get_low_and_high()
 void TMRL::new_clock_source()
 {
 
-  m_bExtClkEnabled = false;
+    m_bExtClkEnabled = false;
 
-  current_value();
+    current_value();
+
+    switch(t1con->get_tmr1cs())
+    {
+    case 0:	// Fosc/4
+	if(verbose & 0x4)
+	    cout << "Tmr1 Fosc/4 \n";
+	put(value.get());
+	break;
+
+    case 1:	// Fosc
+	if(verbose & 0x4)
+	    cout << "Tmr1 Fosc \n";
+	put(value.get());
+	break;
+
+    case 2:	// External pin or crystal
+
+	if(t1con->get_t1oscen())	// External crystal, simulate
+	{
+	    if(verbose & 0x4)
+		cout << "Tmr1 External Crystal\n";
+	    put(value.get());    // let TMRL::put() set a cycle counter break point
+	}
+	else	// external pin
+	{
+	    if(verbose & 0x4)
+		cout << "Tmr1 External Stimuli\n";
+	    if (future_cycle)
+	    {
+	      // Compute value_16bit with old prescale and ext_scale
+		current_value();
+		get_cycles().clear_break(this);
+		future_cycle = 0;
+	    }
+	    prescale = 1 << t1con->get_prescale();
+	    prescale_counter = prescale;
+	    set_ext_scale();
+	    m_bExtClkEnabled = true;
+	}
+	break;
+
+    case 3:	// Capacitor sense oscillator
+	if(verbose & 0x4)
+		cout << "Tmr1 Cap. sensing oscillator\n";
+	if (future_cycle)
+	{
+	  // Compute value_16bit with old prescale and ext_scale
+		current_value();
+		get_cycles().clear_break(this);
+		future_cycle = 0;
+	}
+	prescale = 1 << t1con->get_prescale();
+	prescale_counter = prescale;
+	set_ext_scale();
+	break;
+    }
 
-  if(t1con->get_tmr1cs() && ! t1con->get_t1oscen()) // external stimuli
-  {
-      if(verbose & 0x4)
-	cout << "Tmr1 External Stimuli\n";
-      if (future_cycle)
-      {
-        // Compute value_16bit with old prescale and ext_scale
-      	current_value();
-        get_cycles().clear_break(this);
-        future_cycle = 0;
-      }
-      prescale = 1 << t1con->get_prescale();
-      prescale_counter = prescale;
-      set_ext_scale();
-      m_bExtClkEnabled = true;
-  }
-  else if(! t1con->get_tmr1cs() && ! t1con->get_t1oscen()) // Fosc/4
-  {
-      if(verbose & 0x4)
-	cout << "Tmr1 Fosc/4 \n";
-      put(value.get());
-  }
-  else {
-      if(verbose & 0x4)
-	cout << "Tmr1 External Crystal\n";
-    put(value.get());    // let TMRL::put() set a cycle counter break point
-  }
 }
 
 //
@@ -1573,7 +1982,10 @@ void TMRL::new_clock_source()
 void TMRL::clear_timer()
 {
 
-  last_cycle = get_cycles().get();
+  synchronized_cycle = get_cycles().get();
+  last_cycle = synchronized_cycle;
+  value.put(0);
+  tmrh->value.put(0);
   if(verbose & 0x4)
     cout << "TMR1 has been cleared\n";
 }
@@ -1592,7 +2004,7 @@ void TMRL::callback()
   // If TMRL is being clocked by the external clock, then at some point
   // the simulate code must have switched from the internal clock to
   // external clock. The cycle break point was still set, so just ignore it.
-  if(t1con->get_tmr1cs() && ! t1con->get_t1oscen())
+  if((t1con->get_tmr1cs() == 2) && ! t1con->get_t1oscen())
     {
       if(verbose & 4)
         cout << "TMRL:callback No oscillator\n";
@@ -1661,8 +2073,8 @@ void TMRL::sleep()
 {
     m_sleeping = true;
     Dprintf(("TMRL::sleep t1sysc %d\n", t1con->get_t1sync()));
-    // If tmr1 is running off Fosc/4 this assumes Fosc stops during sleep
-    if (  t1con->get_tmr1on() && t1con->get_tmr1cs() == 0)
+    // If tmr1 is running off Fosc/4 or Fosc this assumes Fosc stops during sleep
+    if (  t1con->get_tmr1on() && t1con->get_tmr1cs() < 2)
     {
       if (future_cycle)
       {
@@ -1679,7 +2091,7 @@ void TMRL::wake()
 {
     m_sleeping = false;
     Dprintf(("TMRL::wake\n"));
-    if (  t1con->get_tmr1on() && t1con->get_tmr1cs() == 0)
+    if (  t1con->get_tmr1on() && t1con->get_tmr1cs() < 2)
     {
 	update();
     }
@@ -2348,10 +2760,7 @@ public:
     assert(_eccpas);
   }
 
-  void release() 
-  {
-    delete this;
-  }
+  virtual void release() { delete this; }
   void setSinkState(char new3State)
   {
     m_eccpas->set_trig_state( m_index, new3State=='0' || new3State=='w');
@@ -2366,10 +2775,11 @@ private:
 //--------------------------------------------------
 ECCPAS::ECCPAS(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    pwm1con(0), ccp1con(0), bit_mask(0xff),
+    pwm1con(0), ccp1con(0), 
     m_PinModule(0)
 {
     trig_state[0] = trig_state[1] = trig_state[2] = false;
+    mValidBits = 0xff;
 }
 
 ECCPAS::~ECCPAS()
@@ -2390,7 +2800,7 @@ void ECCPAS::put_value(unsigned int new_value)
 {
 
   int old_value = value.get();
-  new_value &= bit_mask;
+  new_value &= mValidBits;
 
 
   // Auto-shutdown trigger active
@@ -2471,9 +2881,9 @@ void ECCPAS::c2_output(int state)
 // PWM1CON
 //--------------------------------------------------
 PWM1CON::PWM1CON(Processor *pCpu, const char *pName, const char *pDesc)
-  : sfr_register(pCpu, pName, pDesc),
-  bit_mask(0xff)
+  : sfr_register(pCpu, pName, pDesc)
 {
+  mValidBits = 0xff;
 }
 
 PWM1CON::~PWM1CON()
@@ -2482,7 +2892,7 @@ PWM1CON::~PWM1CON()
 void PWM1CON::put(unsigned int new_value)
 {
 
-  new_value &= bit_mask;
+  new_value &= mValidBits;
   Dprintf(("PWM1CON::put() new_value=0x%x\n",new_value));
   trace.raw(write_trace.get() | value.get());
 
