@@ -149,15 +149,25 @@ char *ADDULNK::name(char *return_str,int len)
 }
 void ADDULNK::execute()
 {
-  if (opcode & 0x100)
-    cpu16->ind2.put_fsr(cpu16->ind2.get_fsr_value() - m_lit); // SUBULNK
+  if (cpu16->extended_instruction())
+  {
+      if (opcode & 0x100)
+    	cpu16->ind2.put_fsr(cpu16->ind2.get_fsr_value() - m_lit); // SUBULNK
+      else
+    	cpu16->ind2.put_fsr(cpu16->ind2.get_fsr_value() + m_lit); // ADDULNK
+  }
   else
-    cpu16->ind2.put_fsr(cpu16->ind2.get_fsr_value() + m_lit); // ADDULNK
+  {
+      printf("Error %s extended instruction not supported, check XINST\n", 
+	(opcode&0x100)?"SUBULNK":"ADDULNK");
+      bp.halt();
+  }
+
   cpu16->pc->new_address(cpu16->stack->pop());
 }
 
 //---------------------------------------------------------
-ADDFSR::ADDFSR(Processor *new_cpu, unsigned int new_opcode, const char *pName, unsigned int address)
+ADDFSR16::ADDFSR16(Processor *new_cpu, unsigned int new_opcode, const char *pName, unsigned int address)
   : instruction(new_cpu,  new_opcode,address)
 {
   m_fsr = (opcode>>6)&3;
@@ -183,7 +193,7 @@ ADDFSR::ADDFSR(Processor *new_cpu, unsigned int new_opcode, const char *pName, u
 
 }
 
-char *ADDFSR::name(char *return_str,int len)
+char *ADDFSR16::name(char *return_str,int len)
 {
 
   snprintf(return_str,len,"%s\t%d,0x%x",
@@ -195,33 +205,46 @@ char *ADDFSR::name(char *return_str,int len)
 }
 
 
-void ADDFSR::execute()
+void ADDFSR16::execute()
 {
-  if (opcode & 0x100)
-    ia->put_fsr(ia->get_fsr_value() - m_lit);  //SUBFSR
+  if (cpu16->extended_instruction())
+  {
+      if (opcode & 0x100)
+    	ia->put_fsr(ia->get_fsr_value() - m_lit);  //SUBFSR
+      else
+    	ia->put_fsr(ia->get_fsr_value() + m_lit);  //ADDFSR
+  }
   else
-    ia->put_fsr(ia->get_fsr_value() + m_lit);  //ADDFSR
+  {
+      printf("Error %s extended instruction not supported, check XINST\n", 
+	(opcode&0x100)?"SUBFSR":"ADDFSR");
+      bp.halt();
+  }
+  
   cpu16->pc->increment();
 }
 
 //--------------------------------------------------
-CALLW::CALLW(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  :instruction (new_cpu, new_opcode, address)
+void CALLW16::execute()
 {
-  new_name("callw");
-}
-char *CALLW::name(char *return_str,int len)
-{
-
-  snprintf(return_str,len,"%s",
-	   gpsimObject::name().c_str());
-  return(return_str);
-}
-void CALLW::execute()
-{
-  cpu16->stack->push(cpu16->pc->get_next());
-  cpu16->pcl->put(cpu16->W->get());
-  cpu16->pc->increment();
+  if (cpu16->extended_instruction())
+  {
+      if(cpu16->stack->push(cpu16->pc->get_next()))
+      {
+    	cpu16->pcl->put(cpu16->Wget());
+    	cpu16->pc->increment();
+      }
+      else	// stack overflow reset
+      {
+          cpu16->pc->jump(0);
+      }
+  } 
+  else
+  {
+      printf("Error %s extended instruction not supported, check XINST\n", 
+	"CALLW");
+      bp.halt();
+  }
 }
 
 //--------------------------------------------------
@@ -240,8 +263,18 @@ char *PUSHL::name(char *return_str,int len)
 }
 void PUSHL::execute()
 {
-  cpu16->ind2.put(m_lit);
-  cpu16->ind2.put_fsr(cpu16->ind2.get_fsr_value() -1);
+//  cpu16->ind2.put(m_lit);
+//  cpu16->ind2.put_fsr(cpu16->ind2.get_fsr_value() -1);
+  if (cpu16->extended_instruction())
+  {
+      cpu16->ind2.postdec.put(m_lit);
+  }
+  else
+  {
+      printf("Error %s extended instruction not supported, check XINST\n", 
+	"PUSHL");
+      bp.halt();
+  }
   cpu16->pc->increment();
 }
 
@@ -306,20 +339,29 @@ char *MOVSF::name(char *return_str,int len)
 
 void MOVSF::execute()
 {
-  if(!initialized)
-    runtime_initialize();
+  if (cpu16->extended_instruction())
+  {
+      if(!initialized)
+	runtime_initialize();
 
-  unsigned int source_addr = (cpu16->ind2.get_fsr_value() + source)&0xfff;
+      unsigned int source_addr = cpu16->ind2.plusk_fsr_value(source);
 
-  unsigned int r =  cpu_pic->registers[source_addr]->get();
-  cpu16->pc->skip();
+      unsigned int r =  cpu_pic->registers[source_addr]->get();
+      cpu16->pc->skip();
 
-  unsigned int destination_addr =
-    (opcode & 0x80) ? 
-    (cpu16->ind2.get_fsr_value() + destination)&0xfff
-    :
-    destination;
-  cpu_pic->registers[destination_addr]->put(r);
+      unsigned int destination_addr =
+    	(opcode & 0x80) ? 
+    	  cpu16->ind2.plusk_fsr_value(destination)
+    	  :
+    	  destination;
+      cpu_pic->registers[destination_addr]->put(r);
+  }
+  else
+  {
+      printf("Error %s extended instruction not supported, check XINST\n", 
+	(opcode & 0x80)?"MOVSS":"MOVSF");
+      bp.halt();
+  }
 
   //cpu16->pc->increment();
 
@@ -330,9 +372,9 @@ void ADDLW16::execute()
 {
   unsigned int old_value,new_value;
 
-  new_value = (old_value = cpu16->W->value.get()) + L;
+  new_value = (old_value = cpu16->Wget()) + L;
 
-  cpu16->W->put(new_value & 0xff);
+  cpu16->Wput(new_value & 0xff);
   cpu16->status->put_Z_C_DC_OV_N(new_value, old_value, L);
 
   cpu16->pc->increment();
@@ -344,12 +386,14 @@ void ADDWF16::execute()
 {
   unsigned int new_value,src_value,w_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  new_value = (src_value = source->get()) + (w_value = cpu16->W->value.get());
+  new_value = (src_value = source->get()) + (w_value = cpu16->Wget());
 
   // Store the result
 
@@ -360,7 +404,7 @@ void ADDWF16::execute()
     }
   else
     {
-      cpu16->W->put(new_value & 0xff);
+      cpu16->Wput(new_value & 0xff);
       cpu16->status->put_Z_C_DC_OV_N(new_value, w_value, src_value);
     }
 
@@ -370,24 +414,19 @@ void ADDWF16::execute()
 
 //--------------------------------------------------
 
-ADDWFC::ADDWFC (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  : Register_op(new_cpu, new_opcode, address)
-{
-  decode(new_cpu, new_opcode);
-  new_name("addwfc");
-}
-
-void ADDWFC::execute()
+void ADDWFC16::execute()
 {
   unsigned int new_value,src_value,w_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   new_value = (src_value = source->get()) + 
-    (w_value = cpu16->W->value.get()) +
+    (w_value = cpu16->Wget()) +
     ((cpu16->status->value.get() & STATUS_C) ? 1 : 0);
 
   // Store the result
@@ -395,7 +434,7 @@ void ADDWFC::execute()
   if(destination)
     source->put(new_value & 0xff);      // Result goes to source
   else
-    cpu16->W->put(new_value & 0xff);
+    cpu16->Wput(new_value & 0xff);
 
   cpu16->status->put_Z_C_DC_OV_N(new_value, src_value, w_value);
 
@@ -409,9 +448,9 @@ void ANDLW16::execute()
 {
   unsigned int new_value;
 
-  new_value = cpu16->W->value.get() & L;
+  new_value = cpu16->Wget() & L;
 
-  cpu16->W->put(new_value);
+  cpu16->Wput(new_value);
   cpu16->status->put_N_Z(new_value);
 
   cpu16->pc->increment();
@@ -424,17 +463,19 @@ void ANDWF16::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  new_value = source->get() & cpu16->W->value.get();
+  new_value = source->get() & cpu16->Wget();
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   cpu16->status->put_N_Z(new_value);
 
@@ -569,7 +610,7 @@ void BOV::execute()
 }
 
 //--------------------------------------------------
-BRA::BRA (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+BRA16::BRA16 (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : instruction(new_cpu, new_opcode, address)
 {
   destination_index = (new_opcode & 0x7ff)+1;
@@ -584,13 +625,13 @@ BRA::BRA (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   new_name("bra");
 }
 
-void BRA::execute()
+void BRA16::execute()
 {
   cpu16->pc->jump(absolute_destination_index);
 
 }
 
-char * BRA::name(char *return_str,int len)
+char * BRA16::name(char *return_str,int len)
 {
 
 
@@ -603,6 +644,85 @@ char * BRA::name(char *return_str,int len)
   return(return_str);
 }
 
+//--------------------------------------------------
+void BSF16::execute()
+{
+
+
+  if (access)
+      reg = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      reg = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      reg = cpu_pic->registers[register_address];
+
+  reg->put(reg->get_value() | mask);    // Must not use reg->value.get() as it breaks indirects
+
+
+  cpu16->pc->increment();
+
+}
+
+//--------------------------------------------------
+void BCF16::execute()
+{
+
+
+  if (access)
+      reg = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      reg = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      reg = cpu_pic->registers[register_address];
+
+  reg->put(reg->get_value() & mask);    // Must not use reg->value.get() as it breaks indirects
+
+
+  cpu16->pc->increment();
+
+}
+
+//--------------------------------------------------
+void BTFSC16::execute()
+{
+
+
+  if (access)
+      reg = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      reg = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      reg = cpu_pic->registers[register_address];
+
+
+  unsigned int result = mask & reg->get();
+
+  if(!result)
+    cpu_pic->pc->skip();                  // Skip next instruction
+  else
+    cpu_pic->pc->increment();
+}
+
+//--------------------------------------------------
+void BTFSS16::execute()
+{
+
+
+  if (access)
+      reg = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      reg = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      reg = cpu_pic->registers[register_address];
+
+
+  unsigned int result = mask & reg->get();
+
+  if(result)
+    cpu_pic->pc->skip();                  // Skip next instruction
+  else
+    cpu_pic->pc->increment();
+}
 
 //--------------------------------------------------
 
@@ -615,10 +735,12 @@ BTG::BTG (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 
 void BTG::execute()
 {
-  if(!access)
-    reg = cpu_pic->registers[register_address];
+  if (access)
+      reg = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      reg = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
   else
-    reg = cpu_pic->register_bank[register_address];
+      reg = cpu_pic->registers[register_address];
 
   reg->put(reg->get() ^ mask);
 
@@ -663,11 +785,15 @@ void CALL16::execute()
   if(!initialized)
     runtime_initialize();
 
-  cpu16->stack->push(cpu16->pc->get_next());
-  if(fast)
-    cpu16->fast_stack.push();
+  if (cpu16->stack->push(cpu16->pc->get_next()))
+  {
+    if(fast)
+      cpu16->fast_stack.push();
 
-  cpu16->pc->jump(destination_index);
+    cpu16->pc->jump(destination_index);
+  }
+  else	// stack overflow reset
+    cpu16->pc->jump(0);
 
 }
 
@@ -689,10 +815,12 @@ void COMF16::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   new_value = source->get() ^ 0xff;
 
@@ -701,7 +829,7 @@ void COMF16::execute()
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   cpu16->status->put_N_Z(new_value);
 
@@ -720,12 +848,14 @@ CPFSEQ::CPFSEQ (Processor *new_cpu, unsigned int new_opcode, unsigned int addres
 
 void CPFSEQ::execute()
 {
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  if(source->get() == cpu16->W->value.get())
+  if(source->get() == cpu16->Wget())
     cpu16->pc->skip();                  // Skip next instruction
   else
     cpu16->pc->increment();
@@ -743,12 +873,14 @@ CPFSGT::CPFSGT (Processor *new_cpu, unsigned int new_opcode, unsigned int addres
 
 void CPFSGT::execute()
 {
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  if(source->get() > cpu16->W->value.get())
+  if(source->get() > cpu16->Wget())
     cpu16->pc->skip();                  // Skip next instruction
   else
     cpu16->pc->increment();
@@ -766,18 +898,35 @@ CPFSLT::CPFSLT (Processor *new_cpu, unsigned int new_opcode, unsigned int addres
 
 void CPFSLT::execute()
 {
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  if(source->get() < cpu16->W->value.get())
+  if(source->get() < cpu16->Wget())
     cpu16->pc->skip();                  // Skip next instruction
   else
     cpu16->pc->increment();
 
 }
 
+void CLRF16::execute()
+{
+
+  if (access)
+      cpu_pic->register_bank[register_address]->put(0);
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      cpu_pic->registers[register_address + cpu16->ind2.fsr_value]->put(0);
+  else
+      cpu_pic->registers[register_address]->put(0);
+
+  cpu16->status->put_Z(1);
+
+  cpu16->pc->increment();
+
+}
 //--------------------------------------------------
 
 DAW::DAW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
@@ -791,14 +940,14 @@ void DAW::execute()
 {
   unsigned int new_value;
 
-  new_value = cpu16->W->value.get();
+  new_value = cpu16->Wget();
   if(((new_value & 0x0f) > 0x9) || (cpu16->status->value.get() & STATUS_DC))
     new_value += 0x6;
 
   if(((new_value & 0xf0) > 0x90) || (cpu16->status->value.get() & STATUS_C))
     new_value += 0x60;
 
-  cpu16->W->put(new_value & 0xff);
+  cpu16->Wput(new_value & 0xff);
   if ( new_value>0xff )
       cpu16->status->put_C(1);
   else if ( cpu16->bugs() & BUG_DAW )
@@ -813,10 +962,12 @@ void DAW::execute()
 void DECF16::execute()
 {
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   unsigned int src_value = source->get();
   unsigned int new_value = src_value - 1;
@@ -824,7 +975,7 @@ void DECF16::execute()
   if(destination)
     source->put(new_value & 0xff);      // Result goes to source
   else
-    cpu16->W->put(new_value & 0xff);
+    cpu16->Wput(new_value & 0xff);
 
   //  cpu16->status->put_N_Z(new_value);
   cpu16->status->put_Z_C_DC_OV_N_for_sub(new_value,src_value,1);
@@ -839,17 +990,19 @@ void DECFSZ16::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   new_value = (source->get() - 1)&0xff;
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   if(0==new_value)
     cpu16->pc->skip();                  // Skip next instruction
@@ -871,17 +1024,19 @@ void DCFSNZ::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   new_value = (source->get() - 1)&0xff;
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   if(0!=new_value)
     cpu16->pc->skip();                  // Skip next instruction
@@ -917,10 +1072,12 @@ void INCF16::execute()
   unsigned int new_value, src_value;
 
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   src_value = source->get();
   new_value = (src_value + 1);
@@ -932,7 +1089,7 @@ void INCF16::execute()
     }
   else
     {
-      cpu16->W->put(new_value & 0xff);
+      cpu16->Wput(new_value & 0xff);
       cpu16->status->put_Z_C_DC_OV_N(new_value, 1, src_value);
     }
 
@@ -946,17 +1103,19 @@ void INCFSZ16::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   new_value = (source->get() + 1)&0xff;
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   if(0==new_value)
     cpu16->pc->skip();                  // Skip next instruction
@@ -978,17 +1137,19 @@ void INFSNZ::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   new_value = (source->get() + 1)&0xff;
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   if(0!=new_value)
     cpu16->pc->skip();                  // Skip next instruction
@@ -1003,9 +1164,9 @@ void IORLW16::execute()
 {
   unsigned int new_value;
 
-  new_value = cpu16->W->value.get() | L;
+  new_value = cpu16->Wget() | L;
 
-  cpu16->W->put(new_value);
+  cpu16->Wput(new_value);
   cpu16->status->put_N_Z(new_value);
 
   cpu16->pc->increment();
@@ -1018,17 +1179,19 @@ void IORWF16::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  new_value = source->get() | cpu16->W->value.get();
+  new_value = source->get() | cpu16->Wget();
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   cpu16->status->put_N_Z(new_value);
 
@@ -1162,10 +1325,12 @@ void MOVF16::execute()
 {
   unsigned int source_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   source_value = source->get();
 
@@ -1174,7 +1339,7 @@ void MOVF16::execute()
   if(destination)
     source->put(source_value);
   else
-    cpu16->W->put(source_value);
+    cpu16->Wput(source_value);
 
 
   cpu16->status->put_N_Z(source_value);
@@ -1305,23 +1470,25 @@ void MOVFP::execute()
 
 }
 
+
 //--------------------------------------------------
 
-MOVLB::MOVLB (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+MOVLB16::MOVLB16 (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : Literal_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("movlb");
 }
 
-void MOVLB::execute()
+void MOVLB16::execute()
 {
-  cpu16->bsr.put(L);
+  cpu16->registers[cpu16->bsr.address]->put(L);
 
   cpu16->pc->increment();
 
 }
 
+
 //--------------------------------------------------
 
 MOVLR::MOVLR (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
@@ -1414,18 +1581,23 @@ void MOVPF::execute()
 MOVWF16::MOVWF16(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : MOVWF(new_cpu,new_opcode, address)
 {
-  register_address = new_opcode & 0xff;
 }
 
 void MOVWF16::execute()
 {
-  source = cpu_pic->register_bank[register_address];
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  source->put(cpu16->W->get());
+  source->put(cpu16->Wget());
 
   cpu16->pc->increment();
 }
 
+#ifdef RRR
 //--------------------------------------------------
 MOVWF16a::MOVWF16a(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : MOVWF(new_cpu,new_opcode, address)
@@ -1439,10 +1611,11 @@ MOVWF16a::MOVWF16a(Processor *new_cpu, unsigned int new_opcode, unsigned int add
 void MOVWF16a::execute()
 {
   source = cpu_pic->registers[register_address];
-  source->put(cpu16->W->get());
+  source->put(cpu16->Wget());
 
   cpu16->pc->increment();
 }
+#endif //RRR
 
 //--------------------------------------------------
 
@@ -1460,7 +1633,7 @@ void MULLW::execute()
 {
   unsigned int value;
 
-  value = (0xff & cpu16->W->get()) * L;
+  value = (0xff & cpu16->Wget()) * L;
 
   cpu16->prodl.put(value &0xff);
   cpu16->prodh.put((value>>8) &0xff);
@@ -1483,14 +1656,16 @@ void MULWF::execute()
 {
   unsigned int value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   //It's not necessary to '&' the get()'s with 0xff, but it doesn't
   //hurt either. 
-  value = (0xff & cpu16->W->get()) * (0xff & source->get());
+  value = (0xff & cpu16->Wget()) * (0xff & source->get());
 
   cpu16->prodl.put(value &0xff);
   cpu16->prodh.put((value>>8) &0xff);
@@ -1512,10 +1687,12 @@ void NEGF::execute()
 {
   unsigned int new_value,src_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   src_value = source->get();
   new_value = 1 + ~src_value;        // two's complement
@@ -1577,9 +1754,10 @@ PUSH::PUSH (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 void PUSH::execute()
 {
 
-  cpu16->stack->push(cpu16->pc->get_next());
-
-  cpu16->pc->increment();
+  if (cpu16->stack->push(cpu16->pc->get_next()))
+      cpu16->pc->increment();
+  else	// stack overflow reset
+      cpu16->pc->jump(0);
 
 }
 
@@ -1599,9 +1777,10 @@ RCALL::RCALL (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 
 void RCALL::execute()
 {
-  cpu16->stack->push(cpu16->pc->get_next());
-
-  cpu16->pc->jump(absolute_destination_index);
+  if(cpu16->stack->push(cpu16->pc->get_next()))
+      cpu16->pc->jump(absolute_destination_index);
+  else	// stack overflow reset
+      cpu16->pc->jump(0);
 
 }
 
@@ -1619,19 +1798,6 @@ char * RCALL::name(char *return_str,int len)
 }
 
 
-//--------------------------------------------------
-
-RESET::RESET (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  : instruction(new_cpu, new_opcode, address)
-{
-  decode(new_cpu, new_opcode);
-  new_name("reset");
-}
-
-void RESET::execute()
-{
-  cpu16->reset(SOFT_RESET);
-}
 
 //--------------------------------------------------
 void RETFIE16::execute()
@@ -1687,10 +1853,12 @@ void RLCF::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   new_value = (source->get() << 1) | cpu16->status->get_C();
 
@@ -1698,7 +1866,7 @@ void RLCF::execute()
   if(destination)
     source->put(new_value&0xff);      // Result goes to source
   else
-    cpu16->W->put(new_value&0xff);
+    cpu16->Wput(new_value&0xff);
 
   cpu16->status->put_Z_C_N(new_value);
 
@@ -1719,10 +1887,12 @@ void RLNCF::execute()
 {
   unsigned int new_value,src_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   src_value = source->get();
   new_value = (src_value << 1) | ( (src_value & 0x80) ? 1 : 0);
@@ -1731,7 +1901,7 @@ void RLNCF::execute()
   if(destination)
     source->put(new_value&0xff);      // Result goes to source
   else
-    cpu16->W->put(new_value&0xff);
+    cpu16->Wput(new_value&0xff);
 
   cpu16->status->put_N_Z(new_value);
 
@@ -1753,10 +1923,12 @@ void RRCF::execute()
 {
   unsigned int new_value,src_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   src_value = source->get() & 0xff;
   new_value = (src_value >> 1) | (cpu16->status->get_C() ? 0x80 : 0);
@@ -1765,7 +1937,7 @@ void RRCF::execute()
   if(destination)
     source->put(new_value&0xff);      // Result goes to source
   else
-    cpu16->W->put(new_value&0xff);
+    cpu16->Wput(new_value&0xff);
 
   cpu16->status->put_Z_C_N(new_value | ((src_value & 1) ? 0x100 : 0) );
 
@@ -1786,10 +1958,12 @@ void RRNCF::execute()
 {
   unsigned int new_value,src_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
   src_value = source->get() & 0xff;
   new_value = (src_value >> 1) | ( (src_value & 1) ? 0x80 : 0);
@@ -1798,7 +1972,7 @@ void RRNCF::execute()
   if(destination)
     source->put(new_value&0xff);      // Result goes to source
   else
-    cpu16->W->put(new_value&0xff);
+    cpu16->Wput(new_value&0xff);
 
   cpu16->status->put_N_Z(new_value | ((src_value & 1) ? 0x100 : 0) );
 
@@ -1817,10 +1991,13 @@ SETF::SETF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 
 void SETF::execute()
 {
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
 
   source->put(0xff);
@@ -1842,9 +2019,9 @@ void SUBLW16::execute()
 {
   unsigned int new_value,old_value;
 
-  new_value = L - (old_value = cpu16->W->value.get());
+  new_value = L - (old_value = cpu16->Wget());
 
-  cpu16->W->put(new_value & 0xff);
+  cpu16->Wput(new_value & 0xff);
 
   cpu16->status->put_Z_C_DC_OV_N_for_sub(new_value, L, old_value);
 
@@ -1866,18 +2043,20 @@ void SUBFWB::execute()
 {
   unsigned int new_value,src_value,w_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  new_value = (w_value = cpu16->W->value.get()) - (src_value = source->get()) -
+  new_value = (w_value = cpu16->Wget()) - (src_value = source->get()) -
     (1 - cpu16->status->get_C());
 
   if(destination)
     source->put(new_value & 0xff);
   else
-    cpu16->W->put(new_value & 0xff);
+    cpu16->Wput(new_value & 0xff);
 
   cpu16->status->put_Z_C_DC_OV_N_for_sub(new_value, w_value, src_value);
 
@@ -1892,17 +2071,19 @@ void SUBWF16::execute()
 {
   unsigned int new_value,src_value,w_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  new_value = (src_value = source->get()) - (w_value = cpu16->W->value.get());
+  new_value = (src_value = source->get()) - (w_value = cpu16->Wget());
 
   if(destination)
     source->put(new_value & 0xff);
   else
-    cpu16->W->put(new_value & 0xff);
+    cpu16->Wput(new_value & 0xff);
 
   cpu16->status->put_Z_C_DC_OV_N_for_sub(new_value, src_value, w_value);
 
@@ -1912,29 +2093,24 @@ void SUBWF16::execute()
 
 //--------------------------------------------------
 
-SUBWFB::SUBWFB (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  : Register_op(new_cpu, new_opcode, address)
-{
-  decode(new_cpu, new_opcode);
-  new_name("subwfb");
-}
-
-void SUBWFB::execute()
+void SUBWFB16::execute()
 {
   unsigned int new_value,src_value,w_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  new_value = (src_value = source->get()) - (w_value = cpu16->W->value.get()) -
+  new_value = (src_value = source->get()) - (w_value = cpu16->Wget()) -
     (1 - cpu16->status->get_C());
 
   if(destination)
     source->put(new_value & 0xff);
   else
-    cpu16->W->put(new_value & 0xff);
+    cpu16->Wput(new_value & 0xff);
 
   cpu16->status->put_Z_C_DC_OV_N_for_sub(new_value, src_value, w_value);
 
@@ -1943,6 +2119,33 @@ void SUBWFB::execute()
 }
 
 
+//--------------------------------------------------
+
+void SWAPF16::execute()
+{
+  unsigned int src_value;
+
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
+
+  src_value = source->get();
+
+  if(destination)
+    source->put( ((src_value >> 4) & 0x0f) | ( (src_value << 4) & 0xf0) );
+  else
+    cpu_pic->Wput( ((src_value >> 4) & 0x0f) | ( (src_value << 4) & 0xf0) );
+
+
+
+  cpu16->pc->increment();
+
+}
+
+
 //--------------------------------------------------
 
 TBLRD::TBLRD (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
@@ -2114,10 +2317,12 @@ TSTFSZ::TSTFSZ (Processor *new_cpu, unsigned int new_opcode, unsigned int addres
 void TSTFSZ::execute()
 {
 
-  if(!access)
-    source = cpu_pic->registers[register_address];
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
   else
-    source = cpu_pic->register_bank[register_address];
+      source = cpu_pic->registers[register_address];
 
   if( 0 == (source->get() & 0xff) )
     cpu16->pc->skip();                  // Skip next instruction
@@ -2131,9 +2336,9 @@ void XORLW16::execute()
 {
   unsigned int new_value;
 
-  new_value = cpu16->W->value.get() ^ L;
+  new_value = cpu16->Wget() ^ L;
 
-  cpu16->W->put(new_value);
+  cpu16->Wput(new_value);
   cpu16->status->put_N_Z(new_value);
 
   cpu16->pc->increment();
@@ -2146,17 +2351,19 @@ void XORWF16::execute()
 {
   unsigned int new_value;
 
-  source = ((!access) ?
-	    cpu_pic->registers[register_address] 
-	    :
-	    cpu_pic->register_bank[register_address] );
+  if (access)
+      source = cpu_pic->register_bank[register_address];
+  else if (cpu16->extended_instruction() && register_address < 0x60)
+      source = cpu_pic->registers[register_address + cpu16->ind2.fsr_value];
+  else
+      source = cpu_pic->registers[register_address];
 
-  new_value = source->get() ^ cpu16->W->value.get();
+  new_value = source->get() ^ cpu16->Wget();
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu16->W->put(new_value);
+    cpu16->Wput(new_value);
 
   cpu16->status->put_N_Z(new_value);
 
