@@ -155,6 +155,57 @@ public:
   }
 };
 
+//------------------------------------------------------------------------
+// Config4L - default 
+//  The default Config4L register controls the 18F series WDT. 
+class Config4L : public ConfigWord 
+{
+#define STKVREN  (1<<0)
+#define LVP	(1<<2)
+#define BBSIZ0	(1<<4)
+#define BBSIZ1	(1<<5)
+#define XINST	(1<<6)
+#define _DEBUG	(1<<7)
+
+#define CONFIG4L_default (STKVREN | LVP | _DEBUG)
+public:
+  Config4L(_16bit_processor *pCpu, unsigned int addr)
+    : ConfigWord("CONFIG4L", CONFIG4L_default, "Config word 4L", pCpu, addr)
+  {
+	set(CONFIG4L_default);
+  }
+#define Cpu16 ((_16bit_processor *)m_pCpu)
+  virtual void set(gint64 v)
+  {
+    Integer::set(v);
+    if (m_pCpu)
+    {
+	Cpu16->set_extended_instruction((v & XINST) == XINST);
+    	if(m_pCpu->stack)
+	    m_pCpu->stack->STVREN = ((v & STKVREN) == STKVREN);
+    }
+
+  }
+
+  virtual string toString()
+  {
+    gint64 i64;
+    get(i64);
+    int i = i64 &0xfff;
+
+    char buff[256];
+
+    snprintf(buff,sizeof(buff),
+             "$%04x\n"
+             " STVREN=%d - BBSIZE=%x XINST=%d\n",
+             i,
+             (i&STKVREN?1:0), (i & (BBSIZ1 | BBSIZ0))>>4,
+	     (i&XINST?1:0));
+
+    return string(buff);
+  }
+};
+
 
 
 #define PWRTEN  1<<0
@@ -164,7 +215,6 @@ public:
 
 #define CCP2MX  1<<0
 
-#define STVREN  1<<0
 //-------------------------------------------------------------------
 _16bit_processor::_16bit_processor(const char *_name, const char *desc)
   : pic_processor(_name,desc),
@@ -256,7 +306,7 @@ _16bit_processor::_16bit_processor(const char *_name, const char *desc)
 //-------------------------------------------------------------------
 _16bit_processor::~_16bit_processor()
 {
-
+    delete_sfr_map();
 }
 
 //-------------------------------------------------------------------
@@ -280,13 +330,143 @@ pic_processor *_16bit_processor::construct()
 
 }
 
+void _16bit_processor :: delete_sfr_map()
+{
+  if(verbose)
+    cout << "deleting 18cxxx common registers " << hex << last_actual_register() <<"\n";
+
+  unassignMCLRPin();
+  delete_file_registers(0x0, last_register);
+  remove_sfr_register(&pie1);
+  remove_sfr_register(&pir1);
+  remove_sfr_register(&ipr1);
+  remove_sfr_register(&pie2);
+  remove_sfr_register(&pir2);
+  remove_sfr_register(&ipr2);
+  remove_sfr_register(&usart.rcsta);
+  remove_sfr_register(&usart.txsta);
+  remove_sfr_register(&usart.spbrg);
+  delete_sfr_register(usart.txreg);
+  delete_sfr_register(usart.rcreg);
+  remove_sfr_register(&t3con);
+  remove_sfr_register(&tmr3l);
+  remove_sfr_register(&tmr3h);
+
+
+  if ( HasCCP2() )
+  {
+    remove_sfr_register(&ccp2con);
+    remove_sfr_register(&ccpr2l);
+    remove_sfr_register(&ccpr2h);
+  }
+  remove_sfr_register(&ccp1con);
+  remove_sfr_register(&ccpr1l);
+  remove_sfr_register(&ccpr1h);
+
+  remove_sfr_register(&adresl);
+  remove_sfr_register(&adresh);
+
+  remove_sfr_register(&ssp.sspcon2);
+  remove_sfr_register(&ssp.sspcon);
+  remove_sfr_register(&ssp.sspstat);
+  remove_sfr_register(&ssp.sspadd);
+  remove_sfr_register(&ssp.sspbuf);
+
+  remove_sfr_register(&t2con);
+  remove_sfr_register(&pr2);
+  remove_sfr_register(&tmr2);
+  remove_sfr_register(&t1con);
+  remove_sfr_register(&tmr1l);
+  remove_sfr_register(&tmr1h);
+
+  remove_sfr_register(&rcon);
+  remove_sfr_register(&wdtcon);
+  remove_sfr_register(&lvdcon);
+  remove_sfr_register(&osccon);
+  remove_sfr_register(&t0con);
+  remove_sfr_register(&tmr0l);
+  remove_sfr_register(&tmr0h);
+  remove_sfr_register(&ind2.fsrl);
+  remove_sfr_register(&ind2.fsrh);
+  remove_sfr_register(&ind2.plusw);
+  remove_sfr_register(&ind2.preinc);
+  remove_sfr_register(&ind2.postdec);
+  remove_sfr_register(&ind2.postinc);
+  remove_sfr_register(&ind2.postinc);
+  remove_sfr_register(&ind2.indf);
+
+  remove_sfr_register(&bsr);
+
+  remove_sfr_register(&ind1.fsrl);
+  remove_sfr_register(&ind1.fsrh);
+  remove_sfr_register(&ind1.plusw);
+  remove_sfr_register(&ind1.preinc);
+  remove_sfr_register(&ind1.postdec);
+  remove_sfr_register(&ind1.postinc);
+  remove_sfr_register(&ind1.indf);
+
+
+  remove_sfr_register(&ind0.fsrl);
+  remove_sfr_register(&ind0.fsrh);
+  remove_sfr_register(&ind0.plusw);
+  remove_sfr_register(&ind0.preinc);
+  remove_sfr_register(&ind0.postdec);
+  remove_sfr_register(&ind0.postinc);
+  remove_sfr_register(&ind0.indf);
+
+  remove_sfr_register(&intcon3);
+  remove_sfr_register(&intcon2);
+  remove_sfr_register(&intcon);
+
+  remove_sfr_register(&prodl);
+  remove_sfr_register(&prodh);
+
+  remove_sfr_register(&tbl.tablat);
+  remove_sfr_register(&tbl.tabptrl);
+  remove_sfr_register(&tbl.tabptrh);
+  remove_sfr_register(&tbl.tabptru);
+  remove_sfr_register(&pclatu);
+
+  Stack16 *stack16 = static_cast<Stack16 *>(stack);
+  remove_sfr_register(&stack16->stkptr);
+  remove_sfr_register(&stack16->tosl);
+  remove_sfr_register(&stack16->tosh);
+  remove_sfr_register(&stack16->tosu);
+
+  EEPROM *e = get_eeprom();
+
+  if (e) {
+    remove_sfr_register(e->get_reg_eedata());
+    remove_sfr_register(e->get_reg_eeadr());
+    if ( e->get_reg_eeadrh() )
+        remove_sfr_register(e->get_reg_eeadrh());
+    remove_sfr_register(e->get_reg_eecon1());
+    remove_sfr_register(e->get_reg_eecon2());
+    delete e;
+  }
+  delete_sfr_register(m_porta);
+  delete_sfr_register(m_lata);
+  delete_sfr_register(m_trisa);
+  delete_sfr_register(m_portb);
+  delete_sfr_register(m_latb);
+  delete_sfr_register(m_trisb);
+  if ( HasPortC() )
+  {
+      delete_sfr_register(m_portc);
+      delete_sfr_register(m_latc);
+      delete_sfr_register(m_trisc);
+  }
+  delete pc;
+}
+
 //-------------------------------------------------------------------
 void _16bit_processor :: create_sfr_map()
 {
   if(verbose)
     cout << "creating 18cxxx common registers\n";
 
-  add_file_registers(0x0, last_actual_register(), 0);
+  last_register = last_actual_register();
+  add_file_registers(0x0, last_register, 0);
 
   RegisterValue porv(0,0);
   RegisterValue porv2(0,0);
@@ -377,7 +557,7 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&ind2.postinc, 0xfde,porv,"postinc2");
   add_sfr_register(&ind2.indf,    0xfdf,porv,"indf2");
 
-  add_sfr_register(&bsr,          0xfe0,porv,"bsr");
+  add_sfr_register(&bsr,          0xfe0,porv, "bsr");
 
   add_sfr_register(&ind1.fsrl,	  0xfe1,porv,"fsr1l");
   add_sfr_register(&ind1.fsrh,    0xfe2,porv,"fsr1h");
@@ -387,7 +567,7 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&ind1.postinc, 0xfe6,porv,"postinc1");
   add_sfr_register(&ind1.indf,    0xfe7,porv,"indf1");
 
-  add_sfr_register(W,            0xfe8);
+  add_sfr_register(Wreg,            0xfe8);
 
   add_sfr_register(&ind0.fsrl,	  0xfe9,porv,"fsr0l");
   add_sfr_register(&ind0.fsrh,    0xfea,porv,"fsr0h");
@@ -417,12 +597,14 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(pcl,     0xff9);
   add_sfr_register(pclath,  0xffa);
   add_sfr_register(&pclatu, 0xffb, porv, "pclatu");
+  pclath->mValidBits = 0xFF;    // Data sheet implies does not depend on memory size
 
   Stack16 *stack16 = static_cast<Stack16 *>(stack);
   add_sfr_register(&stack16->stkptr,  0xffc,porv,"stkptr");
   add_sfr_register(&stack16->tosl,    0xffd,porv,"tosl");
   add_sfr_register(&stack16->tosh,    0xffe,porv,"tosh");
   add_sfr_register(&stack16->tosu,    0xfff,porv,"tosu");
+  stack16->stack_mask = 31;
 
 
   EEPROM *e = get_eeprom();
@@ -613,6 +795,18 @@ void _16bit_processor::exit_sleep()
   }
 }
 
+//------------------------------------------------------------------
+// It is assummed that this will only be set to true for processors
+// that support extended instructions
+//
+void _16bit_processor::set_extended_instruction(bool v)
+{
+    if (verbose)
+	cout << "_16bit_processor::set_extended_instruction " << v <<"\n";
+
+   extended_instruction_flag = v;
+}
+
 //-------------------------------------------------------------------
 // Fetch the rom contents at a particular address.
 unsigned int _16bit_processor::get_program_memory_at_address(unsigned int address)
@@ -669,7 +863,8 @@ bool  _16bit_processor::set_config_word(unsigned int address, unsigned int cfg_w
 {
   if (address >= CONFIG1L && address <= 0x30000D) {
 
-    cout << "Setting config word 0x"<<hex<<address<< " = 0x"<<cfg_word<< endl;
+    if (verbose)
+        cout << "Setting config word 0x"<<hex<<address<< " = 0x"<<cfg_word<< endl;
     address -= CONFIG1L;
 
     if (m_configMemory) {
@@ -699,6 +894,7 @@ void _16bit_processor::create_config_memory()
   m_configMemory = new ConfigMemory(this,configMemorySize());
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H(this, CONFIG1H));
   m_configMemory->addConfigWord(CONFIG2H-CONFIG1L,new Config2H(this, CONFIG2H));
+  m_configMemory->addConfigWord(CONFIG4L-CONFIG1L,new Config4L(this, CONFIG4L));
 }
 
 
@@ -844,9 +1040,16 @@ void _16bit_compat_adc :: a2d_compat()
 
 }
 _16bit_compat_adc::_16bit_compat_adc(const char *_name, const char *desc) 
-	: _16bit_processor(_name, desc)
+	: _16bit_processor(_name, desc), adcon0(0), adcon1(0)
 {
 }
+_16bit_compat_adc::~_16bit_compat_adc()
+{
+    if(adcon0)
+	delete_sfr_register(adcon0);
+    if(adcon1)
+	delete_sfr_register(adcon1);
+}
 void _16bit_v2_adc::create(int nChannels)
 {
 
@@ -890,6 +1093,15 @@ void _16bit_v2_adc::create(int nChannels)
 
 }
 _16bit_v2_adc::_16bit_v2_adc(const char *_name, const char *desc) 
-	: _16bit_processor(_name, desc)
+	: _16bit_processor(_name, desc), adcon0(0), adcon1(0), adcon2(0)
 {
 }
+_16bit_v2_adc::~_16bit_v2_adc()
+{
+    if(adcon0)
+	delete_sfr_register(adcon0);
+    if(adcon1)
+	delete_sfr_register(adcon1);
+    if(adcon2)
+	delete_sfr_register(adcon2);
+}
