@@ -69,7 +69,7 @@ public:
   static const unsigned int CONFIG7L = 0x30000C;
   static const unsigned int CONFIG7H = 0x30000D;
 
-  // So far, all 18xxx parts contain ports A,B,C
+  // The early 18xxx parts all contain ports A,B,C
   PicPortRegister  *m_porta;
   PicTrisRegister  *m_trisa;
   PicLatchRegister *m_lata;
@@ -137,17 +137,23 @@ public:
 
 
 
+  // Some configuration stuff for stripping down where needed
+  virtual bool HasPortC(void) { return true; };
+  virtual bool HasCCP2(void) { return true; };
+
   virtual void create_symbols();
 
   void interrupt();
   virtual void create();// {return;};
   virtual PROCESSOR_TYPE isa(){return _PIC17_PROCESSOR_;};
   virtual PROCESSOR_TYPE base_isa(){return _PIC17_PROCESSOR_;};
+  virtual unsigned int access_gprs() { return 0x80; };
   virtual instruction * disasm (unsigned int address, unsigned int inst)
     {
       return disasm16(this, address, inst);
     }
   virtual void create_sfr_map();
+  virtual void delete_sfr_map();
   virtual void create_config_memory();
 
   // Return the portion of pclath that is used during branching instructions
@@ -185,6 +191,8 @@ public:
   virtual void enter_sleep();
   virtual void exit_sleep();
   virtual void osc_mode(unsigned int );
+  virtual void set_extended_instruction(bool);
+  virtual bool extended_instruction() {return extended_instruction_flag;}
 
 
 
@@ -195,10 +203,14 @@ public:
   unsigned int getCurrentDisasmAddress() { return m_current_disasm_address;}
   unsigned int getCurrentDisasmIndex()   { return m_current_disasm_address/2;}
   void setCurrentDisasmAddress(unsigned a) { m_current_disasm_address =a; }
+
 protected:
   unsigned int m_current_disasm_address;  // Used only when .hex/.cod files are loaded
 
   unsigned int idloc[4];    ///< ID locations - not all 16-bit CPUs have 8 bytes
+  bool	extended_instruction_flag;	// Instruction set extension and Indexed Addressing 
+
+  unsigned int last_register;
 };
 
 class _16bit_compat_adc : public _16bit_processor 
@@ -209,6 +221,7 @@ public:
   ADCON1       *adcon1;
 
   _16bit_compat_adc(const char *_name=0, const char *desc=0);
+  ~_16bit_compat_adc();
   virtual void create_symbols();
   virtual void create();
   virtual void create_sfr_map();
@@ -223,6 +236,7 @@ public:
   ADCON2_V2       *adcon2;
 
   _16bit_v2_adc(const char *_name=0, const char *desc=0);
+  ~_16bit_v2_adc();
   virtual void create(int nChannels);
 };
 #define cpu16 ( (_16bit_processor *)cpu)
