@@ -31,35 +31,6 @@ License along with this library; if not, see
 
 #include "clock_phase.h"
 
-//--------------------------------------------------
-// member functions for the BSR class
-//--------------------------------------------------
-//
-BSR::BSR(Processor *pCpu, const char *pName, const char *pDesc)
-  : sfr_register(pCpu,pName,pDesc),
-    register_page_bits(0)
-{
-}
-
-void  BSR::put(unsigned int new_value)
-{
-  trace.raw(write_trace.get() | value.get());
-
-  value.put(new_value & 0x0f);
-  cpu_pic->register_bank = &cpu_pic->registers[ value.get() << 8 ];
-
-
-}
-
-void  BSR::put_value(unsigned int new_value)
-{
-  put(new_value);
-
-  update();
-  cpu16->indf->update();
-}
-
-
 //--------------------------------------------------
 // member functions for the FSR class
 //--------------------------------------------------
@@ -319,7 +290,7 @@ void PLUSW::put_value(unsigned int new_value)
 
 //------------------------------------------------
 
-Indirect_Addressing::Indirect_Addressing(_16bit_processor *pCpu, const string &n)
+Indirect_Addressing::Indirect_Addressing(pic_processor *pCpu, const string &n)
   : fsrl(pCpu, (string("fsrl")+n).c_str(), "FSR Low", this),
     fsrh(pCpu, (string("fsrh")+n).c_str(), "FSR High", this),
     indf(pCpu, (string("indf")+n).c_str(), "Indirect Register", this),
@@ -337,7 +308,7 @@ Indirect_Addressing::Indirect_Addressing(_16bit_processor *pCpu, const string &n
   postdec.iam = this;
   plusw.iam = this;
   */
-  current_cycle = 0;
+  current_cycle = (guint64)(-1);   // Not zero! See bug #3311944
   fsr_value = 0;
   fsr_state = 0;
   fsr_delta = 0;
@@ -498,7 +469,7 @@ int Indirect_Addressing::plusw_fsr_value()
 
   fsr_value += fsr_delta;
   fsr_delta = 0;
-  int signExtendedW = cpu_pic->W->value.get() | ((cpu_pic->W->value.get() > 127) ? 0xf00 : 0);
+  int signExtendedW = cpu_pic->Wreg->value.get() | ((cpu_pic->Wreg->value.get() > 127) ? 0xf00 : 0);
   unsigned int destination = (fsr_value + signExtendedW) & _16BIT_REGISTER_MASK;
   if(is_indirect_register(destination))
     return -1;
@@ -507,6 +478,19 @@ int Indirect_Addressing::plusw_fsr_value()
 
 }
 
+int Indirect_Addressing::plusk_fsr_value(int k)
+{
+
+  fsr_value += fsr_delta;
+  fsr_delta = 0;
+  unsigned int destination = (fsr_value + k) & _16BIT_REGISTER_MASK;
+  if(is_indirect_register(destination))
+    return -1;
+  else
+    return destination;
+
+}
+
 //------------------------------------------------
 void Fast_Stack::init(_16bit_processor *new_cpu)
 {
@@ -515,7 +499,7 @@ void Fast_Stack::init(_16bit_processor *new_cpu)
 
 void Fast_Stack::push()
 {
-  w = cpu->W->value.get();
+  w = cpu->Wreg->value.get();
   status = cpu->status->value.get();
   bsr = cpu->bsr.value.get();
 
@@ -524,7 +508,7 @@ void Fast_Stack::push()
 void Fast_Stack::pop()
 {
   //cout << "popping fast stack\n";
-  cpu->W->put(w);
+  cpu->Wreg->put(w);
   cpu->status->put(status);
   cpu->bsr.put(bsr);
 
@@ -596,7 +580,7 @@ void Program_Counter16::computed_goto(unsigned int new_address)
   // the instruction (i.e. via the ::increment() method). The second cycle occurs
   // here:
 
-  mCurrentPhase = mExecute1Cycle;
+  mExecute2ndHalf->advance();
 }
 
 //--------------------------------------------------
@@ -605,7 +589,8 @@ void Program_Counter16::computed_goto(unsigned int new_address)
 
 void Program_Counter16::put_value(unsigned int new_value)
 {
-  cout << "Program_Counter16::put_value 0x" << hex << new_value << '\n';
+  if (verbose)
+      cout << "Program_Counter16::put_value 0x" << hex << new_value << '\n';
 
   trace.raw(trace_other | (value<<1));
 
@@ -635,75 +620,6 @@ unsigned int Program_Counter16::get_value()
   return value << 1;
 }
 
-//------------------------------------------------
-// TOSL
-TOSL::TOSL(Processor *pCpu, const char *pName, const char *pDesc)
-  : sfr_register(pCpu,pName,pDesc)
-{}
-
-unsigned int TOSL::get()
-{
-  value.put(stack->get_tos() & 0xff);
-  trace.raw(read_trace.get() | value.get());
-  return(value.get());
-}
-
-unsigned int TOSL::get_value()
-{
-  value.put(stack->get_tos() & 0xff);
-  return(value.get());
-}
-
-void TOSL::put(unsigned int new_value)
-{
-  trace.raw(write_trace.get() | value.get());
-  stack->put_tos( (stack->get_tos() & 0xffffff00) | (new_value & 0xff));
-}
-
-void TOSL::put_value(unsigned int new_value)
-{
-  stack->put_tos( (stack->get_tos() & 0xffffff00) | (new_value & 0xff));
-  update();
-}
-
-
-//------------------------------------------------
-// TOSH
-TOSH::TOSH(Processor *pCpu, const char *pName, const char *pDesc)
-  : sfr_register(pCpu,pName,pDesc)
-{}
-
-unsigned int TOSH::get()
-{
-  value.put((stack->get_tos() >> 8) & 0xff);
-  trace.raw(read_trace.get() | value.get());
-  return(value.get());
-
-}
-
-unsigned int TOSH::get_value()
-{
-
-  value.put((stack->get_tos() >> 8) & 0xff);
-  return(value.get());
-
-}
-
-void TOSH::put(unsigned int new_value)
-{
-  trace.raw(write_trace.get() | value.get());
-  stack->put_tos( (stack->get_tos() & 0xffff00ff) | ( (new_value & 0xff) << 8));
-}
-
-void TOSH::put_value(unsigned int new_value)
-{
-
-  stack->put_tos( (stack->get_tos() & 0xffff00ff) | ( (new_value & 0xff) << 8));
-
-  update();
-
-}
-
 
 //------------------------------------------------
 // TOSU
@@ -741,20 +657,26 @@ void TOSU::put_value(unsigned int new_value)
 
 //------------------------------------------------
 // STKPTR
-STKPTR::STKPTR(Processor *pCpu, const char *pName, const char *pDesc)
+STKPTR16::STKPTR16(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu,pName,pDesc)
 {}
-void STKPTR::put_value(unsigned int new_value)
+void STKPTR16::put_value(unsigned int new_value)
 {
+  stack->pointer = new_value & stack->stack_mask;
   value.put(new_value);
   update();
 }
 
+void STKPTR16::put(unsigned int new_value)
+{
+    trace.raw(write_trace.get() | value.get());
+    put_value(new_value);
+}
 
 //--------------------------------------------------
 //
-Stack16::Stack16(Processor *pCpu)
-  : stkptr(pCpu, "stkptr"),
+Stack16::Stack16(Processor *pCpu) : Stack(pCpu), 
+    stkptr(pCpu, "stkptr", "Stack pointer"),
     tosl(pCpu, "tosl", "Top of Stack low byte"),
     tosh(pCpu, "tosh", "Top of Stack high byte"),
     tosu(pCpu, "tosu", "Top of Stack upper byte")
@@ -766,64 +688,80 @@ Stack16::Stack16(Processor *pCpu)
 
 }
 
-void Stack16::push(unsigned int address)
+Stack16::~Stack16()
 {
 
-  stkptr.value.put(1+stkptr.value.get());
-
-  if((stkptr.value.get() & Stack16_MASK) == 0)
-    {
-      // check the STVREN bit
-      // if(STVREN) {reset(stack_over_flow); return;}
-      stkptr.value.put( stkptr.value.get() | 0x9f);
-    }
-
-  // Push 21-bit address onto the stack
-
-  contents[stkptr.value.get() & Stack16_MASK] = address << 1;
-
-  stkptr.value.put(stkptr.value.get() & 0xdf);
-
+  pic_processor *pCpu = dynamic_cast<pic_processor *>(cpu);
+  if (pCpu)
+  {
+    pCpu->remove_sfr_register(&stkptr);
+    pCpu->remove_sfr_register(&tosl);
+    pCpu->remove_sfr_register(&tosh);
+    pCpu->remove_sfr_register(&tosu);
+  }
 }
 
-unsigned int Stack16::pop()
-{
-  if(stkptr.value.get() & Stack16_MASK)
-    {
-      // read 21-bit address from stack
-      unsigned int ret = (contents[stkptr.value.get() & Stack16_MASK]) >> 1;
-      stkptr.value.put(stkptr.value.get()-1);
-      stkptr.value.put(stkptr.value.get() & 0x5f);
-      return(ret);
 
+// pop of empty stack sets undeflow and returns 0
+unsigned int Stack16::pop() 
+{ 
+    if(pointer <= 0)
+    {
+	pointer = 0;
+	stack_underflow();
+	return(0);
     }
-    // return(contents[ (--stkptr.value) & Stack16_MASK]);
+    --pointer;
+    unsigned int stkptr_status = stkptr.value.get() & ~stack_mask;
+    stkptr.value.put((pointer & stack_mask) | stkptr_status);
+    return(contents[pointer & stack_mask] >> 1);
+}
+// When stack is full last(top) entry is overwritten
+bool Stack16::push(unsigned int address) 
+{ 
+    contents[pointer & stack_mask] = address << 1;
+    if(pointer >= (int)stack_mask)
+    {
+	pointer = stack_mask;
+	return stack_overflow();
+    }
+    pointer++;
+    stkptr.value.put((pointer & stack_mask) | (stkptr.value.get() & ~stack_mask));
+    return true;
+}
+
+void Stack16::reset(RESET_TYPE r)
+{
+  unsigned int reg_value;
+
+  if (r != POR_RESET && r != BOD_RESET)
+	reg_value = stkptr.value.get() & ~stack_mask;
   else
+	reg_value = 0;
+  pointer = 0;
+  stkptr.value.put( reg_value);
+}
+bool Stack16::stack_underflow()
+{
+    stkptr.value.put(STKPTR::STKUNF); // don't decrement past 0, signalize STKUNF
+    if(STVREN)
     {
-      // check the STVREN bit
-      // if(STVREN) {reset(stack_over_flow); return;}
-      stkptr.value.put(0x40); // don't decrement past 0, signalize STKUNF
-      return(0); // return 0 if underflow
+ 	cpu->reset(STKUNF_RESET);
+	return false;
     }
+    cout <<"Stack undeflow\n";
+    return true;
 }
-
-void Stack16::reset()
+bool Stack16::stack_overflow()
 {
-  stkptr.value.put( 0);
-}
-
-unsigned int Stack16::get_tos()
-{
-
-  return (contents[stkptr.value.get() & Stack16_MASK]);
-
-}
-
-void Stack16::put_tos(unsigned int new_tos)
-{
-
-  contents[stkptr.value.get() & Stack16_MASK] = new_tos;
-
+    stkptr.value.put( STKPTR::STKOVF | (pointer & stack_mask));
+    if(STVREN)
+    {
+	cpu->reset(STKOVF_RESET);
+	return false;
+    }
+    cout << "Stack overflow\n";
+    return true;
 }
 
 //--------------------------------------------------
@@ -871,16 +809,12 @@ void T0CON::put(unsigned int new_value)
 
   } 
 
-  // First, check the tmr0 clock source bit to see if we are  changing from
-  // internal to external (or vice versa) clocks.
-  if( (value.get() ^ old_value) & T0CS)
-    cpu16->tmr0l.new_clock_source();
 
   // %%%FIX ME%%% - can changing the state of TOSE cause the timer to
   // increment if tmr0 is being clocked by an external clock?
 
   // Now check the rest of the tmr0 bits.
-  if( (value.get() ^ old_value) & (T0SE | PSA | PS2 | PS1 | PS0))
+  if( (value.get() ^ old_value) & (T0CS | T0SE | PSA | PS2 | PS1 | PS0))
     cpu16->tmr0l.new_prescale();
 
   //cout <<"T0CON::put - new val 0x" << hex << value.get() <<'\n';
