@@ -74,7 +74,6 @@ int Breakpoints::set_breakpoint(BREAKPOINT_TYPES break_type,
 				unsigned arg2, 
 				TriggerObject *f1)
 {
-  Register *fr;
 
   breakpoint_number = find_free();
   if(breakpoint_number >= MAX_BREAKPOINTS)
@@ -90,7 +89,6 @@ int Breakpoints::set_breakpoint(BREAKPOINT_TYPES break_type,
     {
 
     case BREAK_ON_INVALID_FR:
-      fr = cpu->registers[arg1];
       return(breakpoint_number);
       break;
 
@@ -863,6 +861,7 @@ void Breakpoints::dump_traced(unsigned int b)
 
 void Breakpoints::clear_all(Processor *c)
 {
+  GetTraceLog().close_logfile();
   for(int i=0; i<MAX_BREAKPOINTS; i++)
     if(break_status[i].type != BREAK_CLEAR
        && break_status[i].cpu == c)
@@ -1298,7 +1297,6 @@ BreakpointRegister::BreakpointRegister()
 */
 BreakpointRegister::~BreakpointRegister()
 {
-  cout << __FUNCTION__ << " destructor\n";
 }
 
 BreakpointRegister::BreakpointRegister(Processor *_cpu, 
@@ -1716,6 +1714,7 @@ Break_register_write::Break_register_write(Processor *_cpu, int _repl, int bp ):
 { 
 }
 
+
 Break_register_write::~Break_register_write()
 {
 }
@@ -2023,6 +2022,16 @@ int CommandAssertion::printTraced(Trace *pTrace, unsigned int tbi,
 
 //============================================================================
 
+void Log_Register_Write::put(unsigned int new_value)
+{
+  getReplaced()->put(new_value);
+  takeAction();
+}
+void Log_Register_Write::setbit(unsigned int bit_number, bool new_value)
+{
+  getReplaced()->setbit(bit_number,new_value);
+  takeAction();
+}
 void Log_Register_Write::takeAction()
 {
   GetTraceLog().register_write(getReg(),
@@ -2034,6 +2043,32 @@ void Log_Register_Write_value::takeAction()
 				     get_cycles().get());
 }
 
+unsigned int Log_Register_Read::get()
+{
+  unsigned int v = getReplaced()->get();
+  takeAction();
+  return v;
+}
+RegisterValue  Log_Register_Read::getRV()
+{
+  RegisterValue rv = getReplaced()->getRV();
+  takeAction();
+  return rv;
+}
+
+RegisterValue  Log_Register_Read::getRVN()
+{
+  RegisterValue rv = getReplaced()->getRVN();
+  takeAction();
+  return rv;
+}
+
+bool Log_Register_Read::get_bit(unsigned int bit_number)
+{
+  takeAction();
+  return(getReplaced()->get_bit(bit_number));
+}
+
 void Log_Register_Read::takeAction()
 {
   GetTraceLog().register_read(getReg(),
