@@ -235,6 +235,7 @@ public:
 #define  GLOBAL_SLEEP         (1<<2)
 #define  GLOBAL_PM_WRITE      (1<<3)
 #define  GLOBAL_SOCKET        (1<<4)
+#define  GLOBAL_LOG	      (1<<5)
 
   struct BreakStatus
   {
@@ -349,6 +350,7 @@ public:
   bool set_expression(unsigned bp_number, Expression *pExpr);
 
   inline void clear_global() {global_break = GLOBAL_CLEAR;};
+  inline void set_logging() { global_break |= GLOBAL_LOG; }
   void halt();
   inline bool have_halt() 
     { 
@@ -651,6 +653,8 @@ class Log_Register_Write : public Break_register_write
     Break_register_write(_cpu,_repl,bp ) { };
   virtual char const * bpName() { return "log register write"; }
   virtual void takeAction();
+  virtual void put(unsigned int new_value);
+  virtual void setbit(unsigned int bit_number, bool new_value);
 };
 
 class Log_Register_Read : public Break_register_read
@@ -660,6 +664,10 @@ public:
     Break_register_read(_cpu,_repl,bp ) { };
   virtual char const * bpName() { return "log register read"; }
   virtual void takeAction();
+  virtual unsigned int get();
+  virtual RegisterValue getRV();
+  virtual RegisterValue getRVN();
+  virtual bool get_bit(unsigned int bit_number);
 };
 
 class Log_Register_Read_value : public  Break_register_read_value
