@@ -1,7 +1,7 @@
 /*
         
    Copyright (C) 1998 T. Scott Dattalo
-   Copyright (C) 2006,2010 Roy R. Rankin
+   Copyright (C) 2006,2010,2013 Roy R. Rankin
 
 This file is part of the libgpsim library of gpsim
 
@@ -77,18 +77,17 @@ void ComparatorModule::initialize( PIR_SET *pir_set,
 class CMSignalSource : public SignalControl
 {
 public:
-  CMSignalSource()
-    : m_state('0')
+  CMSignalSource(CMCON *_cmcon, int _index)
+    : m_state('0'), m_cmcon(_cmcon), index(_index)
   {
   }
   ~CMSignalSource()
   {
-    cout << "deleting CMsignal source " << this << endl;
+//    cout << "deleting CMsignal source " << this << endl;
   }
   virtual void release()
   {
-    cout << "CMSignalSource:" << this << endl;
-    delete this;
+    m_cmcon->releasePin(index);
   }
   virtual char getState()
   {
@@ -100,6 +99,40 @@ public:
   }
 private:
   char m_state;
+  CMCON * m_cmcon;
+  int  index;
+};
+//--------------------------------------------------
+//
+//--------------------------------------------------
+class CM12CON0;
+
+class CM12SignalSource : public SignalControl
+{
+public:
+  CM12SignalSource( CM12CON0 *_cm12con0)
+    : m_state('0'), m_cm12con0(_cm12con0)
+  {
+  }
+  ~CM12SignalSource()
+  {
+//    cout << "deleting CMsignal source " << this << endl;
+  }
+  virtual void release()
+  {
+    m_cm12con0->releasePin();
+  }
+  virtual char getState()
+  {
+    return m_state;
+  }
+  void putState(bool new_val)
+  {
+        m_state = new_val?'1':'0';
+  }
+private:
+  char m_state;
+  CM12CON0 *m_cm12con0;
 };
 
 CM_stimulus::CM_stimulus(CMCON * arg, const char *cPname,double _Vth, double _Zth)
@@ -108,6 +141,10 @@ CM_stimulus::CM_stimulus(CMCON * arg, const char *cPname,double _Vth, double _Zt
         _cmcon = arg;
 }
 
+CM_stimulus::~CM_stimulus()
+{
+}
+
 void   CM_stimulus::set_nodeVoltage(double v)
 {
 	Dprintf(("set_nodeVoltage =%.1f\n", v));
@@ -163,11 +200,38 @@ CMCON::CMCON(Processor *pCpu, const char *pName, const char *pDesc)
 
 CMCON::~CMCON()
 {
+  unsigned int mode = value.get() & 0x07;
+
+
+  for(int i = 0; i <2; i++)
+  {
+	if (cm_source[i])
+	{
+	    int cfg = m_configuration_bits[i][mode] & CFG_MASK;
+	    
+	    // Our Source active so port still defined if cm_output defined,
+	    // set default source.
+	    if (cfg == i && cm_output[i])
+		cm_output[i]->setSource(0);
+	    delete cm_source[i];
+	}
+  }
+  if (cm_stimulus[0]) delete cm_stimulus[0];
+  if (cm_stimulus[1]) delete cm_stimulus[1];
+  if (cm_stimulus[2]) delete cm_stimulus[2];
+  if (cm_stimulus[3]) delete cm_stimulus[3];
   free(cm_input_pin[0]);  free(cm_input_pin[1]);
   free(cm_input_pin[2]);  free(cm_input_pin[3]);
   free(cm_output_pin[0]); free(cm_output_pin[1]);
-  free(cm_output_pin[2]); free(cm_output_pin[3]);
+}
 
+
+void CMCON::releasePin(int i)
+{
+    if (cm_output[i]) cm_output[i]->setSource(0);
+    if (cm_source[i]) delete cm_source[i];
+    cm_output[i] = 0;
+    cm_source[i] = 0;
 }
 void CMCON::setINpin(int i, PinModule *newPinModule)
 {
@@ -259,10 +323,11 @@ unsigned int CMCON::get()
 
    if (value.get() ^ cmcon_val) // change of state
    {
+	int diff = value.get() ^ cmcon_val;
+
 	// Signal ECCPAS ?
 	if (m_eccpas)
 	{
-	    int diff = value.get() ^ cmcon_val;
 	    if (diff & C1OUT)
 		m_eccpas->c1_output(cmcon_val & C1OUT);
 	    if (diff & C2OUT)
@@ -270,7 +335,13 @@ unsigned int CMCON::get()
 	}
         // Generate interupt ?
         if (pir_set)
-                pir_set->set_cmif();
+	{
+	    if (diff & C1OUT)
+                pir_set->set_c1if();
+
+	    if (diff & C2OUT)
+                pir_set->set_c2if();
+	}
    }
    if (m_tmrl)
 	m_tmrl->compare_gate((cmcon_val & C1OUT) == C1OUT);
@@ -325,7 +396,7 @@ void CMCON::put(unsigned int new_value)
       {
 	  char name[20];
           if ( ! cm_source[i])
-                cm_source[i] = new CMSignalSource();
+                cm_source[i] = new CMSignalSource(this, i);
 	  sprintf(name, "c%dout", i+1);
 	  cm_output[i]->getPin().newGUIname(name);
           cm_output[i]->setSource(cm_source[i]);
@@ -387,11 +458,12 @@ CMCON1::~CMCON1() {}
 void CMCON1::put(unsigned int new_value)
 {
 
+
   if (verbose)
       cout << "CMCON1::put(new_value) =" << hex << new_value << endl;
 
   assert(m_tmrl);
-  m_tmrl->set_T1GSS((new_value & T1GSS) == T1GSS);
+  m_tmrl->set_T1GSS(new_value & T1GSS);
 
   trace.raw(write_trace.get() | value.get());
   value.put(new_value);
@@ -504,6 +576,7 @@ double CM1CON0::CVref()
 }
 void CM2CON0::state_change(unsigned int cmcon_val)
 {
+
    if (value.get() ^ cmcon_val) // change of state
    {
 	// Mirror C2OUT in CM2CON1::MC2OUT
@@ -544,6 +617,16 @@ double CM2CON0::CVref()
 	else
 	    return(0.6);
 }
+
+CM1CON0_2::~CM1CON0_2()
+{
+  if (cm_stimulus[0]) delete cm_stimulus[0]; 
+  if (cm_stimulus[1]) delete cm_stimulus[1]; 
+  cm_stimulus[0] = 0;
+  cm_stimulus[1] = 0;
+  if (cm_cvref) delete cm_cvref;
+  if (cm_v06ref) delete cm_v06ref;
+}
 void CM1CON0_2::state_change(unsigned int cmcon_val)
 {
    if (!cm_stimulus[0])
@@ -670,6 +753,12 @@ CM12CON0::CM12CON0(Processor *pCpu, const char *pName, const char *pDesc)
 CM12CON0::~CM12CON0()
 {
 
+
+  if ((value.get() & (ON | OE)) && cm_output) cm_output->setSource(0);
+  if (cm_source) delete cm_source;
+  if ((!cm_snode[0]) && cm_stimulus[0]) delete cm_stimulus[0];
+  if ((!cm_snode[1]) && cm_stimulus[1]) delete cm_stimulus[1];
+
 }
 void CM12CON0::put(unsigned int new_value)
 {
@@ -697,7 +786,7 @@ void CM12CON0::put(unsigned int new_value)
 		}
 			
 	    }
-	    else if (cm_snode[0] != NULL) // Using CVref so turn of monitoring
+	    else if (cm_snode[0] != NULL) // Using CVref so turn off monitoring
 	    {
 		cm_snode[0]->detach_stimulus(cm_stimulus[0]);
 		cm_snode[0] = NULL;
@@ -795,8 +884,15 @@ void CM12CON0::setpins(PinModule * c12in0, PinModule * c12in1,
    // one output pin
    cm_output = cout;
    if (! cm_source)
-	cm_source = new CMSignalSource();
+	cm_source = new CM12SignalSource(this);
 } 
+
+void CM12CON0::releasePin()
+{
+    if (cm_source) delete cm_source;
+    cm_source = 0;
+    cm_output = 0;
+}
 void CM12CON0::link_registers(PIR_SET *new_pir_set, CM2CON1 *_cm2con1,
         VRCON *_vrcon, SRCON *_srcon, ECCPAS *_eccpas)
 {
@@ -822,7 +918,7 @@ VRCON::VRCON(Processor *pCpu, const char *pName, const char *pDesc)
 
 VRCON::~VRCON()
 {
-  delete pin_name;
+  free(pin_name);
 }
 
 void VRCON::setIOpin(PinModule *newPinModule)
@@ -896,7 +992,7 @@ void VRCON::put(unsigned int new_value)
         {
             if (!strcmp("Vref", vr_PinModule->getPin().name().c_str()))
                 vr_PinModule->getPin().newGUIname(pin_name);
-            if (diff & 0x0f)    // did value of vreference change ?
+            if (diff & 0x2f)    // did value of vreference change ?
                 _cmcon->get();
             if(vr_PinModule && vr_PinModule->getPin().snode)
             {
@@ -905,6 +1001,11 @@ void VRCON::put(unsigned int new_value)
                 vr_PinModule->getPin().snode->update();
             }
         }
+	else	// output pin not defined
+	{
+            if (diff & 0x2f)    // did value of vreference change ?
+                _cmcon->get();
+	}
   }
   else  // vref disable
   {
@@ -998,3 +1099,326 @@ void VRCON_2::put(unsigned int new_value)
 }
 
 
+CMxCON0::CMxCON0(Processor *pCpu, const char *pName, const char *pDesc, unsigned int _cm, ComparatorModule2 *cmModule)
+  : sfr_register(pCpu, pName, pDesc),
+  cm(_cm), m_cmModule(cmModule), cm_source(0)
+{
+}
+CMxCON0::~CMxCON0()
+{
+    if (cm_source)
+	delete cm_source;
+}
+
+void CMxCON0::put(unsigned int new_value)
+{
+  unsigned int old_value = value.get();
+  new_value &= mValidBits;
+  unsigned int diff = new_value ^ old_value;
+
+  if (diff == 0)
+  {
+     get(); 
+     return;
+  }
+
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value);
+  if (diff & CxOE)
+  {
+      PinModule *out_pin = m_cmModule->cmxcon1[cm]->output_pin();
+      if(new_value & CxOE)
+      {
+	  char name[20];
+          if ( ! cm_source)
+                cm_source = new PeripheralSignalSource(out_pin);
+
+	  sprintf(name, "c%dout", cm+1);
+	  assert(out_pin);
+	  out_pin->getPin().newGUIname(name);
+          out_pin->setSource(cm_source);
+      }
+      else if (cm_source)	// Enable output enable turned off
+      {
+	    out_pin->getPin().newGUIname(out_pin->getPin().name().c_str());
+            out_pin->setSource(0);
+      }
+  }
+  get();
+}
+//
+// evaluate inputs and determine output
+//
+unsigned int CMxCON0::get()
+{
+    bool output;
+    unsigned int cmxcon0 = value.get();
+    double Vpos = m_cmModule->cmxcon1[cm]->get_Vpos();
+    double Vneg = m_cmModule->cmxcon1[cm]->get_Vneg();
+    bool old_out = cmxcon0 & CxOUT;
+    
+    if (! cmxcon0 & CxON)
+	output = false;
+
+    else if (cmxcon0 & CxHYS)
+    {
+    	bool cxout = cmxcon0 & CxOUT;
+    	if (cmxcon0 & CxPOL) cxout = !cxout;
+    	    output = cxout;
+    	if (cxout && (Vpos + 0.05) < Vneg)
+    	   output = false;
+    	else if (!cout && Vpos > (Vneg + 0.05))
+    	   output = true;
+    }
+    else
+    {
+        output = Vpos > Vneg;
+    }
+    if (cmxcon0 & CxPOL) output = !output;
+
+    if(output)
+	cmxcon0 |= CxOUT;
+    else
+	cmxcon0 &= ~CxOUT;
+    Dprintf(("cm%d Vpos %.2f Vneg %.2f POL %d output %d cmxcon0=%x\n", cm+1, Vpos, Vneg, (bool)(cmxcon0 & CxPOL), output, cmxcon0));
+    value.put(cmxcon0);
+    m_cmModule->set_cmout(cm, output);
+    if (cmxcon0 & CxOE)
+    {
+        cm_source->putState(output?'1':'0');
+        m_cmModule->cmxcon1[cm]->output_pin()->updatePinModule();
+    }
+    if (old_out != output) // state change
+    {
+	// Positive going edge, set interrupt ? 
+	if (output && (m_cmModule->cmxcon1[cm]->value.get() & CMxCON1::CxINTP))
+	    m_cmModule->set_if(cm);
+
+	// Negative going edge, set interrupt ? 
+	if (!output && (m_cmModule->cmxcon1[cm]->value.get() & CMxCON1::CxINTN))
+	    m_cmModule->set_if(cm);
+    }
+    return cmxcon0;
+}
+
+
+CMxCON1::CMxCON1(Processor *pCpu, const char *pName, const char *pDesc, unsigned int _cm, ComparatorModule2 *cmModule)
+  : sfr_register(pCpu, pName, pDesc),
+  cm(_cm), m_cmModule(cmModule)
+{
+     cm_stimulus[NEG] = new CM_stimulus((CMCON *)m_cmModule->cmxcon0[cm], "cm_stimulus_-", 0, 1e12);
+     cm_stimulus[POS] = new CM_stimulus((CMCON *)m_cmModule->cmxcon0[cm], "cm_stimulus_+", 0, 1e12);
+     stimulus_pin[NEG] = 0;
+     stimulus_pin[POS] = 0;
+    for(int i = 0; i<5; i++)
+	cm_inputNeg[i] = 0;
+    for(int i = 0; i<2; i++)
+	cm_inputPos[i] = 0;
+    cm_output = 0;
+}
+
+CMxCON1::~CMxCON1()
+{
+    delete cm_stimulus[NEG];
+    delete cm_stimulus[POS];
+}
+double CMxCON1::get_Vneg()
+{
+    unsigned int cxNchan = value.get() & CxNMASK;
+    if (!stimulus_pin[NEG])
+	setPinStimulus(cm_inputNeg[cxNchan], NEG);
+    return cm_inputNeg[cxNchan]->getPin().get_nodeVoltage();
+}
+double CMxCON1::get_Vpos()
+{
+    unsigned int cxPchan = (value.get() & CxPMASK) >> 3;
+    double	Voltage;
+
+    switch(cxPchan)
+    {
+    case 0:
+	if (!stimulus_pin[POS])
+	    setPinStimulus(cm_inputPos[cxPchan], POS);
+	Voltage = cm_inputPos[cxPchan]->getPin().get_nodeVoltage();
+	break;
+
+    case 2:
+	Voltage = m_cmModule->DAC_voltage;
+	break;
+
+    case 4:
+	Voltage = m_cmModule->FVR_voltage;
+	break;
+
+    default:
+	printf("CMxCON1::get_Vpos unexpected Pchan %x\n", cxPchan);
+    case 6:
+	Voltage = 0.;
+	break;
+    }
+    return Voltage;
+
+}
+
+// Attach a stimulus to an input pin so that changes
+// in the pin voltage can be reflected in the comparator output.
+//
+// pin may be 0 in which case a current stimulus, if any, will be detached
+// pol is either the enum POS or NEG
+//
+void CMxCON1::setPinStimulus(PinModule *pin, bool pol)
+{
+	if (pin == stimulus_pin[pol]) return;
+
+	if (stimulus_pin[pol])
+	{
+            (stimulus_pin[pol]->getPin().snode)->detach_stimulus(cm_stimulus[pol]);
+	    stimulus_pin[pol] = 0;
+	}
+	if (pin && pin->getPin().snode)
+	{
+	    stimulus_pin[pol] = pin;
+
+            (stimulus_pin[pol]->getPin().snode)->attach_stimulus(cm_stimulus[pol]);
+	}
+}
+void CMxCON1::put(unsigned int new_value)
+{
+    unsigned int old_value = value.get();
+    new_value &= mValidBits;
+    unsigned int diff = old_value ^ new_value;
+    trace.raw(write_trace.get() | value.get());
+    value.put(new_value);
+
+    if ((diff & CxNMASK) || !stimulus_pin[NEG])
+    {
+        unsigned int cxNchan = new_value & CxNMASK;
+
+	setPinStimulus(cm_inputNeg[cxNchan], NEG);
+
+    }
+    if ((diff & CxPMASK) || !stimulus_pin[POS])
+    {
+	unsigned int cxPchan = (new_value & CxPMASK) >> 3;
+
+	if (cxPchan == 0)
+		setPinStimulus(cm_inputPos[cxPchan], POS);
+	else if (stimulus_pin[POS])
+		setPinStimulus(0, POS);
+    }
+    m_cmModule->run_get(cm);
+}
+
+void CMxCON1::set_OUTpin(PinModule *pin_cm0)
+{
+    cm_output = pin_cm0;
+}
+
+void CMxCON1::set_INpinNeg(PinModule *pin_cm0, PinModule *pin_cm1, PinModule *pin_cm2,  PinModule *pin_cm3,  PinModule *pin_cm4)
+{
+    cm_inputNeg[0] = pin_cm0;
+    cm_inputNeg[1] = pin_cm1;
+    cm_inputNeg[2] = pin_cm2;
+    cm_inputNeg[3] = pin_cm3;
+    cm_inputNeg[4] = pin_cm4;
+}
+void CMxCON1::set_INpinPos(PinModule *pin_cm0, PinModule *pin_cm1)
+{
+    cm_inputPos[0] = pin_cm0;
+    cm_inputPos[1] = pin_cm1;
+}
+
+ComparatorModule2::ComparatorModule2(Processor *pCpu)
+{
+    for(int i = 0; i < 4; i++)
+    {
+	cmxcon0[i] = 0;
+	cmxcon1[i] = 0;
+    }
+    cmout = 0;
+    t1gcon = 0;
+
+}
+ComparatorModule2::~ComparatorModule2()
+{
+    for(int i = 0; i < 4; i++)
+    {
+	if (cmxcon0[i])
+	    delete cmxcon0[i];
+	if (cmxcon1[i])
+	    delete cmxcon1[i];
+    }
+    if (cmout)
+	delete cmout;
+}
+
+// this function sets the bits in the CMOUT register and also
+// sends the state to the T1GCON class if t1gcon is defined
+//
+void ComparatorModule2::set_cmout(unsigned int bit, bool value)
+{
+    if (value)
+        cmout->value.put(cmout->value.get() | (1<<bit));
+    else
+        cmout->value.put(cmout->value.get() & ~(1<<bit));
+
+    switch(bit)
+    {
+    case 0:		//CM1
+    	if (t1gcon)
+    	    t1gcon->CM1_gate(value);
+    	if (sr_module)
+    	    sr_module->syncC1out(value);
+    	break;
+
+    case 1:		//CM2
+        if (t1gcon)
+	    t1gcon->CM2_gate(value);
+    	if (sr_module)
+    	    sr_module->syncC2out(value);
+    	break;
+
+    default:	//Do nonthing other CMs
+    	break;
+    }
+}
+
+void ComparatorModule2::set_DAC_volt(double _volt)
+{
+	DAC_voltage = _volt;
+	for (int i=0; i < 4; i++)
+	{
+	    if (cmxcon0[i]) cmxcon0[i]->get();
+	}
+}
+void ComparatorModule2::set_FVR_volt(double _volt)
+{
+	FVR_voltage = _volt;
+	for (int i=0; i < 4; i++)
+	{
+	    if (cmxcon0[i]) cmxcon0[i]->get();
+	}
+}
+// set interrupt for comparator cm
+void ComparatorModule2::set_if(unsigned int cm)
+{
+    switch(cm)
+    {
+    case 0:	
+	pir_set->set_c1if();
+	break;
+
+    case 1:	
+	pir_set->set_c2if();
+	break;
+
+    case 2:	
+	pir_set->set_c3if();
+	break;
+
+    case 3:	
+	pir_set->set_c4if();
+	break;
+
+    }
+}
