@@ -297,13 +297,15 @@ void gpsimInterface::simulation_has_stopped (void)
 }
 
 #if GLIB_MAJOR_VERSION >= 2
+// This code is not used anywhere, but uses currently depreciated calls RRR
+#ifdef GPSIM_THREAD
 
 GMutex *muRunMutex;
 GCond  *cvRunCondition;
 
 static Processor *tcpu=0;
 
-static void *run_thread( void *ptr )
+static void run_thread( void *ptr )
 {
   printf("run thread\n");
   while(1) {
@@ -321,7 +323,6 @@ static void *run_thread( void *ptr )
     g_mutex_unlock(muRunMutex);
 
   }
-
 }
 
 
@@ -348,7 +349,9 @@ void start_run_thread(void)
 
   std::cout << " started thread\n";
 }
-#endif
+#endif //GPSIM_THREAD
+#endif //GLIB_MAJOR_VERSION >= 2
+
 
 
 //========================================================================
