@@ -35,10 +35,12 @@ License along with this library; if not, see
 #include "xref.h"
 //#define DEBUG
 #if defined(DEBUG)
-#define Dprintf(arg) {printf("%s:%d ",__FILE__,__LINE__); printf arg; }
+#define Dprintf(arg) {printf("%s:%d-%s ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
 #else
 #define Dprintf(arg) {}
 #endif
+//#define D2printf(arg) {printf("%s:%d-%s ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
+#define D2printf(arg) {}
 
 //--------------------------------------------------
 // 
@@ -128,7 +130,7 @@ public:
   virtual void release()
   {
     if (verbose)
-        cout << "Deleting SignalSource\n";
+       cout << "Deleting SignalSource 0x" << hex << this << endl;
     delete this;
   }
 
@@ -177,7 +179,7 @@ PortRegister::PortRegister(Module *pCpu, const char *pName, const char *pDesc,
     drivingValue(0), rvDrivenValue(0,0)
 
 {
-  bit_mask = numIopins-1; // ugh.
+    mValidBits = (1<<numIopins)-1; // ugh.
 }
 
 void PortRegister::setEnableMask(unsigned int newEnableMask)
@@ -256,10 +258,15 @@ void PortRegister::updateUI()
 
 void PortRegister::setbit(unsigned int bit_number, char new3State)
 {
-  if(bit_number <= bit_mask) {
+    int set_mask = (1<<bit_number);
+  if(set_mask & mValidBits) {
 
+// The following traces just seem to muddy the water
+#ifdef RRR
     trace.raw(write_trace.get()  | value.data);
     trace.raw(write_trace.geti() | value.init);
+#endif
+
 
     Dprintf(("PortRegister::setbit() %s bit=%d,val=%c\n",name().c_str(), bit_number,new3State));
 
@@ -276,7 +283,7 @@ void PortRegister::setbit(unsigned int bit_number, char new3State)
     value = rvDrivenValue;
   } else {
     Dprintf(("PortRegister::::setbit() %s INVALID BIT bit=%d mask=0x%x\n",
-	     name().c_str(), bit_number, bit_mask));
+	     name().c_str(), bit_number, mValidBits));
 
   }
 
@@ -387,6 +394,10 @@ IOPIN *PortModule::addPin(IOPIN *new_pin, unsigned int iPinNumber)
       iopins[iPinNumber] = new PinModule(this,iPinNumber);
     iopins[iPinNumber]->setPin(new_pin);
   }
+  else
+  {
+	printf("PortModule::addPin ERROR pin %d > %d\n", iPinNumber, mNumIopins);
+  }
   return new_pin;
 }
 
@@ -438,18 +449,31 @@ PinModule::PinModule(PortModule *_port, unsigned int _pinNumber, IOPIN *_pin)
 
 PinModule::~PinModule()
 {
+  if (m_pin)
+    D2printf(("Pin %s sources active %p default %p\n", m_pin->name().c_str(), m_activeSource, m_defaultSource));
   if (m_activeSource && (m_activeSource != m_defaultSource)) {
     //cout << __FUNCTION__ << " deleting active source:"<<m_activeSource<<endl;
     //cout << "    state:" <<m_activeSource->getState() <<endl;
-    //m_activeSource->release();
+    m_activeSource->release();
+    m_activeSource = m_defaultSource;
   }
   if (m_defaultSource)
+  {
     m_defaultSource->release();
+    m_defaultSource = 0;
+  }
 
   if (m_activeControl && (m_activeControl != m_defaultControl))
+  {
     m_activeControl->release();
+    m_activeControl = m_defaultControl;
+  }
+  
   if (m_defaultControl)
+  {
     m_defaultControl->release();
+    m_defaultControl = 0;
+  }
 
   if (m_activePullupControl && (m_activePullupControl != m_defaultPullupControl))
     m_activePullupControl->release();
