@@ -63,12 +63,6 @@ public:
   Config188x(P16F88x *pCpu)
     : ConfigWord("CONFIG188x", 0x3fff, "Configuration Word", pCpu, 0x2007)
   {
-    if (m_pCpu) 
-    {
-	m_pCpu->wdt.initialize(true); // default WDT enabled
-        m_pCpu->wdt.set_timeout(0.000035);
-	m_pCpu->set_config_word(0x2007, 0x3fff);
-    }
   }
 
   enum {
@@ -169,7 +163,84 @@ P16F88x::P16F88x(const char *_name, const char *desc)
 
 P16F88x::~P16F88x()
 {
+  unassignMCLRPin();
+  delete_file_registers(0x20, 0x7f);
   delete_file_registers(0xa0, 0xbf);
+
+  remove_sfr_register(&tmr0);
+  remove_sfr_register(&intcon_reg);
+  remove_sfr_register(&pie2);
+  remove_sfr_register(&pie1);
+  remove_sfr_register(&tmr1l);
+  remove_sfr_register(&tmr1h);
+  remove_sfr_register(&pcon);
+  remove_sfr_register(&t1con);
+  remove_sfr_register(&tmr2);
+  remove_sfr_register(&t2con);
+  remove_sfr_register(&pr2);
+  remove_sfr_register(get_eeprom()->get_reg_eedata());
+  remove_sfr_register(get_eeprom()->get_reg_eeadr());
+  remove_sfr_register(get_eeprom()->get_reg_eedatah());
+  remove_sfr_register(get_eeprom()->get_reg_eeadrh());
+  remove_sfr_register(get_eeprom()->get_reg_eecon1());
+  remove_sfr_register(get_eeprom()->get_reg_eecon2());
+  delete get_eeprom();
+
+  remove_sfr_register(&intcon_reg);
+  remove_sfr_register(&osccon);
+  remove_sfr_register(&osctune);
+  remove_sfr_register(&usart.rcsta);
+  remove_sfr_register(&usart.txsta);
+  remove_sfr_register(&usart.spbrg);
+  remove_sfr_register(&usart.spbrgh);
+  remove_sfr_register(&usart.baudcon);
+  remove_sfr_register(&cm1con0);
+  remove_sfr_register(&cm2con0);
+  remove_sfr_register(&cm2con1);
+  remove_sfr_register(&vrcon);
+  remove_sfr_register(&srcon);
+  remove_sfr_register(&wdtcon);
+  remove_sfr_register(&ccpr2l);
+  remove_sfr_register(&ccpr2h);
+  remove_sfr_register(&ccp2con);
+  remove_sfr_register(&adresl);
+  remove_sfr_register(&adresh);
+  remove_sfr_register(&ansel);
+  remove_sfr_register(&anselh);
+  remove_sfr_register(&adcon0);
+  remove_sfr_register(&adcon1);
+  remove_sfr_register(&ccpr1l);
+  remove_sfr_register(&ccpr1h);
+  remove_sfr_register(&ccp1con);
+  remove_sfr_register(&ccpr2l);
+  remove_sfr_register(&ccpr2h);
+  remove_sfr_register(&ccp2con);
+  remove_sfr_register(&pwm1con);
+  remove_sfr_register(&pstrcon);
+  remove_sfr_register(&eccpas);
+  remove_sfr_register(&ssp.sspcon2);
+  if (hasSSP()) {
+    remove_sfr_register(&ssp.sspbuf);
+    remove_sfr_register(&ssp.sspcon);
+    remove_sfr_register(&ssp.sspadd);
+    remove_sfr_register(&ssp.sspstat);
+  }
+  delete_sfr_register(usart.txreg);
+  delete_sfr_register(usart.rcreg);
+  delete_sfr_register(m_porta);
+  delete_sfr_register(m_trisa);
+  delete_sfr_register(m_portb);
+  delete_sfr_register(m_trisb);
+  delete_sfr_register(m_porte);
+  delete_sfr_register(m_trise);
+  delete_sfr_register(m_portc);
+  delete_sfr_register(m_trisc);
+
+  delete_sfr_register(pir1);
+  delete_sfr_register(pir2);
+  delete_sfr_register(m_wpu);
+  delete_sfr_register(m_ioc);
+
 }
 
 void P16F88x::create_iopin_map()
@@ -199,7 +270,7 @@ void P16F88x::create_sfr_map()
   add_sfr_register(pclath,  0x0a, RegisterValue(0,0));
 
   add_sfr_register(&intcon_reg, 0x0b, RegisterValue(0,0));
-  alias_file_registers(0x0a,0x0b,0x80);
+  //alias_file_registers(0x0a,0x0b,0x80); //Already donw
 
   intcon = &intcon_reg;
 
@@ -228,13 +299,7 @@ void P16F88x::create_sfr_map()
   pie2.setPir(get_pir2());
   alias_file_registers(0x00,0x04,0x100);
   alias_file_registers(0x80,0x84,0x100);
-
-  add_sfr_register(m_porta, 0x05);
-  add_sfr_register(m_trisa, 0x85, RegisterValue(0xff,0));
-
-  add_sfr_register(m_portb, 0x06);
   alias_file_registers(0x06,0x06,0x100);
-  add_sfr_register(m_trisb, 0x86, RegisterValue(0xff,0));
   alias_file_registers(0x86,0x86,0x100);
 
   add_sfr_register(pir1,   0x0c, RegisterValue(0,0),"pir1");
@@ -258,15 +323,12 @@ void P16F88x::create_sfr_map()
   add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x18c, RegisterValue(0,0));
   add_sfr_register(get_eeprom()->get_reg_eecon2(),  0x18d);
 
-  add_sfr_register(&intcon_reg, 0x00b, RegisterValue(0,0));
-
   alias_file_registers(0x0a,0x0b,0x080);
   alias_file_registers(0x0a,0x0b,0x100);
   alias_file_registers(0x0a,0x0b,0x180);
 
 
 
-  intcon = &intcon_reg;
   intcon_reg.set_pir_set(get_pir_set());
 
 
@@ -304,10 +366,6 @@ void P16F88x::create_sfr_map()
 
   add_sfr_register(&wdtcon, 0x105, RegisterValue(0x08,0),"wdtcon");
 
-  add_sfr_register(&ccpr2l, 0x1b, RegisterValue(0,0));
-  add_sfr_register(&ccpr2h, 0x1c, RegisterValue(0,0));
-  add_sfr_register(&ccp2con, 0x1d, RegisterValue(0,0));
-
   add_sfr_register(&adresl,  0x9e, RegisterValue(0,0));
   add_sfr_register(&adresh,  0x1e, RegisterValue(0,0));
                                                                                 
@@ -439,7 +497,7 @@ void P16F88x::create_symbols()
     cout << "88x create symbols\n";
 
   pic_processor::create_symbols();
-  addSymbol(W);
+  addSymbol(Wreg);
 }
 
 void P16F88x::set_out_of_range_pm(unsigned int address, unsigned int value)
@@ -548,6 +606,9 @@ void P16F88x::create_config_memory()
   m_configMemory = new ConfigMemory(this,2);
   m_configMemory->addConfigWord(0,new Config188x(this));
   m_configMemory->addConfigWord(1,new ConfigWord("CONFIG2", 0,"Configuration Word",this,0x2008));
+  wdt.initialize(true); // default WDT enabled
+  wdt.set_timeout(0.000035);
+  set_config_word(0x2007, 0x3fff);
 
 }
 
@@ -696,11 +757,13 @@ P16F883::P16F883(const char *_name, const char *desc)
     cout << "f883 constructor, type = " << isa() << '\n';
 
   m_porta->setEnableMask(0xff);
-
-
-
 }
 
+P16F883::~P16F883()
+{
+  delete_file_registers(0xc0,0xef);
+  delete_file_registers(0x120,0x16f);
+}
 void P16F883::create_symbols(void)
 {
 
@@ -749,11 +812,14 @@ P16F886::P16F886(const char *_name, const char *desc)
     cout << "f886 constructor, type = " << isa() << '\n';
 
   m_porta->setEnableMask(0xff);
-
-
-
 }
 
+P16F886::~P16F886()
+{
+  delete_file_registers(0xc0,0xef);
+  delete_file_registers(0x120,0x16f);
+  delete_file_registers(0x190,0x1ef);
+}
 void P16F886::create_symbols(void)
 {
 
@@ -803,6 +869,12 @@ P16F887::P16F887(const char *_name, const char *desc)
     cout << "f887 constructor, type = " << isa() << '\n';
 
 }
+
+P16F887::~P16F887()
+{
+  delete_file_registers(0x110,0x11f);
+  delete_file_registers(0x190,0x1ef);
+}
 void P16F887::create_symbols(void)
 {
 
@@ -818,8 +890,9 @@ void P16F887::create_symbols(void)
 void P16F887::create_sfr_map()
 {
 
-  add_file_registers(0xb0,0xef,0);
+  add_file_registers(0xc0,0xef,0);
   add_file_registers(0x110,0x16f,0);
+  //add_file_registers(0x110,0x11f,0);
   add_file_registers(0x190,0x1ef,0);
 
 
@@ -865,7 +938,14 @@ P16F884::P16F884(const char *_name, const char *desc)
 
   m_portd = new PicPSP_PortRegister(this,"portd","",8,0xff);
   m_trisd = new PicTrisRegister(this,"trisd","",(PicPortRegister *)m_portd, false);
+}
+P16F884::~P16F884()
+{
+  delete_file_registers(0xc0,0xef);
+  delete_file_registers(0x120,0x16f);
 
+  delete_sfr_register(m_portd);
+  delete_sfr_register(m_trisd);
 }
 
 //------------------------------------------------------------------------
@@ -967,13 +1047,7 @@ public:
   ConfigF631(P16F631 *pCpu)
     : ConfigWord("CONFIG", 0x3fff, "Configuration Word", pCpu, 0x2007)
   {
-    //Dprintf(("ConfigF631::ConfigF631 %p\n", m_pCpu));
-    if (m_pCpu)
-    {
-	m_pCpu->wdt.initialize(true); // default WDT enabled
-        m_pCpu->wdt.set_timeout(0.000035);
-        m_pCpu->set_config_word(0x2007, 0x3fff);
-    }
+    Dprintf(("ConfigF631::ConfigF631 %p\n", m_pCpu));
   }
 
     enum {
@@ -1063,7 +1137,7 @@ P16F631::P16F631(const char *_name, const char *desc)
   m_iocb = new IOC(this, "iocb", "Interrupt-On-Change GPIO Register");
 
   m_porta = new PicPortGRegister(this,"porta","",&intcon_reg, m_ioca, 8,0x3f);
-  m_trisa = new PicTrisRegister(this,"trisa","", m_porta, false);
+  m_trisa = new PicTrisRegister(this,"trisa","", m_porta, false, 0x37);
 
   m_portb = new PicPortGRegister(this,"portb","",&intcon_reg, m_iocb, 8,0xf0);
   m_trisb = new PicTrisRegister(this,"trisb","", m_portb, false);
@@ -1073,7 +1147,7 @@ P16F631::P16F631(const char *_name, const char *desc)
   tmr0.set_cpu(this, m_porta, 4, option_reg);
   tmr0.start(0);
 
-  m_portc = new PicPortRegister(this,"portc","",8,0x3f);
+  m_portc = new PicPortRegister(this,"portc","",8,0xff);
   m_trisc = new PicTrisRegister(this,"trisc","", m_portc, false);
 
 
@@ -1084,6 +1158,30 @@ P16F631::~P16F631()
   if (verbose)
     cout << __FUNCTION__ << endl;
 
+  unassignMCLRPin();
+  delete_file_registers(0x40, 0x7f);
+  remove_sfr_register(get_eeprom()->get_reg_eedata());
+  remove_sfr_register(get_eeprom()->get_reg_eeadr());
+  remove_sfr_register(get_eeprom()->get_reg_eecon1());
+  remove_sfr_register(get_eeprom()->get_reg_eecon2());
+  remove_sfr_register(&tmr0);
+  remove_sfr_register(&vrcon);
+  remove_sfr_register(&cm1con0);
+  remove_sfr_register(&cm2con0);
+  remove_sfr_register(&cm2con1);
+  remove_sfr_register(&ansel);
+  remove_sfr_register(&srcon);
+  remove_sfr_register(&tmr1l);
+  remove_sfr_register(&tmr1h);
+  remove_sfr_register(&t1con);
+  remove_sfr_register(&pcon);
+  remove_sfr_register(&wdtcon);
+  remove_sfr_register(&osccon);
+  remove_sfr_register(&pie1);
+  remove_sfr_register(&pie2);
+  remove_sfr_register(&intcon_reg);
+  remove_sfr_register(&osctune);
+  delete_sfr_register(pir2);
   delete_sfr_register(m_portc);
   delete_sfr_register(m_trisc);
 
@@ -1172,7 +1270,7 @@ void P16F631::create(int eesize)
 void P16F631::create_symbols()
 {
   pic_processor::create_symbols();
-  addSymbol(W);
+  addSymbol(Wreg);
 
 }
 
@@ -1305,6 +1403,9 @@ void P16F631::create_config_memory()
 {
   m_configMemory = new ConfigMemory(this,1);
   m_configMemory->addConfigWord(0,new ConfigF631(this));
+  wdt.initialize(true); // default WDT enabled
+  wdt.set_timeout(0.000035);
+  set_config_word(0x2007, 0x3fff);
 
 };
 
@@ -1320,6 +1421,445 @@ bool P16F631::set_config_word(unsigned int address, unsigned int cfg_word)
   };
 
   
+   if(address == config_word_address())
+    {
+       unsigned int valid_pins = m_porta->getEnableMask();
+
+        if ((cfg_word & CFG_MCLRE) == CFG_MCLRE)
+	{
+            assignMCLRPin(4);  
+	}
+        else
+	{
+            unassignMCLRPin();
+	}
+
+        wdt.initialize((cfg_word & CFG_WDTE) == CFG_WDTE);
+
+
+       set_int_osc(false);
+
+	// AnalogReq is used so ADC does not change clock names
+	// set_config_word is first called with default and then
+	// often called a second time. the following call is to
+	// reset porta so next call to AnalogReq sill set the pin name
+	//
+        (&(*m_porta)[4])->AnalogReq((Register *)this, false, "porta4");
+	valid_pins |= 0x20;
+       	switch(cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) 
+	{
+
+       	case 0:  // LP oscillator: low power crystal is on RA4 and RA5
+       	case 1:     // XT oscillator: crystal/resonator is on RA4 and RA5
+       	case 2:     // HS oscillator: crystal/resonator is on RA4 and RA5
+            (&(*m_porta)[4])->AnalogReq((Register *)this, true, "OSC2");
+            (m_porta->getPin(5))->newGUIname("OSC1");
+	    valid_pins &= 0xcf;
+            break;
+
+	case 3:	// EC I/O on RA4 pin, CLKIN on RA5
+            (m_porta->getPin(5))->newGUIname("CLKIN");
+	    valid_pins &= 0xef;
+            break;
+
+	    
+	case 5: // INTOSC CLKOUT on RA4 pin
+            (&(*m_porta)[4])->AnalogReq((Register *)this, true, "CLKOUT");
+	case 4: // INTOSC
+            (m_porta->getPin(5))->newGUIname("porta5");
+             set_int_osc(true);
+	    osccon.set_rc_frequency();
+	    break;
+
+	case 6: //RC oscillator: I/O on RA4 pin, RC on RA5
+            (m_porta->getPin(5))->newGUIname("RC");
+	    valid_pins &= 0xdf;
+            break;
+
+	case 7: // RC oscillator: CLKOUT on RA4 pin, RC on RA5
+            (&(*m_porta)[4])->AnalogReq((Register *)this, true, "CLKOUT");
+            (m_porta->getPin(5))->newGUIname("RC");
+	    valid_pins &= 0xdf;
+            break;
+	};
+
+	if (valid_pins != m_porta->getEnableMask()) // enable new pins for IO
+    	{
+            m_porta->setEnableMask(valid_pins);
+	    m_trisa->setEnableMask(valid_pins & 0xf7);
+    	}
+	return(true);
+
+    }
+    return false;
+}
+
+//========================================================================
+//
+// Pic 16F684
+//
+P16F684::P16F684(const char *_name, const char *desc)
+   : _14bit_processor(_name, desc),
+    comparator(this),
+    t1con(this, "t1con", "TMR1 Control"),
+    t2con(this, "t2con", "TMR2 Control"),
+    pie1(this,"pie1", "Peripheral Interrupt Enable"),
+    pr2(this, "pr2", "TMR2 Period Register"),
+    tmr2(this, "tmr2", "TMR2 Register"),
+    tmr1l(this, "tmr1l", "TMR1 Low"),
+    tmr1h(this, "tmr1h", "TMR1 High"),
+    osctune(this, "osctune", "OSC Tune"),
+    pcon(this, "pcon", "pcon"),
+    wdtcon(this, "wdtcon", "WDT Control", 0x1f),
+    osccon(this, "osccon", "OSC Control"),
+    ansel(this,"ansel", "Analog Select"),
+    adcon0(this,"adcon0", "A2D Control 0"),
+    adcon1(this,"adcon1", "A2D Control 1"),
+    adresh(this,"adresh", "A2D Result High"),
+    adresl(this,"adresl", "A2D Result Low"),
+    ccp1con(this, "ccp1con", "Capture Compare Control"),
+    ccpr1l(this, "ccpr1l", "Capture Compare 1 Low"),
+    ccpr1h(this, "ccpr1h", "Capture Compare 1 High"),
+    eccpas(this, "eccpas", "ECCP Auto-Shutdown Control Register"),
+    pwm1con(this, "pwm1con", "Enhanced PWM Control Register"),
+    pstrcon(this, "pstrcon", "Pulse Sterring Control Register"),
+    intcon_reg(this,"intcon","Interrupt Control")
+{
+  if(verbose)
+    cout << "P16F684 constructor, type = " << isa() << '\n';
+
+    pir1_3_reg = new PIR1v3(this,"pir1","Peripheral Interrupt Register",&intcon_reg,&pie1);
+  pir1 = pir1_3_reg;
+  pir1->valid_bits = pir1->writable_bits = 0xff;
+
+
+  m_ioca = new IOC(this, "ioca", "Interrupt-On-Change GPIO Register");
+
+  m_porta = new PicPortGRegister(this,"porta","",&intcon_reg, m_ioca, 8,0x3f);
+  m_trisa = new PicTrisRegister(this,"trisa","", m_porta, false);
+
+
+  m_wpua = new WPU(this, "wpua", "Weak Pull-up Register", m_porta, 0x37);
+  tmr0.set_cpu(this, m_porta, 4, option_reg);
+  tmr0.start(0);
+
+  m_portc = new PicPortRegister(this,"portc","",8,0x3f);
+  m_trisc = new PicTrisRegister(this,"trisc","", m_portc, false);
+
+
+}
+
+P16F684::~P16F684()
+{
+  if (verbose)
+    cout << __FUNCTION__ << endl;
+
+  unassignMCLRPin();
+
+  delete_file_registers(0x20, 0x7f);
+  delete_file_registers(0xa0, 0xbf);
+
+  remove_sfr_register(&tmr0);
+
+  remove_sfr_register(&intcon_reg);
+  remove_sfr_register(pir1);
+  remove_sfr_register(&tmr1l);
+  remove_sfr_register(&tmr1h);
+  remove_sfr_register(&t1con);
+  remove_sfr_register(&tmr2);
+  remove_sfr_register(&t2con);
+  remove_sfr_register(&ccpr1l);
+  remove_sfr_register(&ccpr1h);
+  remove_sfr_register(&ccp1con);
+  remove_sfr_register(&pwm1con);
+  remove_sfr_register(&eccpas);
+  remove_sfr_register(&wdtcon);
+  remove_sfr_register(&comparator.cmcon);
+  remove_sfr_register(&comparator.cmcon1);
+  remove_sfr_register(&adresh);
+  remove_sfr_register(&adcon0);
+  remove_sfr_register(&pie1);
+  remove_sfr_register(&pcon);
+  remove_sfr_register(&osccon);
+  remove_sfr_register(&osctune);
+  remove_sfr_register(&ansel);
+  remove_sfr_register(&pr2);
+  remove_sfr_register(&comparator.vrcon);
+  remove_sfr_register(get_eeprom()->get_reg_eedata());
+  remove_sfr_register(get_eeprom()->get_reg_eeadr());
+  remove_sfr_register(get_eeprom()->get_reg_eecon1());
+  remove_sfr_register(get_eeprom()->get_reg_eecon2());
+  remove_sfr_register(&adresl);
+  remove_sfr_register(&adcon1);
+
+  delete_sfr_register(m_portc);
+  delete_sfr_register(m_trisc);
+
+  delete_sfr_register(m_porta);
+  delete_sfr_register(m_trisa);
+  delete_sfr_register(m_ioca);
+  delete_sfr_register(m_wpua);
+  delete_sfr_register(pir1_3_reg);
+  delete e;
+
+}
+void P16F684::create_iopin_map(void)
+{
+  package = new Package(14);
+  if(!package)
+    return;
+
+  package->assign_pin(1, 0);	// Vdd
+
+  package->assign_pin( 2, m_porta->addPin(new IO_bi_directional_pu("porta5"),5));
+  package->assign_pin( 3, m_porta->addPin(new IO_bi_directional_pu("porta4"),4));
+  package->assign_pin( 4, m_porta->addPin(new IOPIN("porta3"),3));
+  package->assign_pin( 5, m_portc->addPin(new IO_bi_directional_pu("portc5"),5));
+  package->assign_pin( 6, m_portc->addPin(new IO_bi_directional("portc4"),4));
+  package->assign_pin( 7, m_portc->addPin(new IO_bi_directional("portc3"),3));
+
+  package->assign_pin( 8, m_portc->addPin(new IO_bi_directional("portc2"),2));
+  package->assign_pin( 9, m_portc->addPin(new IO_bi_directional("portc1"),1));
+  package->assign_pin(10, m_portc->addPin(new IO_bi_directional("portc0"),0));
+
+  package->assign_pin(11, m_porta->addPin(new IO_bi_directional_pu("porta2"),2));
+  package->assign_pin(12, m_porta->addPin(new IO_bi_directional_pu("porta1"),1));
+  package->assign_pin(13, m_porta->addPin(new IO_bi_directional_pu("porta0"),0));
+
+  package->assign_pin(14, 0); //VSS
+
+
+  tmr1l.setIOpin(&(*m_portc)[0]);
+
+}
+Processor * P16F684::construct(const char *name)
+{
+
+  P16F684 *p = new P16F684(name);
+
+  if(verbose)
+    cout << " P16F684 construct\n";
+
+  p->create(256);
+  p->create_invalid_registers ();
+  p->create_symbols();
+  return p;
+
+}
+
+void P16F684::create(int eesize)
+{
+   create_iopin_map();
+
+   _14bit_processor::create();
+
+   e = new EEPROM_WIDE(this,pir1);
+   e->initialize(eesize);
+   e->set_intcon(&intcon_reg);
+   set_eeprom_wide(e);
+
+
+  status->rp_mask = 0x60;  // rp0 and rp1 are valid.
+  indf->base_address_mask1 = 0x80; // used for indirect accesses above 0x100
+  indf->base_address_mask2 = 0x1ff; // used for indirect accesses above 0x100
+
+
+   P16F684::create_sfr_map();
+}
+
+void P16F684::create_symbols()
+{
+  pic_processor::create_symbols();
+  addSymbol(Wreg);
+
+}
+
+//-------------------------------------------------------------------
+void P16F684::create_sfr_map()
+{
+
+  pir_set_def.set_pir1(pir1);
+
+  add_file_registers(0x20, 0x7f, 0);
+  add_file_registers(0xa0, 0xbf, 0);
+  alias_file_registers(0x70, 0x7f, 0x80);
+
+  add_sfr_register(indf,    0x00);
+  alias_file_registers(0x00,0x00,0x80);
+
+  add_sfr_register(&tmr0,   0x01);
+  add_sfr_register(option_reg,  0x81, RegisterValue(0xff,0));
+
+  add_sfr_register(pcl,     0x02, RegisterValue(0,0));
+  add_sfr_register(status,  0x03, RegisterValue(0x18,0));
+  add_sfr_register(fsr,     0x04);
+  alias_file_registers(0x02,0x04,0x80);
+
+  add_sfr_register(m_porta, 0x05);
+  add_sfr_register(m_trisa, 0x85, RegisterValue(0x3f,0));
+
+  add_sfr_register(m_portc, 0x07);
+  add_sfr_register(m_trisc, 0x87, RegisterValue(0xff,0));
+
+  add_sfr_register(pclath,  0x0a, RegisterValue(0,0));
+  add_sfr_register(&intcon_reg, 0x00b, RegisterValue(0,0));
+
+  alias_file_registers(0x0a,0x0b,0x80);
+  add_sfr_register(pir1, 0x0c, RegisterValue(0,0));
+  add_sfr_register(&tmr1l, 0x0e, RegisterValue(0,0), "tmr1l");
+  add_sfr_register(&tmr1h, 0x0f, RegisterValue(0,0), "tmr1h");
+  add_sfr_register(&t1con, 0x10, RegisterValue(0,0));
+  add_sfr_register(&tmr2, 0x11, RegisterValue(0,0));
+  add_sfr_register(&t2con, 0x12, RegisterValue(0,0));
+  add_sfr_register(&ccpr1l, 0x13, RegisterValue(0,0));
+  add_sfr_register(&ccpr1h, 0x14, RegisterValue(0,0));
+  add_sfr_register(&ccp1con, 0x15, RegisterValue(0,0));
+  add_sfr_register(&pwm1con, 0x16, RegisterValue(0,0));
+  add_sfr_register(&eccpas, 0x17, RegisterValue(0,0));
+  add_sfr_register(&wdtcon, 0x18, RegisterValue(0x08,0));
+  add_sfr_register(&comparator.cmcon, 0x19, RegisterValue(0,0), "cmcon0");
+  add_sfr_register(&comparator.cmcon1, 0x1a, RegisterValue(0,0), "cmcon1");
+  add_sfr_register(&adresh,  0x1e, RegisterValue(0,0));
+  add_sfr_register(&adcon0, 0x1f, RegisterValue(0,0));
+
+  add_sfr_register(&pie1,   0x8c, RegisterValue(0,0));
+  add_sfr_register(&pcon, 0x8e, RegisterValue(0,0));
+  add_sfr_register(&osccon, 0x8f, RegisterValue(0,0));
+
+  add_sfr_register(&osctune, 0x90, RegisterValue(0,0),"osctune");
+  add_sfr_register(&ansel, 0x91, RegisterValue(0xff,0));
+  add_sfr_register(&pr2,    0x92, RegisterValue(0xff,0));
+  add_sfr_register(m_wpua, 0x95, RegisterValue(0x37,0),"wpua");
+  add_sfr_register(m_ioca, 0x96, RegisterValue(0,0),"ioca");
+  add_sfr_register(&comparator.vrcon, 0x99, RegisterValue(0,0),"vrcon");
+  add_sfr_register(get_eeprom()->get_reg_eedata(),  0x9a);
+  add_sfr_register(get_eeprom()->get_reg_eeadr(),   0x9b);
+  add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x9c, RegisterValue(0,0));
+  add_sfr_register(get_eeprom()->get_reg_eecon2(),  0x9d);
+  add_sfr_register(&adresl,  0x9e, RegisterValue(0,0));
+  add_sfr_register(&adcon1, 0x9f, RegisterValue(0,0));
+
+  ansel.setAdcon1(&adcon1);
+  ansel.setValidBits(0xff);
+
+  // Link the comparator and voltage ref to porta
+  comparator.initialize(&pir_set_def, NULL, 
+	&(*m_porta)[0], &(*m_porta)[1], 
+	&(*m_portc)[0], &(*m_portc)[1], 
+	&(*m_porta)[2], &(*m_portc)[4]);
+
+  comparator.cmcon.set_tmrl(&tmr1l);
+  comparator.cmcon1.set_tmrl(&tmr1l);
+
+  comparator.cmcon.set_configuration(1, 0, AN0, AN1, AN0, AN1, ZERO);
+  comparator.cmcon.set_configuration(2, 0, AN2, AN3, AN2, AN3, ZERO);
+  comparator.cmcon.set_configuration(1, 1, AN0, AN3, AN1, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(2, 1, AN2, AN3, AN2, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 2, AN1, VREF, AN0, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(2, 2, AN3, VREF, AN2, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(1, 3, AN0, AN3, AN0, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(2, 3, AN2, AN3, AN2, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 4, AN0, AN1, AN0, AN1, NO_OUT);
+  comparator.cmcon.set_configuration(2, 4, AN2, AN3, AN2, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 5, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(2, 5, AN2, AN3, AN2, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 6, AN0, AN3, AN0, AN3, OUT0);
+  comparator.cmcon.set_configuration(2, 6, AN2, AN3, AN2, AN3, OUT1);
+  comparator.cmcon.set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.vrcon.setValidBits(0xaf); 
+
+  adcon0.setAdresLow(&adresl);
+  adcon0.setAdres(&adresh);
+  adcon0.setAdcon1(&adcon1);
+  adcon0.setIntcon(&intcon_reg);
+  adcon0.setA2DBits(10);
+  adcon0.setPir(pir1);
+  adcon0.setChannel_Mask(7);
+  adcon0.setChannel_shift(2);
+
+  adcon1.setAdcon0(&adcon0);	// VCFG0, VCFG1 in adcon0
+  adcon1.setNumberOfChannels(8);
+  adcon1.setIOPin(0, &(*m_porta)[0]);
+  adcon1.setIOPin(1, &(*m_porta)[1]);
+  adcon1.setIOPin(2, &(*m_porta)[2]);
+  adcon1.setIOPin(3, &(*m_porta)[4]);
+  adcon1.setIOPin(4, &(*m_portc)[0]);
+  adcon1.setIOPin(5, &(*m_portc)[1]);
+  adcon1.setIOPin(6, &(*m_portc)[2]);
+  adcon1.setIOPin(7, &(*m_portc)[3]);
+  adcon1.setVrefHiConfiguration(2, 1);
+  intcon = &intcon_reg;
+  intcon_reg.set_pir_set(get_pir_set());
+
+  tmr1l.tmrh = &tmr1h;
+  tmr1l.t1con = &t1con;
+  // FIXME -- can't delete this new'd item
+  tmr1l.setInterruptSource(new InterruptSource(pir1, PIR1v3::TMR1IF));
+  tmr1h.tmrl  = &tmr1l;
+  t1con.tmrl  = &tmr1l;
+
+  tmr1l.setIOpin(&(*m_porta)[5]);
+  tmr1l.setGatepin(&(*m_porta)[4]);
+
+  if (pir1) {
+    pir1->set_intcon(&intcon_reg);
+    pir1->set_pie(&pie1);
+  }
+  pie1.setPir(pir1);
+
+  t2con.tmr2  = &tmr2;
+  tmr2.pir_set   = get_pir_set();
+  tmr2.pr2    = &pr2;
+  tmr2.t2con  = &t2con;
+  tmr2.add_ccp ( &ccp1con );
+  pr2.tmr2    = &tmr2;
+
+  eccpas.setIOpin(0, 0, &(*m_portc)[5]);
+  eccpas.link_registers(&pwm1con, &ccp1con);
+
+  ccp1con.setIOpin(&(*m_portc)[5], &(*m_portc)[4], &(*m_portc)[3], &(*m_portc)[2]);
+  ccp1con.setBitMask(0xff);
+  ccp1con.pstrcon = &pstrcon;
+  ccp1con.pwm1con = &pwm1con;
+  ccp1con.setCrosslinks(&ccpr1l, pir1, PIR1v2::CCP1IF, &tmr2, &eccpas);
+  ccpr1l.ccprh  = &ccpr1h;
+  ccpr1l.tmrl   = &tmr1l;
+  ccpr1h.ccprl  = &ccpr1l;
+
+  osccon.set_osctune(&osctune);
+  osctune.set_osccon(&osccon);
+
+}
+//-------------------------------------------------------------------
+void P16F684::option_new_bits_6_7(unsigned int bits)
+{
+  m_wpua->set_wpu_pu( (bits & OPTION_REG::BIT7) != OPTION_REG::BIT7);
+  m_porta->setIntEdge((bits & OPTION_REG::BIT6) == OPTION_REG::BIT6);
+}
+//-------------------------------------------------------------------
+void P16F684::create_config_memory()
+{
+  m_configMemory = new ConfigMemory(this,1);
+  m_configMemory->addConfigWord(0,new ConfigF631((P16F631*)this));
+  wdt.initialize(true); // default WDT enabled
+  wdt.set_timeout(0.000035);
+  set_config_word(0x2007, 0x3fff);
+
+};
+
+//-------------------------------------------------------------------
+bool P16F684::set_config_word(unsigned int address, unsigned int cfg_word)
+{
+  enum {
+    CFG_FOSC0 = 1<<0,
+    CFG_FOSC1 = 1<<1,
+    CFG_FOSC2 = 1<<2,
+    CFG_WDTE  = 1<<3,
+    CFG_MCLRE = 1<<5,
+  };
+
+  
    if(address == config_word_address())
     {
        unsigned int valid_pins = m_porta->getEnableMask();
@@ -1392,7 +1932,6 @@ bool P16F631::set_config_word(unsigned int address, unsigned int cfg_word)
     }
     return false;
 }
-
 //========================================================================
 //
 // Pic 16F677 
@@ -1404,6 +1943,7 @@ Processor * P16F677::construct(const char *name)
   P16F677 *p = new P16F677(name);
 
   p->create(256);
+  p->set_hasSSP();
   p->create_sfr_map();
   p->create_invalid_registers ();
   p->create_symbols();
@@ -1424,6 +1964,28 @@ P16F677::P16F677(const char *_name, const char *desc)
 
 }
 
+P16F677::~P16F677()
+{
+  delete_file_registers(0x20,0x3f);
+  delete_file_registers(0xa0,0xbf);
+
+  remove_sfr_register(&anselh);
+
+
+  if (hasSSP()) {
+    remove_sfr_register(&ssp.sspbuf);
+    remove_sfr_register(&ssp.sspcon);
+    remove_sfr_register(&ssp.sspadd);
+    remove_sfr_register(&ssp.sspstat);
+  }
+  remove_sfr_register(&adresl);
+  remove_sfr_register(&adresh);
+  remove_sfr_register(&adcon0);
+  remove_sfr_register(&adcon1);
+  delete m_cvref;
+  delete m_v06ref;
+}
+
 void P16F677::create_symbols(void)
 {
 
@@ -1476,10 +2038,6 @@ void P16F677::create_sfr_map()
   // set a2d modes where an1 is Vref+ 
   adcon1.setVrefHiConfiguration(2, 1);
 
-  add_sfr_register(get_eeprom()->get_reg_eedata(),  0x10c);
-  add_sfr_register(get_eeprom()->get_reg_eeadr(),   0x10d);
-  add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x18c, RegisterValue(0,0));
-  add_sfr_register(get_eeprom()->get_reg_eecon2(),  0x18d);
   add_sfr_register(&anselh, 0x11f, RegisterValue(0x0f,0));
   add_file_registers(0x20,0x3f,0);
   add_file_registers(0xa0,0xbf,0);
@@ -1545,9 +2103,25 @@ P16F685::P16F685(const char *_name, const char *desc)
   if(verbose)
     cout << "f685 constructor, type = " << isa() << '\n';
 
+  set_hasSSP();
 
 }
 
+P16F685::~P16F685()
+{
+  delete_file_registers(0xc0,0xef);
+  delete_file_registers(0x120,0x16f);
+  remove_sfr_register(&pstrcon);
+  remove_sfr_register(&tmr2);
+  remove_sfr_register(&t2con);
+  remove_sfr_register(&pr2);
+  remove_sfr_register(&ccpr1l);
+  remove_sfr_register(&ccpr1h);
+  remove_sfr_register(&ccp1con);
+  remove_sfr_register(&pwm1con);
+  remove_sfr_register(&eccpas);
+}
+
 void P16F685::create_symbols(void)
 {
 
@@ -1596,8 +2170,9 @@ void P16F685::create_sfr_map()
 
   add_sfr_register(&pwm1con, 0x1c, RegisterValue(0,0));
   add_sfr_register(&eccpas, 0x1d, RegisterValue(0,0));
-  add_file_registers(0x20,0x3f,0);
-  add_file_registers(0xa0,0xef,0);
+//  add_file_registers(0x20,0x3f,0);
+//  add_file_registers(0xa0,0xef,0);
+  add_file_registers(0xc0,0xef,0);
   add_file_registers(0x120,0x16f,0);
 
 
@@ -1631,10 +2206,20 @@ P16F687::P16F687(const char *_name, const char *desc)
 
   if(verbose)
     cout << "f687 constructor, type = " << isa() << '\n';
-
+  set_hasSSP();
 
 }
 
+P16F687::~P16F687()
+{
+  remove_sfr_register(&usart.rcsta);
+  remove_sfr_register(&usart.txsta);
+  remove_sfr_register(&usart.spbrg);
+  remove_sfr_register(&usart.spbrgh);
+  remove_sfr_register(&usart.baudcon);
+  delete_sfr_register(usart.txreg);
+  delete_sfr_register(usart.rcreg);
+}
 void P16F687::create_symbols(void)
 {
 
@@ -1654,8 +2239,8 @@ void P16F687::create_sfr_map()
   add_sfr_register(get_eeprom()->get_reg_eeadrh(),   0x10f);
 
 
-  add_file_registers(0x20,0x3f,0);
-  add_file_registers(0xa0,0xbf,0);
+//  add_file_registers(0x20,0x3f,0);
+//  add_file_registers(0xa0,0xbf,0);
 
   usart.initialize(pir1,&(*m_portb)[7], &(*m_portb)[5],
 		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
@@ -1697,6 +2282,7 @@ P16F689::P16F689(const char *_name, const char *desc)
   if(verbose)
     cout << "f689 constructor, type = " << isa() << '\n';
 
+  set_hasSSP();
 
 }
 
@@ -1730,9 +2316,21 @@ P16F690::P16F690(const char *_name, const char *desc)
   if(verbose)
     cout << "f690 constructor, type = " << isa() << '\n';
 
+  set_hasSSP();
 
 }
 
+P16F690::~P16F690()
+{
+  remove_sfr_register(&usart.rcsta);
+  remove_sfr_register(&usart.txsta);
+  remove_sfr_register(&usart.spbrg);
+  remove_sfr_register(&usart.spbrgh);
+  remove_sfr_register(&usart.baudcon);
+  delete_sfr_register(usart.txreg);
+  delete_sfr_register(usart.rcreg);
+}
+
 void P16F690::create_symbols(void)
 {
 
@@ -1765,5 +2363,5 @@ void P16F690::create_sfr_map()
   add_sfr_register(usart.rcreg,  0x1a, RegisterValue(0,0),"rcreg");
   usart.set_eusart(true);
 
-  add_sfr_register(&pstrcon, 0x19d, RegisterValue(1,0));
+ // add_sfr_register(&pstrcon, 0x19d, RegisterValue(1,0));
 }
