@@ -61,7 +61,6 @@ public:
     int diff = (i64 ^ v) &0xfff;
     Integer::set(v);
 
-    printf ( "Config3H_2x21::set ( 0x%02X )\n", (int)v );
     if (m_pCpu)
     {
       P18F2x21 *pCpu21 = (P18F2x21*)m_pCpu;
@@ -458,6 +457,17 @@ P18C4x2::P18C4x2(const char *_name, const char *desc)
   m_late  = new PicLatchRegister(this,"late","",m_porte);
 
 }
+P18C4x2::~P18C4x2()
+{
+  delete_sfr_register(m_portd);
+  delete_sfr_register(m_porte);
+
+  delete_sfr_register(m_latd);
+  delete_sfr_register(m_late);
+
+  delete_sfr_register(m_trisd);
+  delete_sfr_register(m_trise);
+}
 
 
 void P18C4x2::create_sfr_map()
@@ -633,7 +643,6 @@ Processor * P18F242::construct(const char *name)
 
   p->create();
   p->create_invalid_registers();
-  p->create_sfr_map();
   p->create_symbols();
   return p;
 
@@ -864,24 +873,81 @@ Processor * P18F452::construct(const char *name)
 // 
 
 P18F2455::P18F2455(const char *_name, const char *desc)
-  : P18F2x21(_name,desc)
+  : P18F2x21(_name,desc),
+      ufrml(this, "ufrml", "USB Frame Number register Low"      ),
+      ufrmh(this, "ufrmh", "USB Frame Number register High"     ),
+      uir  (this, "uir"  , "USB Interrupt Status register"      ),
+      uie  (this, "uie"  , "USB Interrupt Enable register"      ),
+      ueir (this, "ueir" , "USB Error Interrupt Status register"),
+      ueie (this, "ueie" , "USB Error Interrupt Enable register"),
+      ustat(this, "ustat", "USB Transfer Status register"       ),
+      ucon (this, "ucon" , "USB Control register"               ),
+      uaddr(this, "uaddr", "USB Device Address register"        ),
+      ucfg (this, "ucfg" , "USB Configuration register"         ),
+      uep0 (this, "uep0" , "USB Endpoint 0 Enable register"     ),
+      uep1 (this, "uep1" , "USB Endpoint 1 Enable register"     ),
+      uep2 (this, "uep2" , "USB Endpoint 2 Enable register"     ),
+      uep3 (this, "uep3" , "USB Endpoint 3 Enable register"     ),
+      uep4 (this, "uep4" , "USB Endpoint 4 Enable register"     ),
+      uep5 (this, "uep5" , "USB Endpoint 5 Enable register"     ),
+      uep6 (this, "uep6" , "USB Endpoint 6 Enable register"     ),
+      uep7 (this, "uep7" , "USB Endpoint 7 Enable register"     ),
+      uep8 (this, "uep8" , "USB Endpoint 8 Enable register"     ),
+      uep9 (this, "uep9" , "USB Endpoint 9 Enable register"     ),
+      uep10(this, "uep10", "USB Endpoint 10 Enable register"    ),
+      uep11(this, "uep11", "USB Endpoint 11 Enable register"    ),
+      uep12(this, "uep12", "USB Endpoint 12 Enable register"    ),
+      uep13(this, "uep13", "USB Endpoint 13 Enable register"    ),
+      uep14(this, "uep14", "USB Endpoint 14 Enable register"    ),
+      uep15(this, "uep15", "USB Endpoint 15 Enable register"    ),
+      pir2 (this, "pir2" , "Peripheral Interrupt Register",0,0  )
 {
 
   cout << "\nP18F2455 does not support USB registers and functionality\n\n";
   if(verbose)
     cout << "18f2455 constructor, type = " << isa() << '\n';
 
-  // RP : I'm fairly sure this line is wrong
-  m_trisc = new PicTrisRegister(this,"trisc","", (PicPortRegister *)m_portc, true);
 }
 
-void P18F2455::create()
+P18F2455::~P18F2455()
+{
+  remove_sfr_register(&ufrml);
+  remove_sfr_register(&ufrmh);
+  remove_sfr_register(&uir  );
+  remove_sfr_register(&uie  );
+  remove_sfr_register(&ueir );
+  remove_sfr_register(&ueie );
+  remove_sfr_register(&ustat);
+  remove_sfr_register(&ucon );
+  remove_sfr_register(&uaddr);
+  remove_sfr_register(&ucfg );
+  remove_sfr_register(&uep0 );
+  remove_sfr_register(&uep1 );
+  remove_sfr_register(&uep2 );
+  remove_sfr_register(&uep3 );
+  remove_sfr_register(&uep4 );
+  remove_sfr_register(&uep5 );
+  remove_sfr_register(&uep6 );
+  remove_sfr_register(&uep7 );
+  remove_sfr_register(&uep8 );
+  remove_sfr_register(&uep9 );
+  remove_sfr_register(&uep10);
+  remove_sfr_register(&uep11);
+  remove_sfr_register(&uep12);
+  remove_sfr_register(&uep13);
+  remove_sfr_register(&uep14);
+  remove_sfr_register(&uep15);
+  remove_sfr_register(&pir2 );
+}
+void P18F2455::create_sfr_map()
 {
-  P18F2x21::create();
 
   if(verbose)
-    cout << " 18f2455 create \n";
+    cout << " 18f2455 create_sfr_map \n";
 
+
+  P18F2x21::create_sfr_map();
+  package->destroy_pin(14);
   package->assign_pin(14, 0, false);          // Vusb
 
   /* The MSSP/I2CC pins are different on this chip. */
@@ -893,6 +959,44 @@ void P18F2455::create()
                  m_trisb,              // i2c tris port
                  SSP_TYPE_MSSP
        );
+  add_sfr_register(&ufrml,0x0F66, RegisterValue(0,0),"ufrm");
+  add_sfr_register(&ufrmh,0X0F67, RegisterValue(0,0));
+  add_sfr_register(&uir  ,0x0F68, RegisterValue(0,0));
+  add_sfr_register(&uie  ,0x0F69, RegisterValue(0,0));
+  add_sfr_register(&ueir ,0x0F6A, RegisterValue(0,0));
+  add_sfr_register(&ueie ,0x0F6B, RegisterValue(0,0));
+  add_sfr_register(&ustat,0X0F6C, RegisterValue(0,0));
+  add_sfr_register(&ucon ,0x0F6D, RegisterValue(0,0));
+  add_sfr_register(&uaddr,0X0F6E, RegisterValue(0,0));
+  add_sfr_register(&ucfg ,0x0F6F, RegisterValue(0,0));
+  add_sfr_register(&uep0 ,0x0F70, RegisterValue(0,0));
+  add_sfr_register(&uep1 ,0x0F71, RegisterValue(0,0));
+  add_sfr_register(&uep2 ,0x0F72, RegisterValue(0,0));
+  add_sfr_register(&uep3 ,0x0F73, RegisterValue(0,0));
+  add_sfr_register(&uep4 ,0x0F74, RegisterValue(0,0));
+  add_sfr_register(&uep5 ,0x0F75, RegisterValue(0,0));
+  add_sfr_register(&uep6 ,0x0F76, RegisterValue(0,0));
+  add_sfr_register(&uep7 ,0x0F77, RegisterValue(0,0));
+  add_sfr_register(&uep8 ,0x0F78, RegisterValue(0,0));
+  add_sfr_register(&uep9 ,0x0F79, RegisterValue(0,0));
+  add_sfr_register(&uep10,0x0F7A, RegisterValue(0,0));
+  add_sfr_register(&uep11,0x0F7B, RegisterValue(0,0));
+  add_sfr_register(&uep12,0x0F7C, RegisterValue(0,0));
+  add_sfr_register(&uep13,0x0F7D, RegisterValue(0,0));
+  add_sfr_register(&uep14,0x0F7E, RegisterValue(0,0));
+  add_sfr_register(&uep15,0x0F7F, RegisterValue(0,0));
+
+  // Initialize the register cross linkages
+  pir_set_def.set_pir2(&pir2);
+  delete tmr3l.getInterruptSource();
+  tmr3l.setInterruptSource(new InterruptSource(&pir2, PIR2v4::TMR3IF));
+  pir2.set_intcon(&intcon);
+  pir2.set_pie(&pie2);
+  pir2.set_ipr(&ipr2);
+  pie2.setPir(&pir2);
+  //pie2.new_name("pie2");
+
+  // new InterruptSource(&pir2, PIR2v4::USBIF);
 }
 
 Processor * P18F2455::construct(const char *name)
@@ -915,14 +1019,69 @@ Processor * P18F2455::construct(const char *name)
 // 
 
 P18F4455::P18F4455(const char *_name, const char *desc)
-  : P18F4x21(_name,desc)
+  : P18F4x21(_name,desc),
+      ufrml(this, "ufrml", "USB Frame Number register Low"      ),
+      ufrmh(this, "ufrmh", "USB Frame Number register High"     ),
+      uir  (this, "uir"  , "USB Interrupt Status register"      ),
+      uie  (this, "uie"  , "USB Interrupt Enable register"      ),
+      ueir (this, "ueir" , "USB Error Interrupt Status register"),
+      ueie (this, "ueie" , "USB Error Interrupt Enable register"),
+      ustat(this, "ustat", "USB Transfer Status register"       ),
+      ucon (this, "ucon" , "USB Control register"               ),
+      uaddr(this, "uaddr", "USB Device Address register"        ),
+      ucfg (this, "ucfg" , "USB Configuration register"         ),
+      uep0 (this, "uep0" , "USB Endpoint 0 Enable register"     ),
+      uep1 (this, "uep1" , "USB Endpoint 1 Enable register"     ),
+      uep2 (this, "uep2" , "USB Endpoint 2 Enable register"     ),
+      uep3 (this, "uep3" , "USB Endpoint 3 Enable register"     ),
+      uep4 (this, "uep4" , "USB Endpoint 4 Enable register"     ),
+      uep5 (this, "uep5" , "USB Endpoint 5 Enable register"     ),
+      uep6 (this, "uep6" , "USB Endpoint 6 Enable register"     ),
+      uep7 (this, "uep7" , "USB Endpoint 7 Enable register"     ),
+      uep8 (this, "uep8" , "USB Endpoint 8 Enable register"     ),
+      uep9 (this, "uep9" , "USB Endpoint 9 Enable register"     ),
+      uep10(this, "uep10", "USB Endpoint 10 Enable register"    ),
+      uep11(this, "uep11", "USB Endpoint 11 Enable register"    ),
+      uep12(this, "uep12", "USB Endpoint 12 Enable register"    ),
+      uep13(this, "uep13", "USB Endpoint 13 Enable register"    ),
+      uep14(this, "uep14", "USB Endpoint 14 Enable register"    ),
+      uep15(this, "uep15", "USB Endpoint 15 Enable register"    ),
+      pir2 (this, "pir2" , "Peripheral Interrupt Register",0,0  )
+
 {
   cout << "\nP18F4455 does not support USB registers and functionality\n\n";
   if(verbose)
     cout << "18f4455 constructor, type = " << isa() << '\n';
+}
 
-  // RP : I'm fairly sure this line is wrong
-  m_trisc = new PicTrisRegister(this,"trisc","", (PicPortRegister *)m_portc, true);
+P18F4455::~P18F4455()
+{
+  remove_sfr_register(&ufrml);
+  remove_sfr_register(&ufrmh);
+  remove_sfr_register(&uir  );
+  remove_sfr_register(&uie  );
+  remove_sfr_register(&ueir );
+  remove_sfr_register(&ueie );
+  remove_sfr_register(&ustat);
+  remove_sfr_register(&ucon );
+  remove_sfr_register(&uaddr);
+  remove_sfr_register(&ucfg );
+  remove_sfr_register(&uep0 );
+  remove_sfr_register(&uep1 );
+  remove_sfr_register(&uep2 );
+  remove_sfr_register(&uep3 );
+  remove_sfr_register(&uep4 );
+  remove_sfr_register(&uep5 );
+  remove_sfr_register(&uep6 );
+  remove_sfr_register(&uep7 );
+  remove_sfr_register(&uep8 );
+  remove_sfr_register(&uep9 );
+  remove_sfr_register(&uep10);
+  remove_sfr_register(&uep11);
+  remove_sfr_register(&uep12);
+  remove_sfr_register(&uep13);
+  remove_sfr_register(&uep14);
+  remove_sfr_register(&uep15);
 }
 
 void P18F4455::create()
@@ -932,6 +1091,7 @@ void P18F4455::create()
   if(verbose)
     cout << " 18f4455 create \n";
 
+  package->destroy_pin(18);
   package->assign_pin(18, 0, false);          // Vusb
 
   /* The MSSP/I2CC pins are different on this chip. */
@@ -945,6 +1105,44 @@ void P18F4455::create()
        );
 
   // RP: RRR commented out comparator.cmcon.set_eccpas(&eccpas);  ??
+  add_sfr_register(&ufrml,0x0F66, RegisterValue(0,0),"ufrm");
+  add_sfr_register(&ufrmh,0X0F67, RegisterValue(0,0));
+  add_sfr_register(&uir  ,0x0F68, RegisterValue(0,0));
+  add_sfr_register(&uie  ,0x0F69, RegisterValue(0,0));
+  add_sfr_register(&ueir ,0x0F6A, RegisterValue(0,0));
+  add_sfr_register(&ueie ,0x0F6B, RegisterValue(0,0));
+  add_sfr_register(&ustat,0X0F6C, RegisterValue(0,0));
+  add_sfr_register(&ucon ,0x0F6D, RegisterValue(0,0));
+  add_sfr_register(&uaddr,0X0F6E, RegisterValue(0,0));
+  add_sfr_register(&ucfg ,0x0F6F, RegisterValue(0,0));
+  add_sfr_register(&uep0 ,0x0F70, RegisterValue(0,0));
+  add_sfr_register(&uep1 ,0x0F71, RegisterValue(0,0));
+  add_sfr_register(&uep2 ,0x0F72, RegisterValue(0,0));
+  add_sfr_register(&uep3 ,0x0F73, RegisterValue(0,0));
+  add_sfr_register(&uep4 ,0x0F74, RegisterValue(0,0));
+  add_sfr_register(&uep5 ,0x0F75, RegisterValue(0,0));
+  add_sfr_register(&uep6 ,0x0F76, RegisterValue(0,0));
+  add_sfr_register(&uep7 ,0x0F77, RegisterValue(0,0));
+  add_sfr_register(&uep8 ,0x0F78, RegisterValue(0,0));
+  add_sfr_register(&uep9 ,0x0F79, RegisterValue(0,0));
+  add_sfr_register(&uep10,0x0F7A, RegisterValue(0,0));
+  add_sfr_register(&uep11,0x0F7B, RegisterValue(0,0));
+  add_sfr_register(&uep12,0x0F7C, RegisterValue(0,0));
+  add_sfr_register(&uep13,0x0F7D, RegisterValue(0,0));
+  add_sfr_register(&uep14,0x0F7E, RegisterValue(0,0));
+  add_sfr_register(&uep15,0x0F7F, RegisterValue(0,0));
+
+  // Initialize the register cross linkages
+  pir_set_def.set_pir2(&pir2);
+  delete tmr3l.getInterruptSource();
+  tmr3l.setInterruptSource(new InterruptSource(&pir2, PIR2v4::TMR3IF));
+  pir2.set_intcon(&intcon);
+  pir2.set_pie(&pie2);
+  pir2.set_ipr(&ipr2);
+  pie2.setPir(&pir2);
+  //pie2.new_name("pie2");
+
+  //new InterruptSource(&pir2, PIR2v4::USBIF);
 }
 
 Processor * P18F4455::construct(const char *name)
@@ -1010,14 +1208,11 @@ void P18F1220::create()
 
   create_iopin_map();
 
-  //call his before _16bit_processor::create() because 1st one wins
-  //with regards to the rx/tx ports
-  usart.initialize(&pir1,&(*m_portb)[1], &(*m_portb)[4],
-		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
-                   new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
   _16bit_processor::create();
   _16bit_v2_adc::create(7);
+  usart.txsta.setIOpin(&(*m_portb)[1]);
+  usart.rcsta.setIOpin(&(*m_portb)[4]);
   adcon1->setIOPin(4, &(*m_portb)[0]);
   adcon1->setIOPin(5, &(*m_portb)[1]);
   adcon1->setIOPin(6, &(*m_portb)[4]);
@@ -1099,6 +1294,15 @@ P18F1220::P18F1220(const char *_name, const char *desc)
 
 }
 
+P18F1220::~P18F1220()
+{
+  remove_sfr_register(&usart.spbrgh);
+  remove_sfr_register(&usart.baudcon);
+  remove_sfr_register(&osctune);
+  remove_sfr_register(&pwm1con);
+  remove_sfr_register(&eccpas);
+}
+
 void P18F1220::osc_mode(unsigned int value)
 {
   IOPIN *m_pin;
@@ -1235,7 +1439,7 @@ void P18F2x21::create()
   /// data sheet, but bit-restricted according to section 16.4.7
   add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
   add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
-  eccpas.set_mask(0xfc);
+  eccpas.setBitMask(0xfc);
   eccpas.setIOpin(0, 0, &(*m_portb)[0]);
   eccpas.link_registers(&pwm1con, &ccp1con);
   comparator.cmcon.set_eccpas(&eccpas);
@@ -1243,7 +1447,7 @@ void P18F2x21::create()
   ccp1con.setCrosslinks(&ccpr1l, &pir1, PIR1v2::CCP1IF, &tmr2, &eccpas);
   ccp1con.pwm1con = &pwm1con;
   ccp1con.setIOpin(&((*m_portc)[2]), 0, 0, 0);
-  pwm1con.set_mask(0x80);
+  pwm1con.setBitMask(0x80);
 }
 
 //------------------------------------------------------------------------
@@ -1331,6 +1535,18 @@ P18F2x21::P18F2x21(const char *_name, const char *desc)
     // No TRIS register for port E on 28-pin devices
 }
 
+P18F2x21::~P18F2x21()
+{
+    delete_sfr_register(m_porte);
+    remove_sfr_register(&usart.spbrgh);
+    remove_sfr_register(&usart.baudcon);
+    remove_sfr_register(&osctune);
+    remove_sfr_register(&comparator.cmcon);
+    remove_sfr_register(&comparator.vrcon);
+    remove_sfr_register(&pwm1con);
+    remove_sfr_register(&eccpas);
+}
+
 void P18F2x21::create_sfr_map()
 {
 
@@ -1737,15 +1953,25 @@ P18F4x21::P18F4x21(const char *_name, const char *desc)
     cout << "18f4x21 constructor, type = " << isa() << '\n';
 
   m_portd = new PicPSP_PortRegister(this,"portd","",8,0xFF);
-  m_trisd = new PicTrisRegister(this,"trisd","", (PicPortRegister *)m_portd, true);
+  m_trisd = new PicTrisRegister(this,"trisd","", (PicPortRegister *)m_portd, false);
   m_latd  = new PicLatchRegister(this,"latd","",m_portd);
 
 //  m_porte = new PicPortRegister(this,"porte","",8,0x07);
   m_porte->setEnableMask(0x07);     // It's been created by the P18F2x21 constructor, but with the wrong enables
-  m_trise = new PicPSP_TrisRegister(this,"trise","", m_porte, true);
+  m_trise = new PicPSP_TrisRegister(this,"trise","", m_porte, false);
   m_late  = new PicLatchRegister(this,"late","",m_porte);
 
 }
+P18F4x21::~P18F4x21()
+{
+  delete_sfr_register(m_portd);
+  delete_sfr_register(m_trisd);
+  delete_sfr_register(m_latd);
+  delete_sfr_register(m_trise);
+  delete_sfr_register(m_late);
+  remove_sfr_register(&pwm1con);
+  remove_sfr_register(&eccpas);
+}
 
 void P18F4x21::create_sfr_map()
 {
@@ -1832,7 +2058,6 @@ void P18F4x21::create_sfr_map()
 }
 
 
-
 //------------------------------------------------------------------------
 //
 // P18F4221
@@ -2117,6 +2342,14 @@ void P18F6x20::create_iopin_map()
   package->assign_pin(63, m_porte->addPin(new IO_bi_directional("porte3"),3));
   package->assign_pin(64, m_porte->addPin(new IO_bi_directional("porte2"),2));
 
+  psp.initialize(&pir_set_def,    // PIR
+                m_portd,           // Parallel port
+                m_trisd,           // Parallel tris
+                pspcon,           // Control register
+                &(*m_porte)[0],    // NOT RD
+                &(*m_porte)[1],    // NOT WR
+                &(*m_porte)[2]);   // NOT CS
+
   tmr1l.setIOpin(&(*m_portc)[0]);
   ssp.initialize(&pir_set_def,    // PIR
                 &(*m_portc)[3],   // SCK
@@ -2165,24 +2398,68 @@ P18F6x20::P18F6x20(const char *_name, const char *desc)
     cout << "18F6x20 constructor, type = " << isa() << '\n';
 
   m_portd = new PicPSP_PortRegister(this,"portd","",8,0xFF);
-  m_trisd = new PicTrisRegister(this,"trisd","", (PicPortRegister *)m_portd, true);
+  m_trisd = new PicTrisRegister(this,"trisd","", (PicPortRegister *)m_portd, false);
   m_latd  = new PicLatchRegister(this,"latd","",m_portd);
 
   m_porte = new PicPortRegister(this,"porte","",8,0xFF);
-  m_trise = new PicPSP_TrisRegister(this,"trise","", m_porte, true);
+  m_trise = new PicTrisRegister(this,"trise","", m_porte, false);
   m_late  = new PicLatchRegister(this,"late","",m_porte);
 
   m_portf = new PicPortRegister(this,"portf","",8,0xFF);
-  m_trisf = new PicTrisRegister(this,"trisf","", (PicPortRegister *)m_portf, true);
+  m_trisf = new PicTrisRegister(this,"trisf","", m_portf, false);
   m_latf  = new PicLatchRegister(this,"latf","",m_portf);
 
   m_portg = new PicPortRegister(this,"portg","",8,0x1F);
-  m_trisg = new PicTrisRegister(this,"trisg","", m_portg, true);
+  m_trisg = new PicTrisRegister(this,"trisg","", m_portg, false);
   m_latg  = new PicLatchRegister(this,"latg","",m_portg);
 
+  pspcon = new PSPCON(this, "pspcon","");
+
 
 }
 
+P18F6x20::~P18F6x20()
+{
+  delete_sfr_register(m_portd);
+  delete_sfr_register(m_porte);
+  delete_sfr_register(m_portf);
+  delete_sfr_register(m_portg);
+
+  delete_sfr_register(m_latd);
+  delete_sfr_register(m_late);
+  delete_sfr_register(m_latf);
+  delete_sfr_register(m_latg);
+
+  delete_sfr_register(m_trisd);
+  delete_sfr_register(m_trise);
+  delete_sfr_register(m_trisf);
+  delete_sfr_register(m_trisg);
+  delete_sfr_register(pspcon);
+  delete_sfr_register(usart2.txreg);
+  delete_sfr_register(usart2.rcreg);
+
+  remove_sfr_register(&pie3);
+  remove_sfr_register(&pir3);
+  remove_sfr_register(&ipr3);
+  remove_sfr_register(&usart2.rcsta);
+  remove_sfr_register(&usart2.txsta);
+  remove_sfr_register(&usart2.spbrg);
+  remove_sfr_register(&ccp4con);
+  remove_sfr_register(&ccpr4l);
+  remove_sfr_register(&ccpr4h);
+  remove_sfr_register(&ccp5con);
+  remove_sfr_register(&ccpr5l);
+  remove_sfr_register(&ccpr5h);
+  remove_sfr_register(&t4con);
+  remove_sfr_register(&pr4);
+  remove_sfr_register(&tmr4);
+  remove_sfr_register(&ccp3con);
+  remove_sfr_register(&ccpr3l);
+  remove_sfr_register(&ccpr3h);
+  remove_sfr_register(&comparator.cmcon);
+  remove_sfr_register(&comparator.vrcon);
+
+}
 
 void P18F6x20::create_sfr_map()
 {
@@ -2216,6 +2493,7 @@ void P18F6x20::create_sfr_map()
   add_sfr_register(&pir3,	  0xfa4,porv,"pir3");
   add_sfr_register(&ipr3,	  0xfa5,porv,"ipr3");
 
+  add_sfr_register(pspcon,       0xfb0,RegisterValue(0x00,0));
 
   // cout << "Assign ADC pins to " << adcon1 << "\n";
   adcon1->setIOPin(4, &(*m_porta)[5]);
