@@ -26,6 +26,7 @@ License along with this library; if not, see
 #include "16bit-processors.h"
 #include "eeprom.h"
 #include "psp.h"
+#include "pir.h"
 #include "comparator.h"
 
 class PicPortRegister;
@@ -103,6 +104,7 @@ class P18C4x2 : public _16bit_compat_adc
   PSP               psp;
 
   P18C4x2(const char *_name=0, const char *desc=0);
+  ~P18C4x2();
 
   void create();
 
@@ -230,42 +232,18 @@ class P18F452 : public P18F442
   virtual unsigned int last_actual_register () const { return 0x05FF;};
 };
 
-class P18F2455 : public P18F242
-{
- public:
-  ECCPAS        eccpas;
-  PWM1CON       pwm1con;
-  virtual PROCESSOR_TYPE isa(){return _P18F2455_;};
-  P18F2455(const char *_name=0, const char *desc=0);
-  static Processor *construct(const char *name);
-  void create();
 
-  virtual unsigned int program_memory_size() const { return 0x3000; };
-  virtual unsigned int last_actual_register () const { return 0x07FF;};
-
-};
-
-class P18F4455 : public P18F442
-{
- public:
-  ECCPAS        eccpas;
-  PWM1CON       pwm1con;
-
-  virtual PROCESSOR_TYPE isa(){return _P18F4455_;};
-  P18F4455(const char *_name=0, const char *desc=0);
-  static Processor *construct(const char *name);
-  void create();
-
-  virtual unsigned int program_memory_size() const { return 0x3000; };
-  virtual unsigned int last_actual_register () const { return 0x07FF;};
-
-};
 
 class P18Fxx20 : public _16bit_v2_adc
 {
 public:
   P18Fxx20(const char *_name=0, const char *desc=0);
   virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
+  virtual unsigned int last_actual_register () const { return 0x00FF;};
+
+  // Strip down from base class
+  virtual bool HasPortC(void) { return false; };
+  virtual bool HasCCP2(void) { return false; };
 };
 
 class P18F1220 : public P18Fxx20
@@ -277,6 +255,7 @@ class P18F1220 : public P18Fxx20
 
   virtual PROCESSOR_TYPE isa(){return _P18F1220_;};
   P18F1220(const char *_name=0, const char *desc=0);
+  ~P18F1220();
   static Processor *construct(const char *name);
   void create();
   virtual void create_iopin_map();
@@ -321,6 +300,7 @@ class P18F2x21 : public _16bit_v2_adc
   ComparatorModule comparator;
 
   P18F2x21(const char *_name=0, const char *desc=0);
+  ~P18F2x21();
 
   void create();
 
@@ -329,6 +309,7 @@ class P18F2x21 : public _16bit_v2_adc
   virtual void create_symbols();
 
   virtual unsigned int program_memory_size() const { return 0x400; };
+  virtual unsigned int eeprom_memory_size() const { return 0x100; };
 
 // Setting the correct register memory size breaks things
 //  virtual unsigned int register_memory_size () const { return 0x200;};
@@ -344,6 +325,19 @@ class P18F2x21 : public _16bit_v2_adc
   }
   virtual void set_eeprom_pir(EEPROM_PIR *ep) { eeprom = ep; }
   virtual EEPROM_PIR *get_eeprom() { return ((EEPROM_PIR *)eeprom); }
+
+  virtual void osc_mode(unsigned int value);
+};
+
+
+class P18F2221 : public P18F2x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F2221_;};
+  P18F2221(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x800; };
 };
 
 class P18F2321 : public P18F2x21
@@ -352,13 +346,52 @@ class P18F2321 : public P18F2x21
   virtual PROCESSOR_TYPE isa(){return _P18F2321_;};
   P18F2321(const char *_name=0, const char *desc=0);
   static Processor *construct(const char *name);
-  void create();
 
   virtual unsigned int program_memory_size() const { return 0x1000; };
-  virtual void osc_mode(unsigned int value);
+};
+
+class P18F2420 : public P18F2x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F2420_;};
+  P18F2420(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x2000; };
+};
+
+class P18F2455 : public P18F2x21
+{
+ public:
+  sfr_register ufrml, ufrmh, uir,  uie,  ueir,  ueie,  ustat,  ucon,
+               uaddr, ucfg,  uep0, uep1, uep2,  uep3,  uep4,   uep5,
+               uep6,  uep7,  uep8, uep9, uep10, uep11, uep12,  uep13,
+               uep14,  uep15;
+
+  PIR2v4       pir2;
+  virtual PROCESSOR_TYPE isa(){return _P18F2455_;};
+  P18F2455(const char *_name=0, const char *desc=0);
+  ~P18F2455();
+  static Processor *construct(const char *name);
+  void create_sfr_map();
+
+  virtual unsigned int access_gprs() { return 0x60; };  // USB peripheral moves access split
+  virtual unsigned int program_memory_size() const { return 0x3000; };
+  virtual unsigned int last_actual_register () const { return 0x07FF;};
 
 };
 
+class P18F2520 : public P18F2x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F2520_;};
+  P18F2520(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x4000; };
+};
+
+
 
 class P18F4x21 : public P18F2x21
 {
@@ -371,6 +404,7 @@ class P18F4x21 : public P18F2x21
   PSP               psp;
 
   P18F4x21(const char *_name=0, const char *desc=0);
+  ~P18F4x21();
 
   void create();
 
@@ -381,18 +415,178 @@ class P18F4x21 : public P18F2x21
 
 };
 
+
+class P18F4221 : public P18F4x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F4221_;};
+  P18F4221(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x800; };
+};
+
 class P18F4321 : public P18F4x21
 {
  public:
   virtual PROCESSOR_TYPE isa(){return _P18F4321_;};
   P18F4321(const char *_name=0, const char *desc=0);
   static Processor *construct(const char *name);
-  void create();
 
   virtual unsigned int program_memory_size() const { return 0x1000; };
-  virtual void osc_mode(unsigned int value);
+};
+
+class P18F4420 : public P18F4x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F4420_;};
+  P18F4420(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x2000; };
+};
+
+class P18F4455 : public P18F4x21
+{
+ public:
+  sfr_register ufrml, ufrmh, uir,  uie,  ueir,  ueie,  ustat,  ucon,
+               uaddr, ucfg,  uep0, uep1, uep2,  uep3,  uep4,   uep5,
+               uep6,  uep7,  uep8, uep9, uep10, uep11, uep12,  uep13,
+               uep14,  uep15;
+
+  PIR2v4       pir2;
+
+  virtual PROCESSOR_TYPE isa(){return _P18F4455_;};
+  P18F4455(const char *_name=0, const char *desc=0);
+  ~P18F4455();
+  static Processor *construct(const char *name);
+  void create();
+
+  virtual unsigned int access_gprs() { return 0x60; };  // USB peripheral moves access split
+  virtual unsigned int program_memory_size() const { return 0x3000; };
+  virtual unsigned int last_actual_register () const { return 0x07FF;};
 
 };
 
+class P18F4520 : public P18F4x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F4520_;};
+  P18F4520(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x4000; };
+};
+
+
+
+
+
+/***
+PIC18F4620
+Not implemented: 
+  OSCFIF bit in peripheral interrupt register 2 (PIR2v2 pir2)(And Enable Bit)
+  
+***/
+class P18F4620 : public P18F4x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F4620_;};
+  P18F4620(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x8000; };
+  virtual unsigned int eeprom_memory_size() const { return 1024; };
+  virtual unsigned int last_actual_register () const { return 0x0F7F;};
+};
+
+
+
+class P18F6x20 : public _16bit_v2_adc
+{
+ public:
+
+  PicPSP_PortRegister  *m_portd;
+  PicTrisRegister  *m_trisd;
+  PicLatchRegister *m_latd;
+
+  PicPortRegister  *m_porte;
+  //RRRPicPSP_TrisRegister  *m_trise;
+  PicTrisRegister  *m_trise;
+  PicLatchRegister *m_late;
+
+  PicPortRegister  *m_portf;
+  PicTrisRegister  *m_trisf;
+  PicLatchRegister *m_latf;
+
+  PicPortRegister  *m_portg;
+  PicTrisRegister  *m_trisg;
+  PicLatchRegister *m_latg;
+
+  PSP               psp;
+  PSPCON	    *pspcon;
+
+//  ECCPAS        eccpas;
+//  PWM1CON       pwm1con;
+  T2CON        t4con;
+  PR2          pr4;
+  TMR2         tmr4;
+  PIR3v1       pir3;
+  PIE          pie3;
+  sfr_register ipr3;
+  CCPCON       ccp3con;
+  CCPRL        ccpr3l;
+  CCPRH        ccpr3h;
+  CCPCON       ccp4con;
+  CCPRL        ccpr4l;
+  CCPRH        ccpr4h;
+  CCPCON       ccp5con;
+  CCPRL        ccpr5l;
+  CCPRH        ccpr5h;
+  USART_MODULE         usart2;
+
+//  OSCTUNE      osctune;
+  ComparatorModule comparator;
+
+  P18F6x20(const char *_name=0, const char *desc=0);
+  ~P18F6x20();
+
+  void create();
+
+  virtual PROCESSOR_TYPE isa(){return _P18Cxx2_;};
+  virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
+  virtual unsigned int access_gprs() { return 0x60; };
+  virtual void create_symbols();
+
+  virtual unsigned int program_memory_size() const { return 0x4000; };
+  virtual unsigned int eeprom_memory_size() const { return 1024; };
+
+// Setting the correct register memory size breaks things
+//  virtual unsigned int register_memory_size () const { return 0x800;};
+  virtual unsigned int last_actual_register () const { return 0x07FF;};
+
+  virtual void create_iopin_map();
+  virtual void create_sfr_map();
+
+
+  virtual void set_eeprom(EEPROM *ep) {
+    // Use set_eeprom_pir as the 18Fxxx devices use an EEPROM with PIR
+   assert(0);
+  }
+  virtual void set_eeprom_pir(EEPROM_PIR *ep) { eeprom = ep; }
+  virtual EEPROM_PIR *get_eeprom() { return ((EEPROM_PIR *)eeprom); }
+};
+
+
+class P18F6520 : public P18F6x20
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F6520_;};
+  P18F6520(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+//  virtual unsigned int program_memory_size() const { return 0x4000; };
+  virtual unsigned int bugs() { return BUG_DAW; };
+};
 
 #endif
