@@ -47,6 +47,7 @@ public:
                   /*const char *tris_name, */
                   PicPortRegister *,bool bIgnoreWDTResets, unsigned int nEnableMask=0xff);
   virtual void put(unsigned int new_value);
+  virtual void put_value(unsigned int new_value);
   virtual unsigned int get();
   virtual char get3StateBit(unsigned int bitMask);
   void setEnableMask(unsigned int);
@@ -59,6 +60,8 @@ protected:
 };
 
 class INTCON;
+class INTCON2;
+class INTCON3;
 //  PicPortBRegister is usually used for portb and interrupts on selected edge
 //  of bit 0 and sleep wakeup and interrupt on level changes for bits 4-7. 
 class PicPortBRegister : public PicPortRegister
@@ -66,7 +69,11 @@ class PicPortBRegister : public PicPortRegister
 public:
   PicPortBRegister(Processor *pCpu, const char *pName, const char *pDesc,
                    INTCON *pIntcon,
-                   unsigned int numIopins, unsigned int enableMask=0xff);
+                   unsigned int numIopins, 
+		   unsigned int enableMask=0xff,
+		   INTCON2 *pIntcon2 = NULL,
+		   INTCON3 *pIntcon3 = NULL
+		);
   ~PicPortBRegister();
 
   virtual void put(unsigned int new_value);
@@ -75,7 +82,7 @@ public:
   void setRBPU(bool);
   void setIntEdge(bool);
   void assignRBPUSink(unsigned int bitPos, sfr_register *);
-private:
+protected:
   enum {
     eIntEdge = 1<<6,
     eRBPU    = 1<<7
@@ -85,6 +92,8 @@ private:
 
   BitSink *m_bsRBPU;
   INTCON  *m_pIntcon;
+  INTCON2  *m_pIntcon2;
+  INTCON3  *m_pIntcon3;
 };
 
 class IOC;
@@ -94,6 +103,7 @@ class IOC;
 //	Note: as GPIF,GPIE are the same bits as RBIF,RBIE in INTCON we can
 //	use the existing set_rbif function to set the GPIF bit.
 //
+//	 
 class PicPortGRegister : public PicPortBRegister
 {
  
@@ -110,11 +120,27 @@ public:
   }
 
   virtual void setbit(unsigned int bit_number, char new3State);
-  void setIntEdge(bool);
-
-private:
-  bool m_bIntEdge;
 };
+class PicPortIOCRegister : public PicPortBRegister
+{
+ 
+public:
+  INTCON *m_pIntcon;
+  IOC	*m_Iocap;
+  IOC	*m_Iocan;
+  IOC	*m_Iocaf;
+
+  PicPortIOCRegister(Processor *pCpu, const char *pName, const char *pDesc,
+                   INTCON *pIntcon, IOC *pIocap, IOC *pIocan, IOC *pIocaf,
+                   unsigned int numIopins, unsigned int enableMask=0x3f)
+	: PicPortBRegister(pCpu, pName, pDesc, pIntcon, numIopins, enableMask),
+	m_pIntcon(pIntcon), m_Iocap(pIocap), m_Iocan(pIocan), m_Iocaf(pIocaf)
+  {
+  }
+
+  virtual void setbit(unsigned int bit_number, char new3State);
+};
+
 class PSP;
 
 class PicPSP_PortRegister : public PortRegister
