@@ -54,6 +54,7 @@ License along with this library; if not, see
 #include "p16x7x.h"
 #include "p12x.h"
 #include "p12f6xx.h"
+#include "p12f182x.h"
 #ifdef P17C7XX  // code no longer works
 #include "p17c75x.h"
 #endif
@@ -180,6 +181,8 @@ ProcessorConstructor pP12F675(P12F675::construct ,
                               "__12F675",   "pic12f675",    "p12f675",  "12f675");
 ProcessorConstructor pP12F683(P12F683::construct ,
                               "__12F683",   "pic12f683",    "p12f683",  "12f683");
+ProcessorConstructor pP12F1822(P12F1822::construct ,
+                              "__12F1822", "pic12f1822", "p12f1822", "12f1822");
 ProcessorConstructor pP16C54(P16C54::construct ,
                              "__16C54",     "pic16c54",     "p16c54",   "16c54");
 ProcessorConstructor pP16C55(P16C55::construct ,
@@ -262,6 +265,8 @@ ProcessorConstructor pP16F676(P16F676::construct ,
                               "__16F676",   "pic16f676",    "p16f676",  "16f676");
 ProcessorConstructor pP16F677(P16F677::construct ,
                               "__16F677",   "pic16f677",    "p16f677",  "16f677");
+ProcessorConstructor pP16F684(P16F684::construct ,
+                              "__16F684",   "pic16f684",    "p16f684",  "16f684");
 ProcessorConstructor pP16F685(P16F685::construct ,
                               "__16F685",   "pic16f685",    "p16f685",  "16f685");
 ProcessorConstructor pP16F687(P16F687::construct ,
@@ -292,6 +297,8 @@ ProcessorConstructor pP16F876A(P16F876A::construct ,
                               "__16F876a",  "pic16f876a",   "p16f876a", "16f876a");
 ProcessorConstructor pP16F877A(P16F877A::construct ,
                               "__16F877a",  "pic16f877a",   "p16f877a", "16f877a");
+ProcessorConstructor pP16F1823(P16F1823::construct ,
+                              "__16F1823", "pic16f1823", "p16f1823", "16f1823");
 #ifdef P17C7XX  // code no longer works
 ProcessorConstructor pP17C7xx(P17C7xx::construct ,
                               "__17C7xx", "pic17c7xx",  "p17c7xx", "17c7xx");
@@ -427,7 +434,8 @@ int InterruptTraceType::dump_raw(Trace *pTrace,unsigned int tbi, char *buf, int
 void pic_processor::set_eeprom(EEPROM *e)
 {
   eeprom = e;
-  ema.set_Registers(e->rom, e->rom_size);
+  if (e)
+      ema.set_Registers(e->rom, e->rom_size);
 }
 
 //-------------------------------------------------------------------
@@ -730,8 +738,8 @@ void pic_processor::save_state()
 {
   Processor::save_state();
 
-  if(W)
-    W->put_trace_state(W->value);
+  if(Wreg)
+    Wreg->put_trace_state(Wreg->value);
 
   if(eeprom)
     eeprom->save_state();
@@ -838,7 +846,9 @@ void pic_processor::step (unsigned int steps, bool refresh)
   mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
 
   do
+  {
     mCurrentPhase = mCurrentPhase->advance();
+  }
   while(!bp.have_halt() && --steps>0);
 
   // complete the step if this is a multi-cycle instruction.
@@ -961,18 +971,11 @@ void pic_processor::reset (RESET_TYPE r)
   }
 
   m_pResetTT->record(r);
-  if(r == SOFT_RESET) {
-
-    pc->reset();
-    gi.simulation_has_stopped();
-    cout << " --- Soft Reset (not fully implemented)\n";
-    return;
-  }
 
   rma.reset(r);
-  pc->reset();
-  stack->reset();
+  stack->reset(r);
   wdt.reset(r);
+  pc->reset();
 
   bp.clear_global();
 
@@ -987,8 +990,17 @@ void pic_processor::reset (RESET_TYPE r)
     m_ActivityState = ePAActive;
     break;
 
+  case SOFT_RESET:
+    cout << "Reset due to Software reset instruction\n";
+    mCurrentPhase = mExecute1Cycle;
+    mCurrentPhase->setNextPhase(mExecute1Cycle);
+    m_ActivityState = ePAActive;
+    break;
+
+
   case MCLR_RESET:
-    mCurrentPhase = mCurrentPhase ? mCurrentPhase : mIdle;
+    cout << "MCLR reset\n";
+    mCurrentPhase = mIdle;
     mCurrentPhase->setNextPhase(mIdle);
     m_ActivityState = ePAIdle;
     break;
@@ -1000,13 +1012,39 @@ void pic_processor::reset (RESET_TYPE r)
     break;
 
   case WDT_RESET:
-  case EXIT_RESET:
+    cout << "Reset on Watch Dog Timer expire\n";
     mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
     mCurrentPhase->setNextPhase(mExecute1Cycle);
     m_ActivityState = ePAActive;
     break;
 
+  case EXIT_RESET:	// MCLR reset has cleared
+    cout <<"MCLR low, resume execution\n";
+    mCurrentPhase = mCurrentPhase ? mCurrentPhase : mExecute1Cycle;
+    mCurrentPhase->setNextPhase(mExecute1Cycle);
+    m_ActivityState = ePAActive;
+    return;
+    break;
+
+  case STKOVF_RESET:
+    cout << "Reset on Stack overflow\n";
+     mCurrentPhase = mCurrentPhase ? mCurrentPhase : mIdle;
+    mCurrentPhase->setNextPhase(mIdle);
+    m_ActivityState = ePAActive;
+  //  mCurrentPhase->setNextPhase(mExecute1Cycle);
+  //  m_ActivityState = ePAActive;
+    break;
+
+  case STKUNF_RESET:
+    cout << "Reset on Stack undeflow\n";
+    mCurrentPhase = mCurrentPhase ? mCurrentPhase : mIdle;
+    mCurrentPhase->setNextPhase(mIdle);
+    m_ActivityState = ePAActive;
+    break;
+
+
   default:
+    printf("pic_processor::reset unknow reset type %d\n", r);
     m_ActivityState = ePAActive;
     break;
   }
@@ -1025,7 +1063,7 @@ void pic_processor::reset (RESET_TYPE r)
 pic_processor::pic_processor(const char *_name, const char *_desc)
   : Processor(_name,_desc),
     wdt(this, 18.0e-3),indf(0),fsr(0), stack(0), status(0),
-    W(0), pcl(0), pclath(0),m_PCHelper(0),
+    Wreg(0), pcl(0), pclath(0),m_PCHelper(0),
     tmr0(this,"tmr0","Timer 0"),
     m_configMemory(0),
     m_MCLR(0), m_MCLR_Save(0), m_MCLRMonitor(0)
@@ -1062,7 +1100,7 @@ pic_processor::~pic_processor()
   delete m_pResetTT;
   delete m_pInterruptTT;
 
-  delete_sfr_register(W);
+  delete_sfr_register(Wreg);
   delete_sfr_register(pcl);
 
   delete_sfr_register(pclath);
@@ -1107,7 +1145,7 @@ void pic_processor::create ()
   // Now, initialize the core stuff:
   pc->set_cpu(this);
 
-  W = new WREG(this,"W","Working Register");
+  Wreg = new WREG(this,"W","Working Register");
 
   pcl = new PCL(this,"pcl", "Program Counter Low");
   pclath = new PCLATH(this,"pclath", "Program Counter Latch High");
@@ -1122,11 +1160,11 @@ void pic_processor::create ()
     m_PCHelper = new PCHelper(this,pma);
     rma.SpecialRegisters.push_back(m_PCHelper);
     rma.SpecialRegisters.push_back(status);
-    rma.SpecialRegisters.push_back(W);
+    rma.SpecialRegisters.push_back(Wreg);
 
     pma->SpecialRegisters.push_back(m_PCHelper);
     pma->SpecialRegisters.push_back(status);
-    pma->SpecialRegisters.push_back(W);
+    pma->SpecialRegisters.push_back(Wreg);
 
   }
 
@@ -1154,6 +1192,19 @@ void pic_processor::add_sfr_register(Register *reg, unsigned int addr,
   reg->set_cpu(this);
   if(addr < register_memory_size())
     {
+      if (registers[addr]) 
+      {
+	if (registers[addr]->isa() == Register::INVALID_REGISTER)
+	  delete registers[addr];
+	else
+	    printf("%s 0x%x Already register %s\n", __FUNCTION__, addr, registers[addr]->name().c_str());
+      }
+#ifdef DEBUG
+      if (addr == 0xfa1)
+      {
+	printf("DEBUG trace 0x%x\n", addr);
+      }
+#endif
       registers[addr] = reg;
       registers[addr]->address = addr;
       registers[addr]->alias_mask = 0;
@@ -1174,6 +1225,8 @@ void pic_processor::add_sfr_register(Register *reg, unsigned int addr,
 //-------------------------------------------------------------------
 //
 // delete_sfr_register
+// This both deletes the register from the registers array,
+// but also deletes the register class.
 //
 void pic_processor::delete_sfr_register(Register *pReg)
 {
@@ -1181,6 +1234,7 @@ void pic_processor::delete_sfr_register(Register *pReg)
   if (pReg) {
 
     unsigned int a = pReg->getAddress();
+
     if (0)
       cout << __FUNCTION__ << " addr = 0x"<<hex<<a
            <<" reg " << pReg->name()<<endl;
@@ -1196,7 +1250,8 @@ void pic_processor::delete_sfr_register(Register *pReg)
 
 //-------------------------------------------------------------------
 //
-// delete_sfr_register
+// remove_sfr_register
+// This is the inverse of add_sfr_register and does not delete the register.
 //
 void pic_processor::remove_sfr_register(Register *ppReg)
 {
@@ -1204,6 +1259,8 @@ void pic_processor::remove_sfr_register(Register *ppReg)
   if (ppReg) {
 
     unsigned int a = ppReg->getAddress();
+    if (a == AN_INVALID_ADDRESS)
+	return;
     if (registers[a] == ppReg)
       delete_file_registers(a,a,true);
   }
@@ -1266,26 +1323,22 @@ void pic_processor::create_symbols ()
 
 //-------------------------------------------------------------------
 
+
 bool pic_processor::set_config_word(unsigned int address,unsigned int cfg_word)
 {
 
-  if (m_configMemory)
-  {
-      for(int i = 0; m_configMemory->getConfigWord(i); i++)
-      {
-        if (m_configMemory->getConfigWord(i)->ConfigWordAdd() == address)
-        {
-            m_configMemory->getConfigWord(i)->set((int)cfg_word);
-            if (i == 0 && config_modes)
-            {
-                config_word = cfg_word;
-                config_modes->config_mode = (config_modes->config_mode & ~7) |
-                                        (cfg_word & 7);
-            }
+  int i = get_config_index(address);
 
-            return true;
-        }
-     }
+  if ( i >= 0)
+  {
+      m_configMemory->getConfigWord(i)->set((int)cfg_word);
+      if (i == 0 && config_modes)
+      {
+           config_word = cfg_word;
+           config_modes->config_mode = (config_modes->config_mode & ~7) |
+                                        (cfg_word & 7);
+      }
+      return true;
   }
 
   return false;
@@ -1295,7 +1348,33 @@ bool pic_processor::set_config_word(unsigned int address,unsigned int cfg_word)
 
 unsigned int pic_processor::get_config_word(unsigned int address)
 {
-  return address == config_word_address() ? config_word : 0xffffffff;
+  int i;
+
+  if ((i = get_config_index(address)) >= 0)
+       return m_configMemory->getConfigWord(i)->getVal();
+
+  return 0xffffffff;
+}
+
+int pic_processor::get_config_index(unsigned int address)
+{
+  
+  if (m_configMemory)
+  {
+
+      for(int i = 0; i < m_configMemory->getnConfigWords(); i++)
+      {
+	if (m_configMemory->getConfigWord(i))
+        {
+          if (m_configMemory->getConfigWord(i)->ConfigWordAdd() == address)
+          {
+            return i;
+          }
+	}
+     }
+  }
+ 
+  return -1;
 }
 
 
@@ -1597,14 +1676,31 @@ void WDT::callback_print()
 //------------------------------------------------------------------------
 // ConfigMemory - Base class
 ConfigWord::ConfigWord(const char *_name, unsigned int default_val, const char *desc,
-                       pic_processor *pCpu, unsigned int addr)
-  : Integer(_name, default_val, desc), m_pCpu(pCpu), m_addr(addr)
+                       pic_processor *pCpu, unsigned int addr, bool EEw)
+  : Integer(_name, default_val, desc), m_pCpu(pCpu), m_addr(addr),
+    EEWritable(EEw)
 {
   /*
   if (m_pCpu)
     m_pCpu->addSymbol(this);
   */
 }
+// this get controls the display format in the symbols window
+void ConfigWord::get(char *buffer, int buf_size)
+{
+    if(buffer)
+    {
+   	gint64 i;
+    	get(i);
+	long long int j = i;
+    	snprintf(buffer,buf_size,"0x%" PRINTF_INT64_MODIFIER "x",j);
+    }
+}
+void ConfigWord::get(gint64 &i)
+{
+    Integer::get(i);
+}
+
 
 //------------------------------------------------------------------------
 ConfigMemory::ConfigMemory(pic_processor *pCpu, unsigned int nWords)
@@ -1759,8 +1855,9 @@ class IO_SignalControl : public SignalControl
 public:
   IO_SignalControl(char _dir){ direction = _dir; }
   ~IO_SignalControl(){}
-  char getState() { return direction; }
-  void release() { }
+  virtual char getState() { return direction; }
+  virtual void release() {delete this;}
+  void setState(char _dir) { direction = _dir; }
 private:
   char direction;
 };
@@ -1776,6 +1873,9 @@ void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
                 PicTrisRegister *m_tris,
                 PicLatchRegister *m_lat)
 {
+  IO_SignalControl *clksource;
+  IO_SignalControl *clkcontrol;
+
   IOPIN *m_pin = package->get_pin(pkg_Pin_Number);
   if (name)
 	m_pin->newGUIname(name);
@@ -1793,8 +1893,10 @@ void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
 	    if (m_lat)
 		m_lat->setEnableMask(mask);
 	}
-    	PinMod->setSource(new IO_SignalControl('0'));
-    	PinMod->setControl(new IO_SignalControl(in ? '1' : '0'));
+	clksource = new IO_SignalControl('0');
+	clkcontrol = new IO_SignalControl(in ? '1' : '0');
+    	PinMod->setSource(clksource);
+    	PinMod->setControl(clkcontrol);
  	PinMod->updatePinModule();
   }
 }
@@ -1819,7 +1921,11 @@ void pic_processor::clr_clk_pin(unsigned int pkg_Pin_Number,
 	    if (m_lat)
 		m_lat->setEnableMask(mask);
 	}
+	if (PinMod->getActiveSource())
+	    delete (PinMod->getActiveSource());
     	PinMod->setSource(0);
+	if (PinMod->getActiveControl())
+	    delete (PinMod->getActiveControl());
     	PinMod->setControl(0);
  	PinMod->updatePinModule();
   }
@@ -1854,3 +1960,11 @@ void pic_processor::osc_mode(unsigned int value)
   }
   
 }
+void pic_processor::Wput(unsigned int value)
+{
+    Wreg->put(value);
+}
+unsigned int pic_processor::Wget()
+{
+    return Wreg->get();
+}
