@@ -1,5 +1,6 @@
 /*
    Copyright (C) 1998-2000 T. Scott Dattalo
+   Copyright (C) 2013      Roy R. Rankin
 
 This file is part of the libgpsim library of gpsim
 
@@ -45,6 +46,7 @@ enum PROCESSOR_TYPE
 {
   _PIC_PROCESSOR_,
   _14BIT_PROCESSOR_,
+  _14BIT_E_PROCESSOR_, // 14bit enhanced processor
   _12BIT_PROCESSOR_,
   _PIC17_PROCESSOR_,
   _PIC18_PROCESSOR_,
@@ -62,6 +64,7 @@ enum PROCESSOR_TYPE
   _P12F629_,
   _P12F675_,
   _P12F683_,
+  _P12F1822_,
   _P16C84_,
   _P16CR83_,
   _P16CR84_,
@@ -86,6 +89,7 @@ enum PROCESSOR_TYPE
   _P16F648_,
   _P16F676_,
   _P16F677_,
+  _P16F684_,
   _P16F685_,
   _P16F687_,
   _P16F689_,
@@ -119,6 +123,7 @@ enum PROCESSOR_TYPE
   _P16F884_,
   _P16F886_,
   _P16F887_,
+  _P16F1823_,
   _P17C7xx_,
   _P17C75x_,
   _P17C752_,
@@ -298,6 +303,7 @@ enum IOPIN_TYPES
  */
 class PicTrisRegister;
 class PicLatchRegister;
+class IO_SignalControl;
 
 class pic_processor : public Processor
 {
@@ -318,7 +324,7 @@ public:
   Stack        *stack;
 
   Status_register *status;
-  WREG         *W;
+  WREG	       *Wreg;		// Used when W is a normal register
   PCL          *pcl;
   PCLATH       *pclath;
   PCHelper     *m_PCHelper;
@@ -374,8 +380,10 @@ public:
   void BP_set_interrupt();
   void pm_write();
 
+  virtual ConfigMemory * getConfigMemory(){ return m_configMemory;}
   virtual bool set_config_word(unsigned int address, unsigned int cfg_word);
   virtual unsigned int get_config_word(unsigned int address);
+  virtual int get_config_index(unsigned int address);
   virtual unsigned int config_word_address() const {return 0x2007;};
   virtual ConfigMode *create_ConfigMode() { return new ConfigMode; };
   virtual void reset(RESET_TYPE r);
@@ -440,6 +448,11 @@ public:
 
   virtual void set_int_osc(bool val){ internal_osc = val;}
   virtual bool get_int_osc(){ return internal_osc; }
+  virtual void set_pplx4_osc(bool val){ PPLx4 = val;}
+  virtual bool get_pplx4_osc(){ return PPLx4; }
+
+  virtual void Wput(unsigned int);
+  virtual unsigned int Wget();
 
 
 
@@ -458,7 +471,8 @@ protected:
   string m_mclr_pin_name;
   unsigned char osc_pin_Number[2];
   PinModule *m_osc_Monitor[2];
-  bool internal_osc;
+  bool internal_osc;	// internal RC oscilator enabled on Config Word
+  bool PPLx4;		// 4x PPL enabled on Config Word
 
 };
 
@@ -487,11 +501,16 @@ class ConfigWord : public Integer
 {
 public:
   ConfigWord(const char *_name, unsigned int default_val, const char *desc,
-             pic_processor *pCpu, unsigned int addr);
+             pic_processor *pCpu, unsigned int addr, bool EEw=true);
+  virtual void get(char *buffer, int buf_size);
+  virtual void get(gint64 &i);
   unsigned int ConfigWordAdd() { return m_addr; }
+  bool isEEWritable() { return EEWritable;}
+
 protected:
   pic_processor *m_pCpu;
   unsigned int m_addr;
+  bool EEWritable;
 };
 
 class ConfigMemory
@@ -501,6 +520,8 @@ public:
   ~ConfigMemory();
   int addConfigWord(unsigned int addr, ConfigWord *);
   ConfigWord *getConfigWord(unsigned int addr);
+  int getnConfigWords() { return m_nConfigWords; }
+
 protected:
   pic_processor *m_pCpu;
   ConfigWord **m_ConfigWords;
