@@ -1,12 +1,20 @@
 #include <glib.h>	// for guint64
-#include <iostream>		// for cout used in breakpoints.h
-using namespace std;
-//#include "breakpoints.h"
+
 #include "trace.h"
 
 #include "intcon.h"
 #include "pie.h"
 #include "pir.h"
+#include "processor.h"
+
+PIE::PIE(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu,pName,pDesc), pir(0)
+{
+}
+void PIE::setPir(PIR *pPir)
+{
+  pir = pPir;
+}
 
 void PIE::put(unsigned int new_value)
 {
