@@ -1,6 +1,7 @@
 /*
    Copyright (C) 1998-2003 Scott Dattalo
                  2003 Mike Durian
+		 2013 Roy R. Rankin
 
 This file is part of the libgpsim library of gpsim
 
@@ -51,8 +52,10 @@ public:
   virtual void set_bclif(){}
   virtual void set_ccpif(){}
   virtual void set_cmif(){}
-  virtual void set_c1if(){}
-  virtual void set_c2if(){}
+  virtual void set_c1if(){fprintf(stderr, "RRR set_c1if FIX\n");}
+  virtual void set_c2if(){fprintf(stderr, "RRR set_c2if FIX\n");}
+  virtual void set_c3if(){fprintf(stderr, "RRR set_c3if FIX\n");}
+  virtual void set_c4if(){fprintf(stderr, "RRR set_c4if FIX\n");}
   virtual void set_eccp1if(){}
   virtual void set_eeif(){}
   virtual void set_errif(){}
@@ -64,6 +67,7 @@ public:
   virtual void set_rxb1if(){}
   virtual void set_sspif(){}
   virtual void set_tmr1if(){}
+  virtual void set_tmr1gif(){}
   virtual void set_tmr2if(){}
   virtual void set_tmr3if(){}
   virtual void set_txb0if(){}
@@ -71,11 +75,18 @@ public:
   virtual void set_txb2if(){}
   virtual void set_txif(){}
   virtual void set_wakif(){}
+  virtual void set_usbif(){}
 
   virtual unsigned int get_txif() { return 0;}
   virtual unsigned int get_rcif() { return 0;}
   virtual unsigned int get_sspif() { return 0;}
 
+  /// A generic method to set an interrupt bit by mask
+  virtual void set(int mask)
+  {
+    put(get() | mask);
+  }
+
   /// Obtain interrupt request state, as a mask of priorities if relevant
   virtual int interrupt_status();
 
@@ -146,6 +157,9 @@ public:
   virtual void set_rcif();
 
   virtual void set_cmif();
+  virtual void set_c1if(){set_cmif();}
+  virtual void set_c2if(){set_cmif();}
+ 
 
   virtual void set_eeif();
 
@@ -253,18 +267,26 @@ public:
 
   enum {
     TMR1IF  = 1<<0,
+    TMR2IF  = 1<<1,	//16f684
+    OSFIF   = 1<<2,	//16f684
     CMIF    = 1<<3,
+    C1IF    = 1<<3,	//16f684
+    C2IF    = 1<<4,	//16f684
+    CCP1IF  = 1<<5,	//16f684
     ADIF    = 1<<6,
     EEIF    = 1<<7
   };
  
-  virtual void set_tmr1if() { put(get() | TMR1IF); }
-
+  virtual void set_tmr1if();
+  virtual void set_tmr2if();
   virtual void set_cmif();
 
   virtual void set_adif();
 
   virtual void set_eeif();
+  virtual void set_c1if();
+  virtual void set_c2if();
+  virtual void set_ccpif() { put(get() | CCP1IF); }
 
  
   PIR1v3(Processor *pCpu, const char *pName, const char *pDesc,
@@ -340,6 +362,8 @@ enum
   virtual void set_bclif();
   virtual void set_eeif();
   virtual void set_cmif();
+  virtual void set_c1if(){ set_cmif();}
+  virtual void set_c2if(){ set_cmif();}
 
   PIR2v2(Processor *pCpu, const char *pName, const char *pDesc,
          INTCON *, PIE *);
@@ -380,6 +404,98 @@ enum
          INTCON *, PIE *);
 };
 
+//---------------------------------------------------------
+// PIR2 Peripheral Interrupt register # 3
+//
+// This is version 4 of the PIR2 register - as seen on the 18F4455 devices
+
+class PIR2v4 : public PIR
+{
+public:
+
+enum
+{
+    CCP2IF  = 1<<0,
+    TMR3IF  = 1<<1,
+    HLVDIF  = 1<<2,
+    BCLIF   = 1<<3,
+    EEIF    = 1<<4,
+    USBIF   = 1<<5,
+    CMIF    = 1<<6,
+    OSCFIF  = 1<<7
+};
+
+
+  virtual void set_ccpif(){
+      put(get() | CCP2IF);
+    }
+  virtual void set_tmr3if() {
+      put(get() | TMR3IF);
+    }
+  virtual void set_hlvdif() {
+      put(get() | HLVDIF);
+    }
+  virtual void set_oscfif() {
+      put(get() | OSCFIF);
+    }
+
+  virtual void set_bclif();
+  virtual void set_eeif();
+  virtual void set_usbif();
+  virtual void set_cmif();
+  virtual void set_c1if(){ set_cmif(); }
+  virtual void set_c2if(){ set_cmif(); }
+
+  PIR2v4(Processor *pCpu, const char *pName, const char *pDesc,
+         INTCON *, PIE *);
+};
+
+
+//---------------------------------------------------------
+// PIR2 Peripheral Interrupt register # 3
+//
+// This is version 1 of the PIR3 register - as seen on the 18F6520 devices
+
+class PIR3v1 : public PIR
+{
+public:
+
+enum
+{
+    CCP3IF  = 1<<0,
+    CCP4IF  = 1<<1,
+    CCP5IF  = 1<<2,
+    TMR4IF  = 1<<3,
+    TXIF   = 1<<4,
+    RCIF   = 1<<5
+};
+
+  virtual void set_ccpif()    /* RP - needs to define set_ccpif too! */
+    {
+      put(get() | CCP3IF);
+    }
+
+  virtual void set_tmr2if()
+    {
+      put(get() | TMR4IF);
+    }
+  virtual void set_txif();
+  virtual void set_rcif();
+
+  virtual unsigned int get_txif()
+  {
+    return value.get() & TXIF;
+  }
+  virtual void clear_txif();
+  unsigned int get_rcif()
+  {
+    return value.get() & RCIF;
+  }
+  virtual void clear_rcif();
+ 
+  PIR3v1(Processor *pCpu, const char *pName, const char *pDesc,
+         INTCON *, PIE *);
+};
 
 //---------------------------------------------------------
 // PIR2 Peripheral Interrupt register # 3
@@ -499,6 +615,8 @@ public:
   virtual void set_cmif() {}
   virtual void set_c1if() {}
   virtual void set_c2if() {}
+  virtual void set_c3if() {}
+  virtual void set_c4if() {}
 
   // eeprom stuff
   virtual void set_eeif() {}
@@ -508,6 +626,7 @@ public:
 
   // Timer stuff
   virtual void set_tmr1if() {}
+  virtual void set_tmr1gif() {}
   virtual void set_tmr2if() {}
   virtual void set_adif() {}
 };
@@ -610,6 +729,24 @@ class PIR_SET_1 : public PIR_SET
     pir1->set_cmif();
   }
 
+  virtual void set_c1if() {
+    assert(pir1 != 0);
+    pir1->set_c1if();
+  }
+  virtual void set_c2if() {
+    assert(pir1 != 0);
+    pir1->set_c2if();
+  }
+
+  virtual void set_c3if() {
+    assert(pir1 != 0);
+    pir1->set_c3if();
+  }
+  virtual void set_c4if() {
+    assert(pir1 != 0);
+    pir1->set_c4if();
+  }
+
 private:
   PIR   *pir1;
   PIR   *pir2;
@@ -699,6 +836,10 @@ class PIR_SET_2 : public PIR_SET
     assert(pir1 != 0);
     pir1->set_tmr1if();
   }
+  virtual void set_tmr1gif() {
+    assert(pir1 != 0);
+    pir1->set_tmr1gif();
+  }
   virtual void set_tmr2if() {
     assert(pir1 != 0);
     pir1->set_tmr2if();
@@ -724,6 +865,16 @@ class PIR_SET_2 : public PIR_SET
     pir2->set_c2if();
   }
 
+  virtual void set_c3if() {
+    assert(pir2 != 0);
+    pir2->set_c3if();
+  }
+
+  virtual void set_c4if() {
+    assert(pir2 != 0);
+    pir2->set_c4if();
+  }
+
   // I2C master
   virtual void set_bclif() {
     assert(pir2 != 0);
