@@ -57,6 +57,13 @@ License along with this library; if not, see
 
 #include <typeinfo>
 
+//#define DEBUG
+#if defined(DEBUG)
+#define Dprintf(arg) {printf("%s:%d-%s() ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
+#else
+#define Dprintf(arg) {}
+#endif
+
 //------------------------------------------------------------------------
 // active_cpu  is a pointer to the pic processor that is currently 'active'.
 // 'active' means that it's the one currently being simulated or the one
@@ -109,7 +116,7 @@ Processor::Processor(const char *_name, const char *_desc)
     rma(this),
     ema(this),
     pc(0),
-    bad_instruction(0, 0, 0)
+    bad_instruction(0, 0x3fff, 0)
 {
   registers = 0;
 
@@ -334,6 +341,7 @@ void Processor::create_invalid_registers ()
       char nameBuff[100];
       snprintf(nameBuff,sizeof(nameBuff), "INVREG_%X",addr);
 
+
       registers[index] = new InvalidRegister(this, nameBuff);
       registers[index]->setAddress(addr);
     }
@@ -350,12 +358,25 @@ void Processor::delete_invalid_registers ()
   unsigned int i=0;
 
   for (i = 0; i < rma.get_size(); i++) {
-    //cout << __FUNCTION__ << "  reg: 0x"<<hex << i << " ptr:" << registers[i] << endl;
+   // cout << __FUNCTION__ << "  reg: 0x"<<hex << i << " ptr:" << registers[i] << endl;
+	
     InvalidRegister *pReg = dynamic_cast<InvalidRegister *> (registers[i]);
     if (pReg) {
       delete registers[i];
       registers[i]= 0;
+      
     }
+    else if (registers[i])
+   {
+      char reg_name[11];
+      cout << __FUNCTION__ << "  reg: 0x"<<hex << i << " ptr:" << 
+	registers[i] ;
+      cout.flush();
+      strncpy(reg_name, registers[i]->name().c_str(), 10);
+      reg_name[10] = 0;
+      cout << " " << reg_name <<endl;
+   }
+
   }
 }
 
@@ -372,10 +393,23 @@ void Processor::add_file_registers(unsigned int start_address, unsigned int end_
   unsigned int j;
 
   // Initialize the General Purpose Registers:
+  Dprintf((" from 0x%x to 0x%x alias 0x%x\n", start_address, end_address, alias_offset));
 
   char str[100];
   for  (j = start_address; j <= end_address; j++) {
 
+#ifdef DEBUG
+    if (j == 0x11)
+    {
+	printf("Processor::add_file_registers j 0x%x\n", j);
+    }
+#endif
+    if (registers[j] && (registers[j]->isa() == Register::INVALID_REGISTER))
+	delete registers[j];
+    else if (registers[j])
+	cout << __FUNCTION__ << " Already register " << registers[j]->name() 
+		<< " at 0x" << hex << j <<endl; 
+
     //The default register name is simply its address
     snprintf (str, sizeof(str), "REG%03X", j);
     registers[j] = new Register(this, str);
@@ -420,11 +454,15 @@ void Processor::delete_file_registers(unsigned int start_address,
 #define ALIAS_MASK (SMALLEST_ALIAS_DISTANCE-1)
   unsigned int i,j;
 
+  if (start_address != end_address)
+      Dprintf(("from 0x%x to 0x%x\n", start_address, end_address));
 
   for (j = start_address; j <= end_address; j++) {
     if(registers[j]) {
 
       Register *thisReg = registers[j];
+      Register *replaced = thisReg->getReplaced();
+
       if(thisReg->alias_mask) {
         // This register appears in more than one place. Let's find all
         // of its aliases.
@@ -439,8 +477,13 @@ void Processor::delete_file_registers(unsigned int start_address,
         cout << " deleting: " << hex << j << endl;
       registers[j] = 0;
       if (!bRemoveWithoutDelete)
+      {
+	if (replaced) delete replaced;
         delete thisReg;
+      }
     }
+    else
+	printf("%s register 0x%x already deleted\n", __FUNCTION__, j);
   }
 }
 
@@ -460,11 +503,28 @@ void Processor::alias_file_registers(unsigned int start_address, unsigned int en
 
   // FIXME -- it'd probably make better sense to keep a list of register addresses at
   // which a particular register appears.
+#ifdef DEBUG
+  if (start_address == 0x20)
+	printf("DEBUG trace %x\n", start_address);
+  if (start_address != end_address)
+	Dprintf((" from 0x%x to 0x%x alias_offset 0x%x\n", start_address, end_address, alias_offset));
+#endif
 
   for (j = start_address; j <= end_address; j++)
     {
       if (alias_offset && (j+alias_offset < rma.get_size()))
         {
+	  if (registers[j + alias_offset])
+	  {
+		if (registers[j + alias_offset] == registers[j])
+		{
+		    printf("alias_file_register Duplicate alias %s from 0x%x to 0x%x \n",registers[j + alias_offset]->name().c_str(), j, j+alias_offset);
+		}
+		else
+		{
+		    delete registers[j + alias_offset];
+		}
+	   }
           registers[j + alias_offset] = registers[j];
           if (registers[j])
             registers[j]->alias_mask = alias_offset;
@@ -569,6 +629,39 @@ void Processor::init_program_memory(unsigned int address, unsigned int value)
     set_out_of_range_pm(address,value);  // could be e2prom
 
 
+}
+//-------------------------------------------------------------------
+//erase_program_memory(unsigned int address)
+//
+//	Checks if a program memory location contains an instruction
+//	and deletes it if it does. 
+//
+void Processor::erase_program_memory(unsigned int address)
+{
+  unsigned int uIndex = map_pm_address2index(address);
+
+  if (!program_memory) {
+    std::stringstream buf;
+    buf << "ERROR: internal bug " << __FILE__ << ":" << __LINE__;
+    throw new FatalError(buf.str());
+  }
+
+  if(uIndex < program_memory_size()) {
+
+    if(program_memory[uIndex] != 0 && program_memory[uIndex]->isa() != instruction::INVALID_INSTRUCTION) {
+      delete program_memory[uIndex];
+      program_memory[uIndex] = &bad_instruction;
+    }
+  }
+  else
+  {
+    cout << "Erase Program memory\n";
+    cout << "Warning::Out of range address " << hex << address << endl;
+    cout << "Max allowed address is 0x" << hex << (program_address_limit()-1) << '\n';
+  
+  }
+
+
 }
 
 void Processor::init_program_memory_at_index(unsigned int uIndex, unsigned int value)
@@ -2155,7 +2248,11 @@ Register &RegisterMemoryAccess::operator [] (unsigned int address)
 void RegisterMemoryAccess::reset (RESET_TYPE r)
 {
   for(unsigned int i=0; i<nRegisters; i++)
-    operator[](i).reset(r);
+  {
+    // Do not reset aliased registers
+    if ( !(operator[](i).alias_mask && (operator[](i).alias_mask & i)))
+        operator[](i).reset(r);
+  }
 }
 
 //========================================================================
@@ -2216,8 +2313,10 @@ ProcessorConstructorList * ProcessorConstructor::GetList()
 ProcessorConstructor *ProcessorConstructorList::findByType(const char *name)
 {
   ProcessorConstructorList::iterator processor_iterator;
-  for (processor_iterator = processor_list->begin();
-       processor_iterator != processor_list->end();
+  ProcessorConstructorList *pl = ProcessorConstructor::GetList();
+
+  for (processor_iterator = pl->begin();
+       processor_iterator != pl->end();
        ++processor_iterator) {
 
     ProcessorConstructor *p = *processor_iterator;
@@ -2241,16 +2340,16 @@ string ProcessorConstructorList::DisplayString(void)
 
   int i,j,k,longest;
 
+  ProcessorConstructorList *pl = ProcessorConstructor::GetList();
   ProcessorConstructor *p;
 
-
   // loop through all of the processors and find the
   // one with the longest name
 
   longest = 0;
 
-  for (processor_iterator = processor_list->begin();
-       processor_iterator != processor_list->end();
+  for (processor_iterator = pl->begin();
+       processor_iterator != pl->end();
        processor_iterator++) {
 
     p = *processor_iterator;
@@ -2264,10 +2363,10 @@ string ProcessorConstructorList::DisplayString(void)
 
   // Print the name of each processor.
 
-  for (processor_iterator = processor_list->begin();
-       processor_iterator != processor_list->end(); ) {
+  for (processor_iterator = pl->begin();
+       processor_iterator != pl->end(); ) {
 
-    for(i=0; i<nPerRow && processor_iterator != processor_list->end(); i++) {
+    for(i=0; i<nPerRow && processor_iterator != pl->end(); i++) {
 
       p = *processor_iterator++;
       stream << p->names[1];
