@@ -227,8 +227,8 @@ char * RegisterValue::toBitStr(char *s, int len, unsigned int BitPos,
 //
 Register::Register(Module *_cpu, const char *pName, const char *pDesc)
   : Value(pName,pDesc,_cpu),
-    address(0xffffffff),
-    alias_mask(0)
+    address(AN_INVALID_ADDRESS),
+    alias_mask(0), m_replaced(0)
 {
 
   // For now, initialize the register with valid data and set that data equal to 0.
@@ -240,7 +240,7 @@ Register::Register(Module *_cpu, const char *pName, const char *pDesc)
   xref().assign_data(this);
   read_access_count=0;
   write_access_count=0;
-  bit_mask = 7;
+  mValidBits = 0xFF;
 
 }
 Register::~Register()
@@ -319,9 +319,11 @@ double Register::get_bit_voltage(unsigned int bit_number)
 // classes that overide this function
 void Register::setbit(unsigned int bit_number, bool new_value)
 {
-  if(bit_number <= bit_mask) {
+    int set_mask = (1<<bit_number);
+
+  if ( set_mask & mValidBits ) {
     trace.raw(write_trace.get() | value.get());
-    value.put((value.get() & ~(1<<bit_number)) | (new_value ? (1<<bit_number) : 0));
+    value.put((value.get() & ~set_mask) | (new_value ? set_mask : 0));
   }
 }
 
@@ -357,7 +359,7 @@ void Register::put_value(unsigned int new_value)
   // all objects derived from a file_register should
   // automagically be correctly updated.
 
-  put(new_value);
+  value.put(new_value);
 
   // Even though we just wrote a value to this register,
   // it's possible that the register did not get fully
@@ -468,7 +470,8 @@ void Register::set(Value * pVal)
 
 Value *Register::copy()
 {
-  return new ValueWrapper(this);
+  Value *val = new ValueWrapper(this);
+  return val;
 }
 void Register::get(gint64 &i)
 {
@@ -505,7 +508,6 @@ void sfr_register::reset(RESET_TYPE r)
 //--------------------------------------------------
 // member functions for the InvalidRegister class
 //--------------------------------------------------
-#define AN_INVALID_ADDRESS 0xffffffff
 void InvalidRegister::put(unsigned int new_value)
 {
   cout << "attempt write to invalid file register\n";
