@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 #ifndef __STIMULI_H__
@@ -461,6 +461,10 @@ public:
                     double _ZthIn = 1e8);
   virtual double get_Zth();
   virtual double get_Vth();
+  virtual double get_VthIn() { return VthIn;}
+  virtual double get_ZthIn() { return ZthIn;}
+  virtual void set_VthIn(double _VthIn) { VthIn = _VthIn;}
+  virtual void set_ZthIn(double _ZthIn) { ZthIn = _ZthIn;}
   virtual char getBitChar();
 
   virtual void set_nodeVoltage(double new_nodeVoltage);
