@@ -30,6 +30,16 @@ License along with this library; if not, see
 #include "trace_orb.h"
 #include "xref.h"
 
+#define NEWTRACE 
+
+//#define DEBUG
+#if defined(DEBUG)
+#define Dprintf(arg) {printf("%s:%d-%s() ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
+#else
+#define Dprintf(arg) {}
+#endif
+
+
 #define MODE "0x" << hex
 
 Trace trace;               /* Instantiate the trace buffer class.
@@ -322,6 +332,8 @@ void Trace::addToCurrentFrame(TraceObject *to)
 
   if(current_frame)
     current_frame->add(to);
+  else
+    delete to;
 
 }
 void Trace::deleteTraceFrame()
@@ -329,12 +341,14 @@ void Trace::deleteTraceFrame()
   if (!current_frame)
     return;
   list <TraceFrame *> :: iterator tfIter;
+  list <TraceObject *> :: reverse_iterator toIter;
 
   for(tfIter = traceFrames.begin();
       tfIter != traceFrames.end();
-      ++tfIter) {
-    TraceFrame *tf = *tfIter;
-    delete tf;
+      ++tfIter) 
+  {
+      TraceFrame *tf = *tfIter;
+      delete tf;
   }
   traceFrames.clear();
   current_frame = 0;
@@ -514,6 +528,8 @@ const char * resetName(RESET_TYPE r)
   case BOD_RESET:  return "BOD_RESET";
   case SIM_RESET:  return "SIM_RESET";
   case EXIT_RESET: return "EXIT_RESET";
+  case STKUNF_RESET: return "STKUNF_RESET";
+  case STKOVF_RESET: return "STKOVF_RESET";
   case OTHER_RESET: return "OTHER_RESET";
   }
   return "unknown reset";
@@ -584,7 +600,7 @@ int TraceType::dump_raw(Trace *pTrace,unsigned int tbi, char *buf, int bufsize)
 
   for(int i = 0; i < iUsed; i++) {
 
-    int n = snprintf(buf,bufsize,"%08X:", pTrace->get(tbi+i));
+    int n = snprintf(buf,bufsize," %08X:", pTrace->get(tbi+i));
     if(n < 0)
       break;
 
@@ -709,6 +725,8 @@ TraceObject *RegisterWriteTraceType::decode(unsigned int tbi)
 
 int RegisterWriteTraceType::dump_raw(Trace *pTrace,unsigned int tbi, char *buf, int bufsize)
 {
+  unsigned int val = 0;
+
   if (!pTrace)
     return 0;
 
@@ -721,10 +739,15 @@ int RegisterWriteTraceType::dump_raw(Trace *pTrace,unsigned int tbi, char *buf,
   unsigned int address = (tv >> 8) & 0xfff;
 
   Register *reg = cpu->rma.get_register(address);
+  if(reg)
+    val = reg->get_value();
+
   int m = snprintf(buf, bufsize,
-                   "  Reg Write: %s(0x%04X) was 0x%x ",
+                   "  Reg Write: 0x%0x to %s(0x%04X) was 0x%0X ",
+		   val&0xff,
                    (reg ? reg->name().c_str() : ""), address,
                    tv & 0xff);
+  Dprintf(("dump_raw %s %x\n", buf, tv&0xff));
   if(m>0)
     n += m;
 
@@ -802,6 +825,7 @@ TraceObject *PCTraceType::decode(unsigned int tbi)
 
   trace.current_frame->cycle_time = trace.current_cycle_time;
 
+
   return pcto;
 }
 
@@ -1026,6 +1050,7 @@ int Trace::is_cycle_trace(unsigned int index, guint64 *cvt_cycle)
           if( (get(i) & (CYCLE_COUNTER_HI | CYCLE_COUNTER_LO)) &&
               (((get(k) - get(i)) & 0x7fffffff) == 1) )
             return 1;
+
         }
 
       // The current index points to the low int and the next entry is
@@ -1058,6 +1083,7 @@ int Trace::dump1(unsigned index, char *buffer, int bufsize)
   guint64 cycle;
   int return_value = is_cycle_trace(index,&cycle);
 
+
   if(bufsize)
     buffer[0] = 0;   // 0 terminate just in case no string is created
 
@@ -1114,7 +1140,7 @@ int Trace::dump1(unsigned index, char *buffer, int bufsize)
 
           if(tt) {
             tt->dump_raw(this,index,buffer,bufsize);
-            return_value = tt->size();
+	    return_value = tt->entriesUsed(this, index);
           }
           break;
         }
@@ -1219,7 +1245,9 @@ int Trace::dump(int n, FILE *out_stream)
 
         TraceObject *pTO = tt->decode(k);
         if (pTO)
+	{
           addToCurrentFrame(pTO);
+  	}
       }
 
       if(is_cycle_trace(k,&cycle) == 2)
@@ -1239,6 +1267,7 @@ int Trace::dump(int n, FILE *out_stream)
 
   deleteTraceFrame();
 
+  fflush(out_stream);
   return n;
 }
 //------------------------------------------------------------------------
@@ -1332,6 +1361,9 @@ void Trace::dump_raw(int n)
 
 void Trace::dump_last_instruction()
 {
+
+  if (trace_log.log_file)
+	dump(1, trace_log.log_file);
   dump(1,stdout);
 }
 
@@ -1430,6 +1462,7 @@ void TraceLog::open_logfile(const char *new_fname, int format)
       log_file = fopen(new_fname, "w");
       lxtp=0;
       break;
+
   case TRACE_FILE_FORMAT_LXT:
       lxtp = lt_init(new_fname);
       lt_set_timescale(lxtp, -8);
@@ -1450,9 +1483,12 @@ void TraceLog::close_logfile()
       switch(file_format)
       {
       case TRACE_FILE_FORMAT_ASCII:
+#ifndef NEWTRACE
           write_logfile();
+#endif
           fclose(log_file);
           break;
+
       case TRACE_FILE_FORMAT_LXT:
           lt_close(lxtp);
           break;
@@ -1466,6 +1502,7 @@ void TraceLog::close_logfile()
 
 void TraceLog::write_logfile()
 {
+#ifndef NEWTRACE
 
   unsigned int i,j;
   char buf[256];
@@ -1483,7 +1520,9 @@ void TraceLog::write_logfile()
       buf[0] = 0;
 
       if(buffer.is_cycle_trace(i,&cycle))
-        fprintf(log_file,"Cycle 0x%016" PRINTF_GINT64_MODIFIER "X\n",cycle);
+      {
+        fprintf(log_file,"Cycle 0x%016" PRINTF_GINT64_MODIFIER "X ",cycle);
+      }
 
       i = (i + buffer.dump1(i,buf, sizeof(buf))) & TRACE_BUFFER_MASK;
 
@@ -1491,11 +1530,14 @@ void TraceLog::write_logfile()
         items_logged++;
         fprintf(log_file,"%s\n", buf);
       }
+      else
+	fprintf(log_file, "\n");
     }
 
     buffer.trace_index = 0;
   }
 
+#endif //NEWTRACE
 }
 
 void TraceLog::enable_logging(const char *new_fname, int format)
@@ -1552,7 +1594,8 @@ void TraceLog::status()
     // item that is in the log buffer, so the actual events that
     // triggered a log is the total buffer size divided by 2.
 
-    int total_items = (buffer.trace_index + items_logged)/2;
+    //int total_items = (buffer.trace_index + items_logged)/2;
+    int total_items = (buffer.trace_index + items_logged);
     if(total_items) {
       cout << "So far, it contains " << hex << "0x" << total_items << " logged events\n";
     } else {
@@ -1593,6 +1636,7 @@ void TraceLog::lxt_trace(unsigned int address, unsigned int value, guint64 cc)
     char *name;
 
     name = (char *)cpu->registers[address]->name().c_str();
+    items_logged++;
 
     lt_set_time(lxtp, (int)(get_cycles().get()*4.0e8*cpu->get_OSCperiod()));
 
@@ -1619,12 +1663,20 @@ void TraceLog::register_read(Register *pReg, guint64 cc)
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
+      Dprintf(("cycle=%lx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+#ifdef NEWTRACE
+      bp.set_logging();
+      return;
+#else
       buffer.cycle_counter(cc);
       buffer.raw(pReg->read_trace.get() | pReg->get_value());
       if(buffer.near_full())
         write_logfile();
+#endif //NEWTRACE
       break;
+
     case TRACE_FILE_FORMAT_LXT:
+      Dprintf(("LXT cycle=%lx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
@@ -1632,18 +1684,27 @@ void TraceLog::register_read(Register *pReg, guint64 cc)
 
 void TraceLog::register_write(Register *pReg, guint64 cc)
 {
+
   if (!pReg)
     return;
 
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
+      Dprintf(("cycle=%lx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+#ifdef NEWTRACE
+      bp.set_logging();
+      return;
+#else
       buffer.cycle_counter(cc);
       buffer.raw(pReg->write_trace.get() | pReg->get_value());
       if(buffer.near_full())
         write_logfile();
+#endif //NEWTRACE
       break;
+
     case TRACE_FILE_FORMAT_LXT:
+      Dprintf(("LXT cycle=%lx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
@@ -1657,12 +1718,20 @@ void TraceLog::register_read_value(Register *pReg, guint64 cc)
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
+      Dprintf(("cycle=%lx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+#ifdef NEWTRACE
+      bp.set_logging();
+      return;
+#else
       buffer.cycle_counter(cc);
       buffer.raw(pReg->read_trace.get() | pReg->get_value());
       if(buffer.near_full())
         write_logfile();
+#endif //NEWTRACE
       break;
+
     case TRACE_FILE_FORMAT_LXT:
+      Dprintf(("LXT cycle=%lx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
@@ -1670,18 +1739,27 @@ void TraceLog::register_read_value(Register *pReg, guint64 cc)
 
 void TraceLog::register_write_value(Register *pReg, guint64 cc)
 {
+
   if (!pReg)
     return;
 
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
+      Dprintf(("cycle=%lx %s(0x%x) value %x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+#ifdef NEWTRACE
+      bp.set_logging();
+      return;
+#else
       buffer.cycle_counter(cc);
       buffer.raw(pReg->write_trace.get() | pReg->get_value());
       if(buffer.near_full())
         write_logfile();
+#endif //NEWTRACE
       break;
+
     case TRACE_FILE_FORMAT_LXT:
+      Dprintf(("LXT cycle=%lx %s(0x%x) value %x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
@@ -1867,10 +1945,9 @@ inline bool BoolEventBuffer::event(bool state)
 
 unsigned int BoolEventBuffer::get_index(guint64 event_time)
 {
-  guint32 start_index, end_index, search_index, bstep;
+  guint32 start_index, search_index, bstep;
   guint64 time_offset;
 
-  end_index = index;
   start_index = 0;
 
   bstep = (max_events+1) >> 1;
